@@ -17,8 +17,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 967 $
- * $Date: 2014-03-20 17:14:39 -0400 (Thu, 20 Mar 2014) $
+ * $Revision$
+ * $Date$
  *
  */
 
@@ -64,7 +64,6 @@ private:
 	bool				m_bForce;
 	bool				m_bRedirecting;
 	bool				m_bRedirected;
-	int					m_iRedirects;
 	bool				m_bGZip;
 	bool				m_bRetry;
 #ifndef DISABLE_GZIP
@@ -93,6 +92,7 @@ public:
 	virtual void		Run();
 	virtual void		Stop();
 	EStatus				Download();
+	EStatus				DownloadWithRedirects(int iMaxRedirects);
 	bool				Terminate();
 	void				SetInfoName(const char* v);
 	const char*			GetInfoName() { return m_szInfoName; }
