@@ -1,7 +1,7 @@
 /*
  *  This file is part of nzbget
  *
- *  Copyright (C) 2013-2014 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2013-2015 Andrey Prygunkov <hugbug@users.sourceforge.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,8 +17,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 1044 $
- * $Date: 2014-06-17 17:31:09 -0400 (Tue, 17 Jun 2014) $
+ * $Revision$
+ * $Date$
  *
  */
 
@@ -47,10 +47,9 @@
 #include "Util.h"
 #include "FeedFile.h"
 #include "FeedFilter.h"
+#include "FeedScript.h"
 #include "DiskState.h"
-
-extern Options* g_pOptions;
-extern DiskState* g_pDiskState;
+#include "DupeCoordinator.h"
 
 FeedCoordinator::FeedCacheItem::FeedCacheItem(const char* szUrl, int iCacheTimeSec,const char* szCacheId,
 	time_t tLastUsage, FeedItemInfos* pFeedItemInfos)
@@ -70,6 +69,42 @@ FeedCoordinator::FeedCacheItem::~FeedCacheItem()
 	m_pFeedItemInfos->Release();
 }
 
+FeedCoordinator::FilterHelper::FilterHelper()
+{
+	m_pSeasonEpisodeRegEx = NULL;
+}
+
+FeedCoordinator::FilterHelper::~FilterHelper()
+{
+	delete m_pSeasonEpisodeRegEx;
+}
+
+void FeedCoordinator::FilterHelper::CalcDupeStatus(const char* szTitle, const char* szDupeKey, char* szStatusBuf, int iBufLen)
+{
+	const char* szDupeStatusName[] = { "", "QUEUED", "DOWNLOADING", "3", "SUCCESS", "5", "6", "7", "WARNING",
+		"9", "10", "11", "12", "13", "14", "15", "FAILURE" };
+	char szStatuses[200];
+	szStatuses[0] = '\0';
+
+	DownloadQueue* pDownloadQueue = DownloadQueue::Lock();
+	DupeCoordinator::EDupeStatus eDupeStatus = g_pDupeCoordinator->GetDupeStatus(pDownloadQueue, szTitle, szDupeKey);
+	DownloadQueue::Unlock();
+
+	for (int i = 1; i <= (int)DupeCoordinator::dsFailure; i = i << 1)
+	{
+		if (eDupeStatus & i)
+		{
+			if (*szStatuses)
+			{
+				strcat(szStatuses, ",");
+			}
+			strcat(szStatuses, szDupeStatusName[i]);
+		}
+	}
+
+	strncpy(szStatusBuf, szStatuses, iBufLen);
+}
+
 FeedCoordinator::FeedCoordinator()
 {
 	debug("Creating FeedCoordinator");
@@ -361,6 +396,9 @@ void FeedCoordinator::FeedCompleted(FeedDownloader* pFeedDownloader)
 	{
 		if (!pFeedInfo->GetPreview())
 		{
+			FeedScriptController::ExecuteScripts(
+				!Util::EmptyStr(pFeedInfo->GetFeedScript()) ? pFeedInfo->GetFeedScript(): g_pOptions->GetFeedScript(),
+				pFeedInfo->GetOutputFilename(), pFeedInfo->GetID());
 			FeedFile* pFeedFile = FeedFile::Create(pFeedInfo->GetOutputFilename());
 			remove(pFeedInfo->GetOutputFilename());
 
@@ -414,6 +452,7 @@ void FeedCoordinator::FilterFeed(FeedInfo* pFeedInfo, FeedItemInfos* pFeedItemIn
 		pFeedItemInfo->SetAddCategory(pFeedInfo->GetCategory());
 		pFeedItemInfo->SetDupeScore(0);
 		pFeedItemInfo->SetDupeMode(dmScore);
+		pFeedItemInfo->SetFeedFilterHelper(&m_FilterHelper);
 		pFeedItemInfo->BuildDupeKey(NULL, NULL);
 		if (pFeedFilter)
 		{
@@ -440,7 +479,7 @@ void FeedCoordinator::ProcessFeed(FeedInfo* pFeedInfo, FeedItemInfos* pFeedItemI
 		{
 			FeedHistoryInfo* pFeedHistoryInfo = m_FeedHistory.Find(pFeedItemInfo->GetUrl());
 			FeedHistoryInfo::EStatus eStatus = FeedHistoryInfo::hsUnknown;
-			if (bFirstFetch)
+			if (bFirstFetch && pFeedInfo->GetBacklog())
 			{
 				eStatus = FeedHistoryInfo::hsBacklog;
 			}
@@ -479,6 +518,7 @@ NZBInfo* FeedCoordinator::CreateNZBInfo(FeedInfo* pFeedInfo, FeedItemInfo* pFeed
 
 	NZBInfo* pNZBInfo = new NZBInfo();
 	pNZBInfo->SetKind(NZBInfo::nkUrl);
+	pNZBInfo->SetFeedID(pFeedInfo->GetID());
 	pNZBInfo->SetURL(pFeedItemInfo->GetUrl());
 
 	// add .nzb-extension if not present
@@ -517,18 +557,19 @@ bool FeedCoordinator::ViewFeed(int iID, FeedItemInfos** ppFeedItemInfos)
 
 	FeedInfo* pFeedInfo = m_Feeds.at(iID - 1);
 
-	return PreviewFeed(pFeedInfo->GetName(), pFeedInfo->GetUrl(), pFeedInfo->GetFilter(), 
-		pFeedInfo->GetPauseNzb(), pFeedInfo->GetCategory(), pFeedInfo->GetPriority(),
-		0, NULL, ppFeedItemInfos);
+	return PreviewFeed(pFeedInfo->GetID(), pFeedInfo->GetName(), pFeedInfo->GetUrl(), pFeedInfo->GetFilter(), 
+		pFeedInfo->GetBacklog(), pFeedInfo->GetPauseNzb(), pFeedInfo->GetCategory(),
+		pFeedInfo->GetPriority(), pFeedInfo->GetInterval(), pFeedInfo->GetFeedScript(), 0, NULL, ppFeedItemInfos);
 }
 
-bool FeedCoordinator::PreviewFeed(const char* szName, const char* szUrl, const char* szFilter,
-	bool bPauseNzb, const char* szCategory, int iPriority,
+bool FeedCoordinator::PreviewFeed(int iID, const char* szName, const char* szUrl, const char* szFilter,
+	bool bBacklog, bool bPauseNzb, const char* szCategory, int iPriority, int iInterval, const char* szFeedScript,
 	int iCacheTimeSec, const char* szCacheId, FeedItemInfos** ppFeedItemInfos)
 {
 	debug("Preview feed %s", szName);
 
-	FeedInfo* pFeedInfo = new FeedInfo(0, szName, szUrl, 0, szFilter, bPauseNzb, szCategory, iPriority);
+	FeedInfo* pFeedInfo = new FeedInfo(iID, szName, szUrl, bBacklog, iInterval,
+		szFilter, bPauseNzb, szCategory, iPriority, szFeedScript);
 	pFeedInfo->SetPreview(true);
 	
 	FeedItemInfos* pFeedItemInfos = NULL;
@@ -583,6 +624,9 @@ bool FeedCoordinator::PreviewFeed(const char* szName, const char* szUrl, const c
 
 		if (pFeedInfo->GetStatus() == FeedInfo::fsFinished)
 		{
+			FeedScriptController::ExecuteScripts(
+				!Util::EmptyStr(pFeedInfo->GetFeedScript()) ? pFeedInfo->GetFeedScript(): g_pOptions->GetFeedScript(),
+				pFeedInfo->GetOutputFilename(), pFeedInfo->GetID());
 			pFeedFile = FeedFile::Create(pFeedInfo->GetOutputFilename());
 		}
 
@@ -601,7 +645,7 @@ bool FeedCoordinator::PreviewFeed(const char* szName, const char* szUrl, const c
 		for (FeedItemInfos::iterator it = pFeedItemInfos->begin(); it != pFeedItemInfos->end(); it++)
 		{
 			FeedItemInfo* pFeedItemInfo = *it;
-			pFeedItemInfo->SetStatus(bFirstFetch ? FeedItemInfo::isBacklog : FeedItemInfo::isNew);
+			pFeedItemInfo->SetStatus(bFirstFetch && pFeedInfo->GetBacklog() ? FeedItemInfo::isBacklog : FeedItemInfo::isNew);
 			FeedHistoryInfo* pFeedHistoryInfo = m_FeedHistory.Find(pFeedItemInfo->GetUrl());
 			if (pFeedHistoryInfo)
 			{
