@@ -1,7 +1,7 @@
 /*
  *  This file is part of nzbget
  *
- *  Copyright (C) 2013-2014 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2013-2015 Andrey Prygunkov <hugbug@users.sourceforge.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
  * $Revision: 0 $
- * $Date: 2014-08-11 00:14:03 +0200 (Mon, 11 Aug 2014) $
+ * $Date$
  *
  */
 
@@ -53,6 +53,7 @@ private:
 	unsigned int		m_iFilterHash;
 	bool				m_bPauseNzb;
 	char*				m_szCategory;
+	char*				m_szFeedScript;
 	int					m_iPriority;
 	time_t				m_tLastUpdate;
 	bool				m_bPreview;
@@ -60,10 +61,12 @@ private:
 	char*				m_szOutputFilename;
 	bool				m_bFetch;
 	bool				m_bForce;
+	bool				m_bBacklog;
 
 public:
-						FeedInfo(int iID, const char* szName, const char* szUrl, int iInterval,
-							const char* szFilter, bool bPauseNzb, const char* szCategory, int iPriority);
+						FeedInfo(int iID, const char* szName, const char* szUrl, bool bBacklog, int iInterval,
+							const char* szFilter, bool bPauseNzb, const char* szCategory, int iPriority,
+							const char* szFeedScript);
 						~FeedInfo();
 	int					GetID() { return m_iID; }
 	const char*			GetName() { return m_szName; }
@@ -74,6 +77,7 @@ public:
 	bool				GetPauseNzb() { return m_bPauseNzb; }
 	const char*			GetCategory() { return m_szCategory; }
 	int					GetPriority() { return m_iPriority; }
+	const char*			GetFeedScript() { return m_szFeedScript; }
 	time_t				GetLastUpdate() { return m_tLastUpdate; }
 	void				SetLastUpdate(time_t tLastUpdate) { m_tLastUpdate = tLastUpdate; }
 	bool				GetPreview() { return m_bPreview; }
@@ -86,19 +90,17 @@ public:
 	void				SetFetch(bool bFetch) { m_bFetch = bFetch; }
 	bool				GetForce() { return m_bForce; }
 	void				SetForce(bool bForce) { m_bForce = bForce; }
+	bool				GetBacklog() { return m_bBacklog; }
+	void				SetBacklog(bool bBacklog) { m_bBacklog = bBacklog; }
 };
 
 typedef std::deque<FeedInfo*> Feeds;
 
-class SharedFeedData
+class FeedFilterHelper
 {
-private:
-	RegEx*				m_pSeasonEpisodeRegEx;
-
 public:
-						SharedFeedData();
-						~SharedFeedData();
-	RegEx*				GetSeasonEpisodeRegEx();
+	virtual RegEx**		GetSeasonEpisodeRegEx() = 0;
+	virtual void		CalcDupeStatus(const char* szTitle, const char* szDupeKey, char* szStatusBuf, int iBufLen) = 0;
 };
 
 class FeedItemInfo
@@ -166,7 +168,7 @@ private:
 	int					m_iDupeScore;
 	EDupeMode			m_eDupeMode;
 	char*				m_szDupeStatus;
-	SharedFeedData*		m_pSharedFeedData;
+	FeedFilterHelper*	m_pFeedFilterHelper;
 	Attributes			m_Attributes;
 
 	int					ParsePrefixedInt(const char *szValue);
@@ -175,7 +177,7 @@ private:
 public:
 						FeedItemInfo();
 						~FeedItemInfo();
-	void				SetSharedFeedData(SharedFeedData* pSharedFeedData) { m_pSharedFeedData = pSharedFeedData; }
+	void				SetFeedFilterHelper(FeedFilterHelper* pFeedFilterHelper) { m_pFeedFilterHelper = pFeedFilterHelper; }
 	const char*			GetTitle() { return m_szTitle; }
 	void				SetTitle(const char* szTitle);
 	const char*			GetFilename() { return m_szFilename; }
@@ -230,7 +232,6 @@ class FeedItemInfos : public FeedItemInfosBase
 {
 private:
 	int					m_iRefCount;
-	SharedFeedData		m_SharedFeedData;
 
 public:
 						FeedItemInfos();
