@@ -18,8 +18,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 1242 $
- * $Date: 2015-03-24 13:18:22 -0400 (Tue, 24 Mar 2015) $
+ * $Revision$
+ * $Date$
  *
  */
 
@@ -37,33 +37,6 @@
 class Options
 {
 public:
-	enum EClientOperation
-	{
-		opClientNoOperation,
-		opClientRequestDownload,
-		opClientRequestListFiles,
-		opClientRequestListGroups,
-		opClientRequestListStatus,
-		opClientRequestSetRate,
-		opClientRequestDumpDebug,
-		opClientRequestEditQueue,
-		opClientRequestLog,
-		opClientRequestShutdown,
-		opClientRequestReload,
-		opClientRequestVersion,
-		opClientRequestPostQueue,
-		opClientRequestWriteLog,
-		opClientRequestScanSync,
-		opClientRequestScanAsync,
-		opClientRequestDownloadPause,
-		opClientRequestDownloadUnpause,
-		opClientRequestPostPause,
-		opClientRequestPostUnpause,
-		opClientRequestScanPause,
-		opClientRequestScanUnpause,
-		opClientRequestHistory,
-		opClientRequestHistoryAll
-	};
 	enum EWriteLog
 	{
 		wlNone,
@@ -94,8 +67,9 @@ public:
 	enum EParScan
 	{
 		psLimited,
+		psExtended,
 		psFull,
-		psAuto
+		psDupe
 	};
 	enum EHealthCheck
 	{
@@ -103,11 +77,20 @@ public:
 		hcDelete,
 		hcNone
 	};
-	enum EMatchMode
+	enum ESchedulerCommand
 	{
-		mmID = 1,
-		mmName,
-		mmRegEx
+		scPauseDownload,
+		scUnpauseDownload,
+		scPausePostProcess,
+		scUnpausePostProcess,
+		scDownloadRate,
+		scScript,
+		scProcess,
+		scPauseScan,
+		scUnpauseScan,
+		scActivateServer,
+		scDeactivateServer,
+		scFetchFeed
 	};
 
 	class OptEntry
@@ -118,8 +101,6 @@ public:
 		char*			m_szDefValue;
 		int				m_iLineNo;
 
-		void			SetName(const char* szName);
-		void			SetValue(const char* szValue);
 		void			SetLineNo(int iLineNo) { m_iLineNo = iLineNo; }
 
 		friend class Options;
@@ -128,7 +109,9 @@ public:
 						OptEntry();
 						OptEntry(const char* szName, const char* szValue);
 						~OptEntry();
+		void			SetName(const char* szName);
 		const char*		GetName() { return m_szName; }
+		void			SetValue(const char* szValue);
 		const char*		GetValue() { return m_szValue; }
 		const char*		GetDefValue() { return m_szDefValue; }
 		int				GetLineNo() { return m_iLineNo; }
@@ -145,6 +128,7 @@ public:
 	};
 
 	typedef std::vector<char*>  NameList;
+	typedef std::vector<const char*>  CmdOptList;
 
 	class Category
 	{
@@ -174,77 +158,28 @@ public:
 		Category*		FindCategory(const char* szName, bool bSearchAliases);
 	};
 
-	class Script
-	{
-	private:
-		char*			m_szName;
-		char*			m_szLocation;
-		char*			m_szDisplayName;
-		bool			m_bPostScript;
-		bool			m_bScanScript;
-		bool			m_bQueueScript;
-		bool			m_bSchedulerScript;
-		char*			m_szQueueEvents;
-
-	public:
-						Script(const char* szName, const char* szLocation);
-						~Script();
-		const char*		GetName() { return m_szName; }
-		const char*		GetLocation() { return m_szLocation; }
-		void			SetDisplayName(const char* szDisplayName);
-		const char*		GetDisplayName() { return m_szDisplayName; }
-		bool			GetPostScript() { return m_bPostScript; }
-		void			SetPostScript(bool bPostScript) { m_bPostScript = bPostScript; }
-		bool			GetScanScript() { return m_bScanScript; }
-		void			SetScanScript(bool bScanScript) { m_bScanScript = bScanScript; }
-		bool			GetQueueScript() { return m_bQueueScript; }
-		void			SetQueueScript(bool bQueueScript) { m_bQueueScript = bQueueScript; }
-		bool			GetSchedulerScript() { return m_bSchedulerScript; }
-		void			SetSchedulerScript(bool bSchedulerScript) { m_bSchedulerScript = bSchedulerScript; }
-		void			SetQueueEvents(const char* szQueueEvents);
-		const char*		GetQueueEvents() { return m_szQueueEvents; }
-	};
-
-	typedef std::list<Script*>  ScriptsBase;
-
-	class Scripts: public ScriptsBase
+	class Extender
 	{
 	public:
-						~Scripts();
-		void			Clear();
-		Script*			Find(const char* szName);	
-	};
-
-	class ConfigTemplate
-	{
-	private:
-		Script*			m_pScript;
-		char*			m_szTemplate;
-
-		friend class Options;
-
-	public:
-						ConfigTemplate(Script* pScript, const char* szTemplate);
-						~ConfigTemplate();
-		Script*			GetScript() { return m_pScript; }
-		const char*		GetTemplate() { return m_szTemplate; }
-	};
-	
-	typedef std::vector<ConfigTemplate*>  ConfigTemplatesBase;
-
-	class ConfigTemplates: public ConfigTemplatesBase
-	{
-	public:
-						~ConfigTemplates();
+		virtual void	AddNewsServer(int iID, bool bActive, const char* szName, const char* szHost,
+							int iPort, const char* szUser, const char* szPass, bool bJoinGroup,
+							bool bTLS, const char* szCipher, int iMaxConnections, int iRetention,
+							int iLevel, int iGroup) = 0;
+		virtual void	AddFeed(int iID, const char* szName, const char* szUrl, int iInterval,
+							const char* szFilter, bool bBacklog, bool bPauseNzb, const char* szCategory,
+							int iPriority, const char* szFeedScript) {}
+		virtual void	AddTask(int iID, int iHours, int iMinutes, int iWeekDaysBits, ESchedulerCommand eCommand,
+							const char* szParam) {}
+		virtual void	SetupFirstStart() {}
 	};
 
 private:
 	OptEntries			m_OptEntries;
-	bool				m_bConfigInitialized;
 	Mutex				m_mutexOptEntries;
 	Categories			m_Categories;
-	Scripts				m_Scripts;
-	ConfigTemplates		m_ConfigTemplates;
+	bool				m_bNoDiskAccess;
+	bool				m_bFatalError;
+	Extender*			m_pExtender;
 
 	// Options
 	bool				m_bConfigErrors;
@@ -259,6 +194,7 @@ private:
 	char*				m_szWebDir;
 	char*				m_szConfigTemplate;
 	char*				m_szScriptDir;
+	char*				m_szRequiredDir;
 	EMessageTarget		m_eInfoTarget;
 	EMessageTarget		m_eWarningTarget;
 	EMessageTarget		m_eErrorTarget;
@@ -275,6 +211,7 @@ private:
 	int					m_iRetries;
 	int					m_iRetryInterval;
 	bool				m_bSaveQueue;
+	bool				m_bFlushQueue;
 	bool				m_bDupeCheck;
 	char*				m_szControlIP;
 	char*				m_szControlUsername;
@@ -310,6 +247,7 @@ private:
 	char*				m_szScriptOrder;
 	char*				m_szScanScript;
 	char*				m_szQueueScript;
+	char*				m_szFeedScript;
 	bool				m_bNoConfig;
 	int					m_iUMask;
 	int					m_iUpdateInterval;
@@ -347,99 +285,61 @@ private:
 	int					m_iArticleCache;
 	int					m_iEventInterval;
 
-	// Parsed command-line parameters
-	bool				m_bServerMode;
-	bool				m_bDaemonMode;
-	bool				m_bRemoteClientMode;
-	int					m_iEditQueueAction;
-	int					m_iEditQueueOffset;
-	int*				m_pEditQueueIDList;
-	int					m_iEditQueueIDCount;
-	NameList			m_EditQueueNameList;
-	EMatchMode			m_EMatchMode;
-	char*				m_szEditQueueText;
-	char*				m_szArgFilename;
-	char*				m_szAddCategory;
-	int					m_iAddPriority;
-	bool				m_bAddPaused;
-	char*				m_szAddNZBFilename;
-	char*				m_szLastArg;
-	bool				m_bPrintOptions;
-	bool				m_bAddTop;
-	char*				m_szAddDupeKey;
-	int					m_iAddDupeScore;
-	int					m_iAddDupeMode;
-	int					m_iSetRate;
-	int					m_iLogLines;
-	int					m_iWriteLogKind;
-	bool				m_bTestBacktrace;
-	bool				m_bWebGet;
-	char*				m_szWebGetFilename;
-
 	// Current state
+	bool				m_bServerMode;
+	bool				m_bRemoteClientMode;
 	bool				m_bPauseDownload;
 	bool				m_bPausePostProcess;
 	bool				m_bPauseScan;
 	bool				m_bTempPauseDownload;
 	int					m_iDownloadRate;
-	EClientOperation	m_eClientOperation;
 	time_t				m_tResumeTime;
 	int					m_iLocalTimeOffset;
+	bool				m_bTempPausePostprocess;
 
-	void				InitDefault();
-	void				InitOptFile();
-	void				InitCommandLine(int argc, char* argv[]);
+	void				Init(const char* szExeName, const char* szConfigFilename, bool bNoConfig,
+							 CmdOptList* pCommandLineOptions, bool bNoDiskAccess, Extender* pExtender);
+	void				InitDefaults();
 	void				InitOptions();
-	void				InitFileArg(int argc, char* argv[]);
+	void				InitOptFile();
 	void				InitServers();
 	void				InitCategories();
 	void				InitScheduler();
 	void				InitFeeds();
-	void				InitScripts();
-	void				InitConfigTemplates();
+	void				InitCommandLineOptions(CmdOptList* pCommandLineOptions);
 	void				CheckOptions();
-	void				PrintUsage(char* com);
 	void				Dump();
 	int					ParseEnumValue(const char* OptName, int argc, const char* argn[], const int argv[]);
 	int					ParseIntValue(const char* OptName, int iBase);
-	float				ParseFloatValue(const char* OptName);
 	OptEntry*			FindOption(const char* optname);
 	const char*			GetOption(const char* optname);
 	void				SetOption(const char* optname, const char* value);
 	bool				SetOptionString(const char* option);
-	bool				SplitOptionString(const char* option, char** pOptName, char** pOptValue);
 	bool				ValidateOptionName(const char* optname, const char* optvalue);
 	void				LoadConfigFile();
 	void				CheckDir(char** dir, const char* szOptionName, const char* szParentDir,
 							bool bAllowEmpty, bool bCreate);
-	void				ParseFileIDList(int argc, char* argv[], int optind);
-	void				ParseFileNameList(int argc, char* argv[], int optind);
 	bool				ParseTime(const char* szTime, int* pHours, int* pMinutes);
 	bool				ParseWeekDays(const char* szWeekDays, int* pWeekDaysBits);
 	void				ConfigError(const char* msg, ...);
 	void				ConfigWarn(const char* msg, ...);
 	void				LocateOptionSrcPos(const char *szOptionName);
 	void				ConvertOldOption(char *szOption, int iOptionBufLen, char *szValue, int iValueBufLen);
-	static bool			CompareScripts(Script* pScript1, Script* pScript2);
-	void				LoadScriptDir(Scripts* pScripts, const char* szDirectory, bool bIsSubDir);
-	void				BuildScriptDisplayNames(Scripts* pScripts);
-	void				LoadScripts(Scripts* pScripts);
 
 public:
-						Options();
+						Options(const char* szExeName, const char* szConfigFilename, bool bNoConfig,
+							CmdOptList* pCommandLineOptions, Extender* pExtender);
+						Options(CmdOptList* pCommandLineOptions, Extender* pExtender);
 						~Options();
-	void				Init(int argc, char* argv[]);
 
-	bool				LoadConfig(OptEntries* pOptEntries);
-	bool				SaveConfig(OptEntries* pOptEntries);
-	bool				LoadConfigTemplates(ConfigTemplates* pConfigTemplates);
-	Scripts*			GetScripts() { return &m_Scripts; }
-	ConfigTemplates*	GetConfigTemplates() { return &m_ConfigTemplates; }
-
-	// Options
+	bool				SplitOptionString(const char* option, char** pOptName, char** pOptValue);
+	bool				GetFatalError() { return m_bFatalError; }
 	OptEntries*			LockOptEntries();
 	void				UnlockOptEntries();
+
+	// Options
 	const char*			GetConfigFilename() { return m_szConfigFilename; }
+	bool				GetConfigErrors() { return m_bConfigErrors; }
 	const char*			GetAppDir() { return m_szAppDir; }
 	const char*			GetDestDir() { return m_szDestDir; }
 	const char*			GetInterDir() { return m_szInterDir; }
@@ -449,6 +349,7 @@ public:
 	const char*			GetWebDir() { return m_szWebDir; }
 	const char*			GetConfigTemplate() { return m_szConfigTemplate; }
 	const char*			GetScriptDir() { return m_szScriptDir; }
+	const char*			GetRequiredDir() { return m_szRequiredDir; }
 	bool				GetBrokenLog() const { return m_bBrokenLog; }
 	bool				GetNzbLog() const { return m_bNzbLog; }
 	EMessageTarget		GetInfoTarget() const { return m_eInfoTarget; }
@@ -465,8 +366,9 @@ public:
 	int					GetRetries() { return m_iRetries; }
 	int					GetRetryInterval() { return m_iRetryInterval; }
 	bool				GetSaveQueue() { return m_bSaveQueue; }
+	bool				GetFlushQueue() { return m_bFlushQueue; }
 	bool				GetDupeCheck() { return m_bDupeCheck; }
-	const char*			GetControlIP();
+	const char*			GetControlIP() { return m_szControlIP; }
 	const char*			GetControlUsername() { return m_szControlUsername; }
 	const char*			GetControlPassword() { return m_szControlPassword; }
 	const char*			GetRestrictedUsername() { return m_szRestrictedUsername; }
@@ -500,6 +402,7 @@ public:
 	const char*			GetPostScript() { return m_szPostScript; }
 	const char*			GetScanScript() { return m_szScanScript; }
 	const char*			GetQueueScript() { return m_szQueueScript; }
+	const char*			GetFeedScript() { return m_szFeedScript; }
 	int					GetUMask() { return m_iUMask; }
 	int					GetUpdateInterval() {return m_iUpdateInterval; }
 	bool				GetCursesNZBName() { return m_bCursesNZBName; }
@@ -539,36 +442,11 @@ public:
 	Categories*			GetCategories() { return &m_Categories; }
 	Category*			FindCategory(const char* szName, bool bSearchAliases) { return m_Categories.FindCategory(szName, bSearchAliases); }
 
-	// Parsed command-line parameters
-	bool				GetServerMode() { return m_bServerMode; }
-	bool				GetDaemonMode() { return m_bDaemonMode; }
-	bool				GetRemoteClientMode() { return m_bRemoteClientMode; }
-	EClientOperation	GetClientOperation() { return m_eClientOperation; }
-	int					GetEditQueueAction() { return m_iEditQueueAction; }
-	int					GetEditQueueOffset() { return m_iEditQueueOffset; }
-	int*				GetEditQueueIDList() { return m_pEditQueueIDList; }
-	int					GetEditQueueIDCount() { return m_iEditQueueIDCount; }
-	NameList*			GetEditQueueNameList() { return &m_EditQueueNameList; }
-	EMatchMode			GetMatchMode() { return m_EMatchMode; }
-	const char*			GetEditQueueText() { return m_szEditQueueText; }
-	const char*			GetArgFilename() { return m_szArgFilename; }
-	const char*			GetAddCategory() { return m_szAddCategory; }
-	bool				GetAddPaused() { return m_bAddPaused; }
-	const char*			GetLastArg() { return m_szLastArg; }
-	int					GetAddPriority() { return m_iAddPriority; }
-	char*				GetAddNZBFilename() { return m_szAddNZBFilename; }
-	bool				GetAddTop() { return m_bAddTop; }
-	const char*			GetAddDupeKey() { return m_szAddDupeKey; }
-	int					GetAddDupeScore() { return m_iAddDupeScore; }
-	int					GetAddDupeMode() { return m_iAddDupeMode; }
-	int					GetSetRate() { return m_iSetRate; }
-	int					GetLogLines() { return m_iLogLines; }
-	int					GetWriteLogKind() { return m_iWriteLogKind; }
-	bool				GetTestBacktrace() { return m_bTestBacktrace; }
-	bool				GetWebGet() { return m_bWebGet; }
-	const char*			GetWebGetFilename() { return m_szWebGetFilename; }
-
 	// Current state
+	void				SetServerMode(bool bServerMode) { m_bServerMode = bServerMode; }
+	bool				GetServerMode() { return m_bServerMode; }
+	void				SetRemoteClientMode(bool bRemoteClientMode) { m_bRemoteClientMode = bRemoteClientMode; }
+	bool				GetRemoteClientMode() { return m_bRemoteClientMode; }
 	void				SetPauseDownload(bool bPauseDownload) { m_bPauseDownload = bPauseDownload; }
 	bool				GetPauseDownload() const { return m_bPauseDownload; }
 	void				SetPausePostProcess(bool bPausePostProcess) { m_bPausePostProcess = bPausePostProcess; }
@@ -577,6 +455,8 @@ public:
 	bool				GetPauseScan() const { return m_bPauseScan; }
 	void				SetTempPauseDownload(bool bTempPauseDownload) { m_bTempPauseDownload = bTempPauseDownload; }
 	bool				GetTempPauseDownload() const { return m_bTempPauseDownload; }
+	bool				GetTempPausePostprocess() const { return m_bTempPausePostprocess; }
+	void				SetTempPausePostprocess(bool bTempPausePostprocess) { m_bTempPausePostprocess = bTempPausePostprocess; }
 	void				SetDownloadRate(int iRate) { m_iDownloadRate = iRate; }
 	int					GetDownloadRate() const { return m_iDownloadRate; }
 	void				SetResumeTime(time_t tResumeTime) { m_tResumeTime = tResumeTime; }
@@ -585,4 +465,6 @@ public:
 	int					GetLocalTimeOffset() { return m_iLocalTimeOffset; }
 };
 
+extern Options* g_pOptions;
+
 #endif
