@@ -1,7 +1,7 @@
 /*
  *  This file is part of nzbget
  *
- *  Copyright (C) 2008-2014 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2008-2015 Andrey Prygunkov <hugbug@users.sourceforge.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,8 +17,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 1124 $
- * $Date: 2014-09-15 12:28:55 -0400 (Mon, 15 Sep 2014) $
+ * $Revision$
+ * $Date$
  *
  */
 
@@ -31,8 +31,9 @@
 #include <time.h>
 
 #include "Thread.h"
+#include "Service.h"
 
-class Scheduler
+class Scheduler : public Service
 {
 public:
 	enum ECommand
@@ -84,6 +85,8 @@ private:
 	bool				m_bPauseScanChanged;
 	bool				m_bServerChanged;
 	ServerStatusList	m_ServerStatusList;
+	bool				m_bFirstChecked;
+
 	void				ExecuteTask(Task* pTask);
 	void				CheckTasks();
 	static bool			CompareTasks(Scheduler::Task* pTask1, Scheduler::Task* pTask2);
@@ -92,13 +95,18 @@ private:
 	void				EditServer(bool bActive, const char* szServerList);
 	void				FetchFeed(const char* szFeedList);
 	void				CheckScheduledResume();
+	void				FirstCheck();
+
+protected:
+	virtual int			ServiceInterval() { return 1000; }
+	virtual void		ServiceWork();
 
 public:
 						Scheduler();
 						~Scheduler();
 	void				AddTask(Task* pTask);
-	void				FirstCheck();
-	void				IntervalCheck();
 };
 
+extern Scheduler* g_pScheduler;
+
 #endif
