@@ -18,8 +18,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 1243 $
- * $Date: 2015-03-25 21:31:53 +0100 (Wed, 25 Mar 2015) $
+ * $Revision$
+ * $Date$
  *
  */
 
@@ -58,12 +58,15 @@
 #include "Log.h"
 #include "NZBFile.h"
 #include "Options.h"
+#include "CommandLineParser.h"
+#include "ScriptConfig.h"
 #include "Thread.h"
 #include "ColoredFrontend.h"
 #include "NCursesFrontend.h"
 #include "QueueCoordinator.h"
 #include "UrlCoordinator.h"
 #include "RemoteServer.h"
+#include "WebServer.h"
 #include "RemoteClient.h"
 #include "MessageBase.h"
 #include "DiskState.h"
@@ -74,6 +77,8 @@
 #include "Scheduler.h"
 #include "Scanner.h"
 #include "FeedCoordinator.h"
+#include "Service.h"
+#include "DiskService.h"
 #include "Maintenance.h"
 #include "ArticleWriter.h"
 #include "StatMeter.h"
@@ -85,6 +90,9 @@
 #include "WinConsole.h"
 #include "WebDownloader.h"
 #endif
+#ifdef ENABLE_TESTS
+#include "TestMain.h"
+#endif
 
 // Prototypes
 void RunMain();
@@ -93,22 +101,23 @@ void Reload();
 void Cleanup();
 void ProcessClientRequest();
 void ProcessWebGet();
+void ProcessSigVerify();
 #ifndef WIN32
 void Daemonize();
 #endif
 #ifndef DISABLE_PARCHECK
 void DisableCout();
 #endif
+void BootConfig();
 
 Thread* g_pFrontend = NULL;
-Options* g_pOptions = NULL;
+CommandLineParser* g_pCommandLineParser = NULL;
 ServerPool* g_pServerPool = NULL;
 QueueCoordinator* g_pQueueCoordinator = NULL;
 UrlCoordinator* g_pUrlCoordinator = NULL;
 RemoteServer* g_pRemoteServer = NULL;
 RemoteServer* g_pRemoteSecureServer = NULL;
 StatMeter* g_pStatMeter = NULL;
-Log* g_pLog = NULL;
 PrePostProcessor* g_pPrePostProcessor = NULL;
 HistoryCoordinator* g_pHistoryCoordinator = NULL;
 DupeCoordinator* g_pDupeCoordinator = NULL;
@@ -119,6 +128,8 @@ FeedCoordinator* g_pFeedCoordinator = NULL;
 Maintenance* g_pMaintenance = NULL;
 ArticleCache* g_pArticleCache = NULL;
 QueueScriptCoordinator* g_pQueueScriptCoordinator = NULL;
+ServiceCoordinator* g_pServiceCoordinator = NULL;
+DiskService* g_pDiskService = NULL;
 int g_iArgumentCount;
 char* (*g_szEnvironmentVariables)[] = NULL;
 char* (*g_szArguments)[] = NULL;
@@ -144,8 +155,26 @@ int main(int argc, char *argv[], char *argp[])
 #endif
 #endif
 
-	Util::InitVersionRevision();
-	
+	Util::Init();
+
+	g_iArgumentCount = argc;
+	g_szArguments = (char*(*)[])argv;
+	g_szEnvironmentVariables = (char*(*)[])argp;
+
+	if (argc > 1 && (!strcmp(argv[1], "-tests") || !strcmp(argv[1], "--tests")))
+	{
+#ifdef ENABLE_TESTS
+		return TestMain(argc, argv);
+#else
+		printf("ERROR: Could not start tests, the program was compiled without tests\n");
+		return 1;
+#endif
+	}
+
+#ifdef ENABLE_TESTS
+	TestCleanup();
+#endif
+
 #ifdef WIN32
 	InstallUninstallServiceCheck(argc, argv);
 #endif
@@ -154,11 +183,7 @@ int main(int argc, char *argv[], char *argp[])
 	DisableCout();
 #endif
 
-	srand (time(NULL));
-
-	g_iArgumentCount = argc;
-	g_szArguments = (char*(*)[])argv;
-	g_szEnvironmentVariables = (char*(*)[])argp;
+	srand(time(NULL));
 
 #ifdef WIN32
 	for (int i=0; i < argc; i++)
@@ -197,7 +222,7 @@ void RunMain()
 
 void Run(bool bReload)
 {
-	g_pLog = new Log();
+	Log::Init();
 
 	debug("nzbget %s", Util::VersionRevision());
 
@@ -211,6 +236,7 @@ void Run(bool bReload)
 	g_pWinConsole->InitAppMode();
 #endif
 
+	g_pServiceCoordinator = new ServiceCoordinator();
 	g_pServerPool = new ServerPool();
 	g_pScheduler = new Scheduler();
 	g_pQueueCoordinator = new QueueCoordinator();
@@ -224,10 +250,9 @@ void Run(bool bReload)
 	g_pArticleCache = new ArticleCache();
 	g_pMaintenance = new Maintenance();
 	g_pQueueScriptCoordinator = new QueueScriptCoordinator();
+	g_pDiskService = new DiskService();
 
-	debug("Reading options");
-	g_pOptions = new Options();
-	g_pOptions->Init(g_iArgumentCount, *g_szArguments);
+	BootConfig();
 
 #ifndef WIN32
 	if (g_pOptions->GetUMask() < 01000)
@@ -237,11 +262,10 @@ void Run(bool bReload)
 	}
 #endif
 	
-	g_pLog->InitOptions();
 	g_pScanner->InitOptions();
 	g_pQueueScriptCoordinator->InitOptions();
 
-	if (g_pOptions->GetDaemonMode())
+	if (g_pCommandLineParser->GetDaemonMode())
 	{
 #ifdef WIN32
 		info("nzbget %s service-mode", Util::VersionRevision());
@@ -257,7 +281,7 @@ void Run(bool bReload)
 	{
 		info("nzbget %s server-mode", Util::VersionRevision());
 	}
-	else if (g_pOptions->GetRemoteClientMode())
+	else if (g_pCommandLineParser->GetRemoteClientMode())
 	{
 		info("nzbget %s remote-mode", Util::VersionRevision());
 	}
@@ -267,7 +291,7 @@ void Run(bool bReload)
 		Connection::Init();
 	}
 
-	if (!g_pOptions->GetRemoteClientMode())
+	if (!g_pCommandLineParser->GetRemoteClientMode())
 	{
 		g_pServerPool->InitConnections();
 		g_pStatMeter->Init();
@@ -276,20 +300,26 @@ void Run(bool bReload)
 	InstallErrorHandler();
 
 #ifdef DEBUG
-	if (g_pOptions->GetTestBacktrace())
+	if (g_pCommandLineParser->GetTestBacktrace())
 	{
 		TestSegFault();
 	}
 #endif
 
-	if (g_pOptions->GetWebGet())
+	if (g_pCommandLineParser->GetWebGet())
 	{
 		ProcessWebGet();
 		return;
 	}
 
+	if (g_pCommandLineParser->GetSigVerify())
+	{
+		ProcessSigVerify();
+		return;
+	}
+
 	// client request
-	if (g_pOptions->GetClientOperation() != Options::opClientNoOperation)
+	if (g_pCommandLineParser->GetClientOperation() != CommandLineParser::opClientNoOperation)
 	{
 		ProcessClientRequest();
 		Cleanup();
@@ -299,6 +329,7 @@ void Run(bool bReload)
 	// Setup the network-server
 	if (g_pOptions->GetServerMode())
 	{
+		WebProcessor::Init();
 		g_pRemoteServer = new RemoteServer(false);
 		g_pRemoteServer->Start();
 
@@ -310,7 +341,7 @@ void Run(bool bReload)
 	}
 
 	// Create the frontend
-	if (!g_pOptions->GetDaemonMode())
+	if (!g_pCommandLineParser->GetDaemonMode())
 	{
 		switch (g_pOptions->GetOutputMode())
 		{
@@ -335,16 +366,17 @@ void Run(bool bReload)
 	}
 
 	// Starting QueueCoordinator and PrePostProcessor
-	if (!g_pOptions->GetRemoteClientMode())
+	if (!g_pCommandLineParser->GetRemoteClientMode())
 	{
 		// Standalone-mode
-		if (!g_pOptions->GetServerMode())
+		if (!g_pCommandLineParser->GetServerMode())
 		{
-			const char* szCategory = g_pOptions->GetAddCategory() ? g_pOptions->GetAddCategory() : "";
-			NZBFile* pNZBFile = NZBFile::Create(g_pOptions->GetArgFilename(), szCategory);
-			if (!pNZBFile)
+			const char* szCategory = g_pCommandLineParser->GetAddCategory() ? g_pCommandLineParser->GetAddCategory() : "";
+			NZBFile* pNZBFile = new NZBFile(g_pCommandLineParser->GetArgFilename(), szCategory);
+			if (!pNZBFile->Parse())
 			{
-				abort("FATAL ERROR: Parsing NZB-document %s failed\n\n", g_pOptions->GetArgFilename() ? g_pOptions->GetArgFilename() : "N/A");
+				printf("Parsing NZB-document %s failed\n\n", g_pCommandLineParser->GetArgFilename() ? g_pCommandLineParser->GetArgFilename() : "N/A");
+				delete pNZBFile;
 				return;
 			}
 			g_pScanner->InitPPParameters(szCategory, pNZBFile->GetNZBInfo()->GetParameters(), false);
@@ -364,6 +396,7 @@ void Run(bool bReload)
 		g_pUrlCoordinator->Start();
 		g_pPrePostProcessor->Start();
 		g_pFeedCoordinator->Start();
+		g_pServiceCoordinator->Start();
 		if (g_pOptions->GetArticleCache() > 0)
 		{
 			g_pArticleCache->Start();
@@ -374,6 +407,7 @@ void Run(bool bReload)
 			g_pUrlCoordinator->IsRunning() || 
 			g_pPrePostProcessor->IsRunning() ||
 			g_pFeedCoordinator->IsRunning() ||
+			g_pServiceCoordinator->IsRunning() ||
 #ifdef WIN32
 			g_pWinConsole->IsRunning() ||
 #endif
@@ -405,6 +439,10 @@ void Run(bool bReload)
 				{
 					g_pArticleCache->Stop();
 				}
+				if (!g_pServiceCoordinator->IsStopped())
+				{
+					g_pServiceCoordinator->Stop();
+				}
 			}
 			usleep(100 * 1000);
 		}
@@ -414,6 +452,7 @@ void Run(bool bReload)
 		debug("UrlCoordinator stopped");
 		debug("PrePostProcessor stopped");
 		debug("FeedCoordinator stopped");
+		debug("ServiceCoordinator stopped");
 		debug("ArticleCache stopped");
 	}
 
@@ -459,7 +498,7 @@ void Run(bool bReload)
 	// Stop Frontend
 	if (g_pFrontend)
 	{
-		if (!g_pOptions->GetRemoteClientMode())
+		if (!g_pCommandLineParser->GetRemoteClientMode())
 		{
 			debug("Stopping Frontend");
 			g_pFrontend->Stop();
@@ -474,107 +513,184 @@ void Run(bool bReload)
 	Cleanup();
 }
 
+class OptionsExtender : public Options::Extender
+{
+protected:
+#ifdef WIN32
+	virtual void		SetupFirstStart()
+	{
+		g_pWinConsole->SetupFirstStart();
+	}
+#endif
+
+	virtual void		AddNewsServer(int iID, bool bActive, const char* szName, const char* szHost,
+							int iPort, const char* szUser, const char* szPass, bool bJoinGroup,
+							bool bTLS, const char* szCipher, int iMaxConnections, int iRetention,
+							int iLevel, int iGroup)
+	{
+		g_pServerPool->AddServer(new NewsServer(iID, bActive, szName, szHost, iPort, szUser, szPass, bJoinGroup,
+							bTLS, szCipher, iMaxConnections, iRetention, iLevel, iGroup));
+	}
+
+	virtual void		AddFeed(int iID, const char* szName, const char* szUrl, int iInterval,
+							const char* szFilter, bool bBacklog, bool bPauseNzb, const char* szCategory,
+							int iPriority, const char* szFeedScript)
+	{
+		g_pFeedCoordinator->AddFeed(new FeedInfo(iID, szName, szUrl, bBacklog, iInterval, szFilter, bPauseNzb, szCategory, iPriority, szFeedScript));
+	}
+
+	virtual void		AddTask(int iID, int iHours, int iMinutes, int iWeekDaysBits,
+							Options::ESchedulerCommand eCommand, const char* szParam)
+	{
+		g_pScheduler->AddTask(new Scheduler::Task(iID, iHours, iMinutes, iWeekDaysBits, (Scheduler::ECommand)eCommand, szParam));
+	}
+} g_OptionsExtender;
+
+void BootConfig()
+{
+	debug("Parsing command line");
+	g_pCommandLineParser = new CommandLineParser(g_iArgumentCount, (const char**)(*g_szArguments));
+	if (g_pCommandLineParser->GetPrintVersion())
+	{
+		printf("nzbget version: %s\n", Util::VersionRevision());
+		exit(0);
+	}
+	if (g_pCommandLineParser->GetPrintUsage() || g_pCommandLineParser->GetErrors() || g_iArgumentCount <= 1)
+	{
+		g_pCommandLineParser->PrintUsage(((const char**)(*g_szArguments))[0]);
+		exit(0);
+	}
+
+	debug("Reading options");
+	g_pOptions = new Options((*g_szArguments)[0], g_pCommandLineParser->GetConfigFilename(),
+							 g_pCommandLineParser->GetNoConfig(), (Options::CmdOptList*)g_pCommandLineParser->GetOptionList(),
+							 &g_OptionsExtender);
+	g_pOptions->SetRemoteClientMode(g_pCommandLineParser->GetRemoteClientMode());
+	g_pOptions->SetServerMode(g_pCommandLineParser->GetServerMode());
+	g_pOptions->SetPauseDownload(g_pCommandLineParser->GetPauseDownload());
+
+	g_pLog->InitOptions();
+
+	if (g_pOptions->GetFatalError())
+	{
+		exit(1);
+	}
+	else if (g_pOptions->GetConfigErrors() &&
+			 g_pCommandLineParser->GetClientOperation() == CommandLineParser::opClientNoOperation)
+	{
+		info("Pausing all activities due to errors in configuration");
+		g_pOptions->SetPauseDownload(true);
+		g_pOptions->SetPausePostProcess(true);
+		g_pOptions->SetPauseScan(true);
+	}
+
+	g_pServerPool->SetTimeout(g_pOptions->GetArticleTimeout());
+	g_pServerPool->SetRetryInterval(g_pOptions->GetRetryInterval());
+
+	g_pScriptConfig = new ScriptConfig();
+}
+
 void ProcessClientRequest()
 {
 	RemoteClient* Client = new RemoteClient();
 
-	switch (g_pOptions->GetClientOperation())
+	switch (g_pCommandLineParser->GetClientOperation())
 	{
-		case Options::opClientRequestListFiles:
-			Client->RequestServerList(true, false, g_pOptions->GetMatchMode() == Options::mmRegEx ? g_pOptions->GetEditQueueText() : NULL);
+		case CommandLineParser::opClientRequestListFiles:
+			Client->RequestServerList(true, false, g_pCommandLineParser->GetMatchMode() == CommandLineParser::mmRegEx ? g_pCommandLineParser->GetEditQueueText() : NULL);
 			break;
 
-		case Options::opClientRequestListGroups:
-			Client->RequestServerList(false, true, g_pOptions->GetMatchMode() == Options::mmRegEx ? g_pOptions->GetEditQueueText() : NULL);
+		case CommandLineParser::opClientRequestListGroups:
+			Client->RequestServerList(false, true, g_pCommandLineParser->GetMatchMode() == CommandLineParser::mmRegEx ? g_pCommandLineParser->GetEditQueueText() : NULL);
 			break;
 
-		case Options::opClientRequestListStatus:
+		case CommandLineParser::opClientRequestListStatus:
 			Client->RequestServerList(false, false, NULL);
 			break;
 
-		case Options::opClientRequestDownloadPause:
+		case CommandLineParser::opClientRequestDownloadPause:
 			Client->RequestServerPauseUnpause(true, eRemotePauseUnpauseActionDownload);
 			break;
 
-		case Options::opClientRequestDownloadUnpause:
+		case CommandLineParser::opClientRequestDownloadUnpause:
 			Client->RequestServerPauseUnpause(false, eRemotePauseUnpauseActionDownload);
 			break;
 
-		case Options::opClientRequestSetRate:
-			Client->RequestServerSetDownloadRate(g_pOptions->GetSetRate());
+		case CommandLineParser::opClientRequestSetRate:
+			Client->RequestServerSetDownloadRate(g_pCommandLineParser->GetSetRate());
 			break;
 
-		case Options::opClientRequestDumpDebug:
+		case CommandLineParser::opClientRequestDumpDebug:
 			Client->RequestServerDumpDebug();
 			break;
 
-		case Options::opClientRequestEditQueue:
-			Client->RequestServerEditQueue((DownloadQueue::EEditAction)g_pOptions->GetEditQueueAction(),
-				g_pOptions->GetEditQueueOffset(), g_pOptions->GetEditQueueText(),
-				g_pOptions->GetEditQueueIDList(), g_pOptions->GetEditQueueIDCount(),
-				g_pOptions->GetEditQueueNameList(), (eRemoteMatchMode)g_pOptions->GetMatchMode());
+		case CommandLineParser::opClientRequestEditQueue:
+			Client->RequestServerEditQueue((DownloadQueue::EEditAction)g_pCommandLineParser->GetEditQueueAction(),
+				g_pCommandLineParser->GetEditQueueOffset(), g_pCommandLineParser->GetEditQueueText(),
+				g_pCommandLineParser->GetEditQueueIDList(), g_pCommandLineParser->GetEditQueueIDCount(),
+				g_pCommandLineParser->GetEditQueueNameList(), (eRemoteMatchMode)g_pCommandLineParser->GetMatchMode());
 			break;
 
-		case Options::opClientRequestLog:
-			Client->RequestServerLog(g_pOptions->GetLogLines());
+		case CommandLineParser::opClientRequestLog:
+			Client->RequestServerLog(g_pCommandLineParser->GetLogLines());
 			break;
 
-		case Options::opClientRequestShutdown:
+		case CommandLineParser::opClientRequestShutdown:
 			Client->RequestServerShutdown();
 			break;
 
-		case Options::opClientRequestReload:
+		case CommandLineParser::opClientRequestReload:
 			Client->RequestServerReload();
 			break;
 
-		case Options::opClientRequestDownload:
-			Client->RequestServerDownload(g_pOptions->GetAddNZBFilename(), g_pOptions->GetArgFilename(),
-				g_pOptions->GetAddCategory(), g_pOptions->GetAddTop(), g_pOptions->GetAddPaused(), g_pOptions->GetAddPriority(),
-				g_pOptions->GetAddDupeKey(), g_pOptions->GetAddDupeMode(), g_pOptions->GetAddDupeScore());
+		case CommandLineParser::opClientRequestDownload:
+			Client->RequestServerDownload(g_pCommandLineParser->GetAddNZBFilename(), g_pCommandLineParser->GetArgFilename(),
+				g_pCommandLineParser->GetAddCategory(), g_pCommandLineParser->GetAddTop(), g_pCommandLineParser->GetAddPaused(), g_pCommandLineParser->GetAddPriority(),
+				g_pCommandLineParser->GetAddDupeKey(), g_pCommandLineParser->GetAddDupeMode(), g_pCommandLineParser->GetAddDupeScore());
 			break;
 
-		case Options::opClientRequestVersion:
+		case CommandLineParser::opClientRequestVersion:
 			Client->RequestServerVersion();
 			break;
 
-		case Options::opClientRequestPostQueue:
+		case CommandLineParser::opClientRequestPostQueue:
 			Client->RequestPostQueue();
 			break;
 
-		case Options::opClientRequestWriteLog:
-			Client->RequestWriteLog(g_pOptions->GetWriteLogKind(), g_pOptions->GetLastArg());
+		case CommandLineParser::opClientRequestWriteLog:
+			Client->RequestWriteLog(g_pCommandLineParser->GetWriteLogKind(), g_pCommandLineParser->GetLastArg());
 			break;
 
-		case Options::opClientRequestScanAsync:
+		case CommandLineParser::opClientRequestScanAsync:
 			Client->RequestScan(false);
 			break;
 
-		case Options::opClientRequestScanSync:
+		case CommandLineParser::opClientRequestScanSync:
 			Client->RequestScan(true);
 			break;
 
-		case Options::opClientRequestPostPause:
+		case CommandLineParser::opClientRequestPostPause:
 			Client->RequestServerPauseUnpause(true, eRemotePauseUnpauseActionPostProcess);
 			break;
 
-		case Options::opClientRequestPostUnpause:
+		case CommandLineParser::opClientRequestPostUnpause:
 			Client->RequestServerPauseUnpause(false, eRemotePauseUnpauseActionPostProcess);
 			break;
 
-		case Options::opClientRequestScanPause:
+		case CommandLineParser::opClientRequestScanPause:
 			Client->RequestServerPauseUnpause(true, eRemotePauseUnpauseActionScan);
 			break;
 
-		case Options::opClientRequestScanUnpause:
+		case CommandLineParser::opClientRequestScanUnpause:
 			Client->RequestServerPauseUnpause(false, eRemotePauseUnpauseActionScan);
 			break;
 
-		case Options::opClientRequestHistory:
-		case Options::opClientRequestHistoryAll:
-			Client->RequestHistory(g_pOptions->GetClientOperation() == Options::opClientRequestHistoryAll);
+		case CommandLineParser::opClientRequestHistory:
+		case CommandLineParser::opClientRequestHistoryAll:
+			Client->RequestHistory(g_pCommandLineParser->GetClientOperation() == CommandLineParser::opClientRequestHistoryAll);
 			break;
 
-		case Options::opClientNoOperation:
+		case CommandLineParser::opClientNoOperation:
 			break;
 	}
 
@@ -584,31 +700,37 @@ void ProcessClientRequest()
 void ProcessWebGet()
 {
 	WebDownloader downloader;
-	downloader.SetURL(g_pOptions->GetLastArg());
+	downloader.SetURL(g_pCommandLineParser->GetLastArg());
 	downloader.SetForce(true);
 	downloader.SetRetry(false);
-	downloader.SetOutputFilename(g_pOptions->GetWebGetFilename());
+	downloader.SetOutputFilename(g_pCommandLineParser->GetWebGetFilename());
 	downloader.SetInfoName("WebGet");
 
-	int iRedirects = 0;
-	WebDownloader::EStatus eStatus = WebDownloader::adRedirect;
-	while (eStatus == WebDownloader::adRedirect && iRedirects < 5)
-	{
-		iRedirects++;
-		eStatus = downloader.Download();
-	}
+	WebDownloader::EStatus eStatus = downloader.DownloadWithRedirects(5);
 	bool bOK = eStatus == WebDownloader::adFinished;
 
 	exit(bOK ? 0 : 1);
 }
 
+void ProcessSigVerify()
+{
+#ifdef HAVE_OPENSSL
+	bool bOK = Maintenance::VerifySignature(g_pCommandLineParser->GetLastArg(),
+		g_pCommandLineParser->GetSigFilename(), g_pCommandLineParser->GetPubKeyFilename());
+	exit(bOK ? 93 : 1);
+#else
+	printf("ERROR: Could not verify signature, the program was compiled without OpenSSL support\n");
+	exit(1);	
+#endif
+}
+
 void ExitProc()
 {
 	if (!g_bReloading)
 	{
 		info("Stopping, please wait...");
 	}
-	if (g_pOptions->GetRemoteClientMode())
+	if (g_pCommandLineParser->GetRemoteClientMode())
 	{
 		if (g_pFrontend)
 		{
@@ -621,11 +743,13 @@ void ExitProc()
 		if (g_pQueueCoordinator)
 		{
 			debug("Stopping QueueCoordinator");
+			g_pServiceCoordinator->Stop();
 			g_pQueueCoordinator->Stop();
 			g_pUrlCoordinator->Stop();
 			g_pPrePostProcessor->Stop();
 			g_pFeedCoordinator->Stop();
 			g_pArticleCache->Stop();
+			g_pQueueScriptCoordinator->Stop();
 #ifdef WIN32
 			g_pWinConsole->Stop();
 #endif
@@ -694,16 +818,31 @@ void Cleanup()
 	debug("Deleting Options");
 	if (g_pOptions)
 	{
-		if (g_pOptions->GetDaemonMode() && !g_bReloading)
+		if (g_pCommandLineParser->GetDaemonMode() && !g_bReloading)
 		{
 			info("Deleting lock file");
 			remove(g_pOptions->GetLockFile());
 		}
 		delete g_pOptions;
-		g_pOptions = NULL;
 	}
 	debug("Options deleted");
 
+	debug("Deleting CommandLineParser");
+	if (g_pCommandLineParser)
+	{
+		delete g_pCommandLineParser;
+		g_pCommandLineParser = NULL;
+	}
+	debug("CommandLineParser deleted");
+
+	debug("Deleting ScriptConfig");
+	if (g_pScriptConfig)
+	{
+		delete g_pScriptConfig;
+		g_pScriptConfig = NULL;
+	}
+	debug("ScriptConfig deleted");
+
 	debug("Deleting ServerPool");
 	delete g_pServerPool;
 	g_pServerPool = NULL;
@@ -739,6 +878,16 @@ void Cleanup()
 	g_pStatMeter = NULL;
 	debug("StatMeter deleted");
 
+	debug("Deleting ServiceCoordinator");
+	delete g_pServiceCoordinator;
+	g_pServiceCoordinator = NULL;
+	debug("ServiceCoordinator deleted");
+
+	debug("Deleting DiskService");
+	delete g_pDiskService;
+	g_pDiskService = NULL;
+	debug("DiskService deleted");
+
 	if (!g_bReloading)
 	{
 		Connection::Final();
@@ -752,8 +901,7 @@ void Cleanup()
 
 	debug("Global objects cleaned up");
 
-	delete g_pLog;
-	g_pLog = NULL;
+	Log::Final();
 }
 
 #ifndef WIN32
@@ -849,3 +997,4 @@ void DisableCout()
 	std::cout.rdbuf(&NullStreamBufInstance);
 }
 #endif
+
