@@ -1,7 +1,7 @@
 /*
  *  This file is part of nzbget
  *
- *  Copyright (C) 2007-2013 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2007-2015 Andrey Prygunkov <hugbug@users.sourceforge.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,8 +17,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 1122 $
- * $Date: 2014-09-08 21:35:11 +0200 (Mon, 08 Sep 2014) $
+ * $Revision$
+ * $Date$
  *
  */
 
@@ -39,7 +39,9 @@
 #define gmtime_r(time, tm) gmtime_s(tm, time)
 #define strtok_r(str, delim, saveptr) strtok_s(str, delim, saveptr)
 #define strerror_r(errnum, buffer, size) strerror_s(buffer, size, errnum)
+#if (_MSC_VER < 1600)
 #define int32_t __int32
+#endif
 #define mkdir(dir, flags) _mkdir(dir)
 #define rmdir _rmdir
 #define strcasecmp(a, b) _stricmp(a, b)
@@ -60,6 +62,9 @@
 #define atoll _atoi64
 #define fseek _fseeki64
 #define ftell _ftelli64
+#if _MSC_VER < 1800 // va_copy is available in vc2013 and onwards
+#define va_copy(d,s) ((d) = (s))
+#endif
 #ifndef FSCTL_SET_SPARSE
 #define FSCTL_SET_SPARSE 590020
 #endif
@@ -70,6 +75,11 @@
 #define FOPEN_AB "abN"
 #define FOPEN_ABP "ab+N"
 
+#ifdef DEBUG
+// redefine "exit" to avoid printing memory leaks report when terminated because of wrong command line switches
+#define exit(code) ExitProcess(code)
+#endif
+
 #pragma warning(disable:4800) // 'type' : forcing value to bool 'true' or 'false' (performance warning)
 #pragma warning(disable:4267) // 'var' : conversion from 'size_t' to 'type', possible loss of data
 
