@@ -2,7 +2,7 @@
  *  This file is part of nzbget
  *
  *  Copyright (C) 2004 Sven Henkel <sidddy@users.sourceforge.net>
- *  Copyright (C) 2007-2014 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2007-2015 Andrey Prygunkov <hugbug@users.sourceforge.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,8 +18,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 1253 $
- * $Date: 2015-04-03 07:34:33 -0400 (Fri, 03 Apr 2015) $
+ * $Revision$
+ * $Date$
  *
  */
 
@@ -54,10 +54,6 @@
 #include "StatMeter.h"
 #include "Util.h"
 
-extern Options* g_pOptions;
-extern ServerPool* g_pServerPool;
-extern StatMeter* g_pStatMeter;
-
 ArticleDownloader::ArticleDownloader()
 {
 	debug("Creating ArticleDownloader");
@@ -456,9 +452,16 @@ ArticleDownloader::EStatus ArticleDownloader::Download()
 				}
 			}
 		}
-		else if (m_eFormat == Decoder::efUnknown && g_pOptions->GetDecode())
+
+		if (m_eFormat == Decoder::efUnknown && g_pOptions->GetDecode())
 		{
-			m_eFormat = Decoder::DetectFormat(line, iLen);
+			m_eFormat = Decoder::DetectFormat(line, iLen, bBody);
+			if (m_eFormat != Decoder::efUnknown)
+			{
+				// sometimes news servers misbehave and send article body without new line separator between headers and body
+				// if we found decoder signature we know the body is already arrived
+				bBody = true;
+			}
 		}
 
 		// write to output file
