@@ -17,8 +17,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 1222 $
- * $Date: 2015-02-27 13:43:15 -0500 (Fri, 27 Feb 2015) $
+ * $Revision$
+ * $Date$
  *
  */
 
@@ -51,11 +51,6 @@
 #include "Log.h"
 #include "Util.h"
 
-extern Options* g_pOptions;
-extern DiskState* g_pDiskState;
-extern ArticleCache* g_pArticleCache;
-
-
 ArticleWriter::ArticleWriter()
 {
 	debug("Creating ArticleWriter");
