@@ -17,8 +17,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 1122 $
- * $Date: 2014-09-08 15:35:11 -0400 (Mon, 08 Sep 2014) $
+ * $Revision$
+ * $Date$
  *
  */
 
@@ -65,14 +65,14 @@ void Decoder::Clear()
 	m_szArticleFilename = NULL;
 }
 
-Decoder::EFormat Decoder::DetectFormat(const char* buffer, int len)
+Decoder::EFormat Decoder::DetectFormat(const char* buffer, int len, bool inBody)
 {
 	if (!strncmp(buffer, "=ybegin ", 8))
 	{
 		return efYenc;
 	}
 	
-	if ((len == 62 || len == 63) && (buffer[62] == '\n' || buffer[62] == '\r') && *buffer == 'M')
+	if (inBody && (len == 62 || len == 63) && (buffer[62] == '\n' || buffer[62] == '\r') && *buffer == 'M')
 	{
 		return efUx;
 	}
