@@ -17,8 +17,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 1238 $
- * $Date: 2015-03-21 12:12:01 -0400 (Sat, 21 Mar 2015) $
+ * $Revision$
+ * $Date$
  *
  */
 
@@ -42,10 +42,6 @@
 #include "DiskState.h"
 #include "Util.h"
 
-extern ServerPool* g_pServerPool;
-extern Options* g_pOptions;
-extern DiskState* g_pDiskState;
-
 static const int DAYS_UP_TO_2013_JAN_1 = 15706;
 static const int DAYS_IN_TWENTY_YEARS = 366*20;
 
@@ -388,11 +384,7 @@ void StatMeter::AddSpeedReading(int iBytes)
 
 	if (g_pOptions->GetAccurateRate())
 	{
-#ifdef HAVE_SPINLOCK
-		m_spinlockSpeed.Lock();
-#else
 		m_mutexSpeed.Lock();
-#endif
 	}
 
 	if (tCurTime != m_tCurSecTime)
@@ -445,11 +437,7 @@ void StatMeter::AddSpeedReading(int iBytes)
 
 	if (g_pOptions->GetAccurateRate())
 	{
-#ifdef HAVE_SPINLOCK
-		m_spinlockSpeed.Unlock();
-#else
 		m_mutexSpeed.Unlock();
-#endif
 	}
 }
 
@@ -472,9 +460,9 @@ void StatMeter::ResetSpeedStat()
 void StatMeter::LogDebugInfo()
 {
 	info("   ---------- SpeedMeter");
-	float fSpeed = (float)(CalcCurrentDownloadSpeed() / 1024.0);
+	int iSpeed = CalcCurrentDownloadSpeed() / 1024;
 	int iTimeDiff = (int)time(NULL) - m_iSpeedStartTime * SPEEDMETER_SLOTSIZE;
-	info("      Speed: %f", fSpeed);
+	info("      Speed: %i", iSpeed);
 	info("      SpeedStartTime: %i", m_iSpeedStartTime);
 	info("      SpeedTotalBytes: %i", m_iSpeedTotalBytes);
 	info("      SpeedBytesIndex: %i", m_iSpeedBytesIndex);
