@@ -1,7 +1,7 @@
 /*
  *  This file is part of nzbget
  *
- *  Copyright (C) 2014 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2014-2015 Andrey Prygunkov <hugbug@users.sourceforge.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,8 +17,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 1238 $
- * $Date: 2015-03-21 12:12:01 -0400 (Sat, 21 Mar 2015) $
+ * $Revision$
+ * $Date$
  *
  */
 
@@ -95,11 +95,7 @@ private:
 	int					m_iSpeedBytesIndex;
 	int					m_iCurSecBytes;
 	time_t				m_tCurSecTime;
-#ifdef HAVE_SPINLOCK
-	SpinLock			m_spinlockSpeed;
-#else
 	Mutex				m_mutexSpeed;
-#endif
 
 	// time
 	long long			m_iAllBytes;
@@ -140,4 +136,6 @@ public:
 	bool				Load(bool* pPerfectServerMatch);
 };
 
+extern StatMeter* g_pStatMeter;
+
 #endif
