@@ -17,8 +17,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 1229 $
- * $Date: 2015-03-10 18:13:06 -0400 (Tue, 10 Mar 2015) $
+ * $Revision$
+ * $Date$
  *
  */
 
@@ -50,13 +50,11 @@
 #include "nzbget.h"
 #include "Thread.h"
 #include "ParChecker.h"
-#include "ParCoordinator.h"
+#include "ParParser.h"
 #include "Log.h"
 #include "Options.h"
 #include "Util.h"
 
-extern Options* g_pOptions;
-
 const char* Par2CmdLineErrStr[] = { "OK",
 	"data files are damaged and there is enough recovery data available to repair them",
 	"data files are damaged and there is insufficient recovery data available to be able to repair them",
@@ -86,12 +84,7 @@ private:
 	ParChecker*		m_pOwner;
 	Threads			m_Threads;
 	bool			m_bParallel;
-
-#ifdef HAVE_SPINLOCK
-	SpinLock		progresslock;
-#else
 	Mutex			progresslock;
-#endif
 
 	virtual void	BeginRepair();
 	virtual void	EndRepair();
@@ -99,7 +92,7 @@ private:
 
 protected:
 	virtual void	sig_filename(std::string filename) { m_pOwner->signal_filename(filename); }
-	virtual void	sig_progress(double progress) { m_pOwner->signal_progress(progress); }
+	virtual void	sig_progress(int progress) { m_pOwner->signal_progress(progress); }
 	virtual void	sig_done(std::string filename, int available, int total) { m_pOwner->signal_done(filename, available, total); }
 
 	virtual bool	ScanDataFile(DiskFile *diskfile, Par2RepairerSourceFile* &sourcefile,
@@ -171,7 +164,9 @@ Result Repairer::PreProcess(const char *szParFilename)
 
 Result Repairer::Process(bool dorepair)
 {
-	return Par2Repairer::Process(commandLine, dorepair);
+	Result res = Par2Repairer::Process(commandLine, dorepair);
+	debug("ParChecker: Process-result=%i", res);
+	return res;
 }
 
 
@@ -186,16 +181,19 @@ bool Repairer::ScanDataFile(DiskFile *diskfile, Par2RepairerSourceFile* &sourcef
 
 		sig_filename(name);
 
-		int iAvailableBlocks = sourcefile->BlockCount();
-		ParChecker::EFileStatus eFileStatus = m_pOwner->VerifyDataFile(diskfile, sourcefile, &iAvailableBlocks);
-		if (eFileStatus != ParChecker::fsUnknown)
+		if (!(m_pOwner->GetStage() == ParChecker::ptVerifyingRepaired && m_pOwner->GetParFull()))
 		{
-			sig_done(name, iAvailableBlocks, sourcefile->BlockCount());
-			sig_progress(1000.0);
-			matchtype = eFileStatus == ParChecker::fsSuccess ? eFullMatch :
-				eFileStatus == ParChecker::fsPartial ? ePartialMatch : eNoMatch;
-			m_pOwner->SetParFull(false);
-			return true;
+			int iAvailableBlocks = sourcefile->BlockCount();
+			ParChecker::EFileStatus eFileStatus = m_pOwner->VerifyDataFile(diskfile, sourcefile, &iAvailableBlocks);
+			if (eFileStatus != ParChecker::fsUnknown)
+			{
+				sig_done(name, iAvailableBlocks, sourcefile->BlockCount());
+				sig_progress(1000);
+				matchtype = eFileStatus == ParChecker::fsSuccess ? eFullMatch :
+					eFileStatus == ParChecker::fsPartial ? ePartialMatch : eNoMatch;
+				m_pOwner->SetParFull(false);
+				return true;
+			}
 		}
 	}
 
@@ -395,6 +393,18 @@ ParChecker::SegmentList::~SegmentList()
 	}
 }
 
+ParChecker::DupeSource::DupeSource(int iID, const char* szDirectory)
+{
+	m_iID = iID;
+	m_szDirectory = strdup(szDirectory);
+	m_iUsedBlocks = 0;
+}
+
+ParChecker::DupeSource::~DupeSource()
+{
+	free(m_szDirectory);
+}
+
 
 ParChecker::ParChecker()
 {
@@ -411,6 +421,7 @@ ParChecker::ParChecker()
 	m_iFileProgress = 0;
 	m_iStageProgress = 0;
 	m_iExtraFiles = 0;
+	m_iQuickFiles = 0;
 	m_bVerifyingExtraFiles = false;
 	m_bCancelled = false;
 	m_eStage = ptLoadingPars;
@@ -450,6 +461,12 @@ void ParChecker::Cleanup()
 
 	m_sourceFiles.clear();
 
+	for (DupeSourceList::iterator it = m_DupeSources.begin(); it != m_DupeSources.end() ;it++)
+	{
+		free(*it);
+	}
+	m_DupeSources.clear();
+
 	free(m_szErrMsg);
 	m_szErrMsg = NULL;
 }
@@ -488,8 +505,8 @@ void ParChecker::Run()
 
 ParChecker::EStatus ParChecker::RunParCheckAll()
 {
-	ParCoordinator::ParFileList fileList;
-	if (!ParCoordinator::FindMainPars(m_szDestDir, &fileList))
+	ParParser::ParFileList fileList;
+	if (!ParParser::FindMainPars(m_szDestDir, &fileList))
 	{
 		PrintMessage(Message::mkError, "Could not start par-check for %s. Could not find any par-files", m_szNZBName);
 		return psFailed;
@@ -499,7 +516,7 @@ ParChecker::EStatus ParChecker::RunParCheckAll()
 	m_bCancelled = false;
 	m_bParFull = true;
 
-	for (ParCoordinator::ParFileList::iterator it = fileList.begin(); it != fileList.end(); it++)
+	for (ParParser::ParFileList::iterator it = fileList.begin(); it != fileList.end(); it++)
 	{
 		char* szParFilename = *it;
 		debug("Found par: %s", szParFilename);
@@ -512,7 +529,7 @@ ParChecker::EStatus ParChecker::RunParCheckAll()
 
 			char szInfoName[1024];
 			int iBaseLen = 0;
-			ParCoordinator::ParseParFilename(szParFilename, &iBaseLen, NULL);
+			ParParser::ParseParFilename(szParFilename, &iBaseLen, NULL);
 			int maxlen = iBaseLen < 1024 ? iBaseLen : 1024 - 1;
 			strncpy(szInfoName, szParFilename, maxlen);
 			szInfoName[maxlen] = '\0';
@@ -550,6 +567,7 @@ ParChecker::EStatus ParChecker::RunParCheck(const char* szParFilename)
 	m_eStage = ptLoadingPars;
 	m_iProcessedFiles = 0;
 	m_iExtraFiles = 0;
+	m_iQuickFiles = 0;
 	m_bVerifyingExtraFiles = false;
 	m_bHasDamagedFiles = false;
 	EStatus eStatus = psFailed;
@@ -574,7 +592,6 @@ ParChecker::EStatus ParChecker::RunParCheck(const char* szParFilename)
 	m_eStage = ptVerifyingSources;
 	Repairer* pRepairer = (Repairer*)m_pRepairer;
 	res = pRepairer->Process(false);
-    debug("ParChecker: Process-result=%i", res);
 
 	if (!m_bParQuick)
 	{
@@ -588,18 +605,17 @@ ParChecker::EStatus ParChecker::RunParCheck(const char* szParFilename)
 		if (bAddedSplittedFragments)
 		{
 			res = pRepairer->Process(false);
-			debug("ParChecker: Process-result=%i", res);
 		}
 	}
 
 	if (m_bHasDamagedFiles && !IsStopped() && pRepairer->missingfilecount > 0 && 
 		!(bAddedSplittedFragments && res == eRepairPossible) &&
-		g_pOptions->GetParScan() == Options::psAuto)
+		(g_pOptions->GetParScan() == Options::psExtended ||
+		 g_pOptions->GetParScan() == Options::psDupe))
 	{
 		if (AddMissingFiles())
 		{
 			res = pRepairer->Process(false);
-			debug("ParChecker: Process-result=%i", res);
 		}
 	}
 
@@ -608,6 +624,19 @@ ParChecker::EStatus ParChecker::RunParCheck(const char* szParFilename)
 		res = (Result)ProcessMorePars();
 	}
 
+	if (m_bHasDamagedFiles && !IsStopped() && res == eRepairNotPossible &&
+		g_pOptions->GetParScan() == Options::psDupe)
+	{
+		if (AddDupeFiles())
+		{
+			res = pRepairer->Process(false);
+			if (!IsStopped() && res == eRepairNotPossible)
+			{
+				res = (Result)ProcessMorePars();
+			}
+		}
+	}
+
 	if (IsStopped())
 	{
 		Cleanup();
@@ -639,11 +668,11 @@ ParChecker::EStatus ParChecker::RunParCheck(const char* szParFilename)
 			UpdateProgress();
 
 			res = pRepairer->Process(true);
-    		debug("ParChecker: Process-result=%i", res);
 			if (res == eSuccess)
 			{
     			PrintMessage(Message::mkInfo, "Successfully repaired %s", m_szInfoName);
 				eStatus = psRepaired;
+				StatDupeSources(&m_DupeSources);
 				DeleteLeftovers();
 			}
 		}
@@ -858,7 +887,6 @@ int ParChecker::ProcessMorePars()
 		{
 			pRepairer->UpdateVerificationResults();
 			res = pRepairer->Process(false);
-			debug("ParChecker: Process-result=%i", res);
 		}
 	}
 
@@ -908,20 +936,9 @@ void ParChecker::QueueChanged()
 
 bool ParChecker::AddSplittedFragments()
 {
-	char szDirectory[1024];
-	strncpy(szDirectory, m_szParFilename, 1024);
-	szDirectory[1024-1] = '\0';
-
-	char* szBasename = Util::BaseFileName(szDirectory);
-	if (szBasename == szDirectory)
-	{
-		return false;
-	}
-	szBasename[-1] = '\0';
-
 	std::list<CommandLine::ExtraFile> extrafiles;
 
-	DirBrowser dir(szDirectory);
+	DirBrowser dir(m_szDestDir);
 	while (const char* filename = dir.Next())
 	{
 		if (strcmp(filename, ".") && strcmp(filename, "..") && strcmp(filename, "_brokenlog.txt") &&
@@ -948,7 +965,7 @@ bool ParChecker::AddSplittedFragments()
 							debug("Found splitted fragment %s", filename);
 
 							char fullfilename[1024];
-							snprintf(fullfilename, 1024, "%s%c%s", szDirectory, PATH_SEPARATOR, filename);
+							snprintf(fullfilename, 1024, "%s%c%s", m_szDestDir, PATH_SEPARATOR, filename);
 							fullfilename[1024-1] = '\0';
 
 							CommandLine::ExtraFile extrafile(fullfilename, Util::FileSize(fullfilename));
@@ -977,18 +994,64 @@ bool ParChecker::AddSplittedFragments()
 
 bool ParChecker::AddMissingFiles()
 {
-    PrintMessage(Message::mkInfo, "Performing extra par-scan for %s", m_szInfoName);
+	return AddExtraFiles(true, false, m_szDestDir);
+}
 
+bool ParChecker::AddDupeFiles()
+{
 	char szDirectory[1024];
 	strncpy(szDirectory, m_szParFilename, 1024);
 	szDirectory[1024-1] = '\0';
 
-	char* szBasename = Util::BaseFileName(szDirectory);
-	if (szBasename == szDirectory)
+	bool bAdded = AddExtraFiles(false, false, szDirectory);
+
+	if (((Repairer*)m_pRepairer)->missingblockcount > 0)
 	{
-		return false;
+		// scanning directories of duplicates
+		RequestDupeSources(&m_DupeSources);
+
+		if (!m_DupeSources.empty())
+		{
+			int iWasBlocksMissing = ((Repairer*)m_pRepairer)->missingblockcount;
+
+			for (DupeSourceList::iterator it = m_DupeSources.begin(); it != m_DupeSources.end(); it++)
+			{
+				DupeSource* pDupeSource = *it;
+				if (((Repairer*)m_pRepairer)->missingblockcount > 0 && Util::DirectoryExists(pDupeSource->GetDirectory()))
+				{
+					int iWasBlocksMissing2 = ((Repairer*)m_pRepairer)->missingblockcount;
+					bool bOneAdded = AddExtraFiles(false, true, pDupeSource->GetDirectory());
+					bAdded |= bOneAdded;
+					int iBlocksMissing2 = ((Repairer*)m_pRepairer)->missingblockcount;
+					pDupeSource->SetUsedBlocks(pDupeSource->GetUsedBlocks() + (iWasBlocksMissing2 - iBlocksMissing2));
+				}
+			}
+
+			int iBlocksMissing = ((Repairer*)m_pRepairer)->missingblockcount;
+			if (iBlocksMissing < iWasBlocksMissing)
+			{
+				PrintMessage(Message::mkInfo, "Found extra %i blocks in dupe sources", iWasBlocksMissing - iBlocksMissing);
+			}
+			else
+			{
+				PrintMessage(Message::mkInfo, "No extra blocks found in dupe sources");
+			}
+		}
+	}
+
+	return bAdded;
+}
+
+bool ParChecker::AddExtraFiles(bool bOnlyMissing, bool bExternalDir, const char* szDirectory)
+{
+	if (bExternalDir)
+	{
+		PrintMessage(Message::mkInfo, "Performing dupe par-scan for %s in %s", m_szInfoName, Util::BaseFileName(szDirectory));
+	}
+	else
+	{
+		PrintMessage(Message::mkInfo, "Performing extra par-scan for %s", m_szInfoName);
 	}
-	szBasename[-1] = '\0';
 
 	std::list<CommandLine::ExtraFile*> extrafiles;
 
@@ -996,7 +1059,7 @@ bool ParChecker::AddMissingFiles()
 	while (const char* filename = dir.Next())
 	{
 		if (strcmp(filename, ".") && strcmp(filename, "..") && strcmp(filename, "_brokenlog.txt") &&
-			!IsParredFile(filename) && !IsProcessedFile(filename))
+			(bExternalDir || (!IsParredFile(filename) && !IsProcessedFile(filename))))
 		{
 			char fullfilename[1024];
 			snprintf(fullfilename, 1024, "%s%c%s", szDirectory, PATH_SEPARATOR, filename);
@@ -1023,7 +1086,7 @@ bool ParChecker::AddMissingFiles()
 
 		// adding files one by one until all missing files are found
 
-		while (!IsStopped() && !m_bCancelled && extrafiles.size() > 0 && ((Repairer*)m_pRepairer)->missingfilecount > 0)
+		while (!IsStopped() && !m_bCancelled && extrafiles.size() > 0)
 		{
 			CommandLine::ExtraFile* pExtraFile = extrafiles.front();
 			extrafiles.pop_front();
@@ -1031,19 +1094,40 @@ bool ParChecker::AddMissingFiles()
 			extrafiles1.clear();
 			extrafiles1.push_back(*pExtraFile);
 
-			int iWasMissing = ((Repairer*)m_pRepairer)->missingfilecount;
+			int iWasFilesMissing = ((Repairer*)m_pRepairer)->missingfilecount;
+			int iWasBlocksMissing = ((Repairer*)m_pRepairer)->missingblockcount;
+
 			((Repairer*)m_pRepairer)->VerifyExtraFiles(extrafiles1);
-			bool bAdded = iWasMissing > (int)((Repairer*)m_pRepairer)->missingfilecount;
-			if (bAdded)
+			((Repairer*)m_pRepairer)->UpdateVerificationResults();
+
+			bool bFileAdded = iWasFilesMissing > (int)((Repairer*)m_pRepairer)->missingfilecount;
+			bool bBlockAdded = iWasBlocksMissing > (int)((Repairer*)m_pRepairer)->missingblockcount;
+
+			if (bFileAdded && !bExternalDir)
 			{
 				PrintMessage(Message::mkInfo, "Found missing file %s", Util::BaseFileName(pExtraFile->FileName().c_str()));
 				RegisterParredFile(Util::BaseFileName(pExtraFile->FileName().c_str()));
 			}
+			else if (bBlockAdded)
+			{
+				PrintMessage(Message::mkInfo, "Found %i missing blocks", iWasBlocksMissing - (int)((Repairer*)m_pRepairer)->missingblockcount);
+			}
 
-			bFilesAdded |= bAdded;
-			((Repairer*)m_pRepairer)->UpdateVerificationResults();
+			bFilesAdded |= bFileAdded | bBlockAdded;
 
 			delete pExtraFile;
+
+			if (bOnlyMissing && ((Repairer*)m_pRepairer)->missingfilecount == 0)
+			{
+				PrintMessage(Message::mkInfo, "All missing files found, aborting par-scan");
+				break;
+			}
+
+			if (!bOnlyMissing && ((Repairer*)m_pRepairer)->missingblockcount == 0)
+			{
+				PrintMessage(Message::mkInfo, "All missing blocks found, aborting par-scan");
+				break;
+			}
 		}
 
 		m_bVerifyingExtraFiles = false;
@@ -1106,7 +1190,7 @@ void ParChecker::signal_filename(std::string str)
 	UpdateProgress();
 }
 
-void ParChecker::signal_progress(double progress)
+void ParChecker::signal_progress(int progress)
 {
 	m_iFileProgress = (int)progress;
 
@@ -1120,6 +1204,7 @@ void ParChecker::signal_progress(double progress)
 		// processing individual files
 
 		int iTotalFiles = 0;
+		int iProcessedFiles = m_iProcessedFiles;
 		if (m_eStage == ptVerifyingRepaired)
 		{
 			// repairing individual files
@@ -1129,17 +1214,25 @@ void ParChecker::signal_progress(double progress)
 		{
 			// verifying individual files
 			iTotalFiles = ((Repairer*)m_pRepairer)->sourcefiles.size() + m_iExtraFiles;
+			if (m_iExtraFiles > 0)
+			{
+				// during extra par scan don't count quickly verified files;
+				// extra files require much more time for verification;
+				// counting only fully scanned files improves estimated time accuracy.
+				iTotalFiles -= m_iQuickFiles;
+				iProcessedFiles -= m_iQuickFiles;
+			}
 		}
 
 		if (iTotalFiles > 0)
 		{
 			if (m_iFileProgress < 1000)
 			{
-				m_iStageProgress = (m_iProcessedFiles * 1000 + m_iFileProgress) / iTotalFiles;
+				m_iStageProgress = (iProcessedFiles * 1000 + m_iFileProgress) / iTotalFiles;
 			}
 			else
 			{
-				m_iStageProgress = m_iProcessedFiles * 1000 / iTotalFiles;
+				m_iStageProgress = iProcessedFiles * 1000 / iTotalFiles;
 			}
 		}
 		else
@@ -1148,7 +1241,7 @@ void ParChecker::signal_progress(double progress)
 		}
 	}
 
-	debug("Current-progres: %i, Total-progress: %i", m_iFileProgress, m_iStageProgress);
+	debug("Current-progress: %i, Total-progress: %i", m_iFileProgress, m_iStageProgress);
 
 	UpdateProgress();
 }
@@ -1190,6 +1283,11 @@ void ParChecker::signal_done(std::string str, int available, int total)
 				PrintMessage(Message::mkWarning, "File %s with %i block(s) is missing%s",
 					szFilename, total, bIgnore ? ", ignoring" : "");
 			}
+
+			if (!IsProcessedFile(szFilename))
+			{
+				m_ProcessedFiles.push_back(strdup(szFilename));
+			}
 		}
 	}
 }
@@ -1204,12 +1302,14 @@ void ParChecker::CheckEmptyFiles()
 	for (std::vector<Par2RepairerSourceFile*>::iterator it = ((Repairer*)m_pRepairer)->sourcefiles.begin();
 		 it != ((Repairer*)m_pRepairer)->sourcefiles.end(); it++)
 	{
-		Par2RepairerSourceFile *sourcefile = *it;
+		Par2RepairerSourceFile* sourcefile = *it;
 
 		if (sourcefile && sourcefile->GetDescriptionPacket())
 		{
-			const char* szFilename = sourcefile->GetDescriptionPacket()->FileName().c_str();
-			if (!IsProcessedFile(szFilename))
+			// GetDescriptionPacket()->FileName() returns a temp string object, which we need to hold for a while
+			std::string filename = sourcefile->GetDescriptionPacket()->FileName();
+			const char* szFilename = filename.c_str();
+			if (!Util::EmptyStr(szFilename) && !IsProcessedFile(szFilename))
 			{
 				bool bIgnore = Util::MatchFileExt(szFilename, g_pOptions->GetParIgnoreExt(), ",;") ||
 					Util::MatchFileExt(szFilename, g_pOptions->GetExtCleanupDisk(), ",;");
@@ -1424,6 +1524,7 @@ ParChecker::EFileStatus ParChecker::VerifyDataFile(void* pDiskfile, void* pSourc
 		}
 	}
 
+	m_iQuickFiles++;
 	PrintMessage(Message::mkDetail, "Quickly verified %s file %s",
 		eFileStatus == fsSuccess ? "good" : "damaged", Util::BaseFileName(szFilename));
 
