@@ -1,7 +1,7 @@
 /*
  *  This file is part of nzbget
  *
- *  Copyright (C) 2007-2014 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2007-2015 Andrey Prygunkov <hugbug@users.sourceforge.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,8 +17,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 1139 $
- * $Date: 2014-10-09 23:11:42 +0200 (Thu, 09 Oct 2014) $
+ * $Revision$
+ * $Date$
  *
  */
 
@@ -86,6 +86,24 @@ public:
 							~SegmentList();
 	};
 
+	class DupeSource
+	{
+	private:
+		int					m_iID;
+		char*				m_szDirectory;
+		int					m_iUsedBlocks;
+
+	public:
+							DupeSource(int iID, const char* szDirectory);
+							~DupeSource();
+		int					GetID() { return m_iID; }
+		const char*			GetDirectory() { return m_szDirectory; }
+		int					GetUsedBlocks() { return m_iUsedBlocks; }
+		void				SetUsedBlocks(int iUsedBlocks) { m_iUsedBlocks = iUsedBlocks; }
+	};
+
+	typedef std::deque<DupeSource*>	DupeSourceList;
+
 	typedef std::deque<char*>		FileList;
 	typedef std::deque<void*>		SourceList;
 	typedef std::vector<bool>		ValidBlocks;
@@ -109,6 +127,7 @@ private:
 	int					m_iProcessedFiles;
 	int					m_iFilesToRepair;
 	int					m_iExtraFiles;
+	int					m_iQuickFiles;
 	bool				m_bVerifyingExtraFiles;
 	char*				m_szProgressLabel;
 	int					m_iFileProgress;
@@ -119,6 +138,8 @@ private:
 	bool				m_bHasDamagedFiles;
 	bool				m_bParQuick;
 	bool				m_bForceRepair;
+	bool				m_bParFull;
+	DupeSourceList		m_DupeSources;
 
 	void				Cleanup();
 	EStatus				RunParCheckAll();
@@ -129,12 +150,14 @@ private:
 	bool				LoadMorePars();
 	bool				AddSplittedFragments();
 	bool				AddMissingFiles();
+	bool				AddDupeFiles();
+	bool				AddExtraFiles(bool bOnlyMissing, bool bExternalDir, const char* szDirectory);
 	bool				IsProcessedFile(const char* szFilename);
 	void				WriteBrokenLog(EStatus eStatus);
 	void				SaveSourceList();
 	void				DeleteLeftovers();
 	void				signal_filename(std::string str);
-	void				signal_progress(double progress);
+	void				signal_progress(int progress);
 	void				signal_done(std::string str, int available, int total);
 	// declared as void* to prevent the including of libpar2-headers into this header-file
 	// DiskFile* pDiskfile, Par2RepairerSourceFile* pSourcefile
@@ -144,6 +167,7 @@ private:
 	bool				SmartCalcFileRangeCrc(FILE* pFile, long long lStart, long long lEnd, SegmentList* pSegments,
 							unsigned long* pDownloadCrc);
 	bool				DumbCalcFileRangeCrc(FILE* pFile, long long lStart, long long lEnd, unsigned long* pDownloadCrc);
+	void				CheckEmptyFiles();
 
 protected:
 	/**
@@ -158,6 +182,8 @@ protected:
 	virtual void		RegisterParredFile(const char* szFilename) {}
 	virtual bool		IsParredFile(const char* szFilename) { return false; }
 	virtual EFileStatus	FindFileCrc(const char* szFilename, unsigned long* lCrc, SegmentList* pSegments) { return fsUnknown; }
+	virtual void		RequestDupeSources(DupeSourceList* pDupeSourceList) {}
+	virtual void		StatDupeSources(DupeSourceList* pDupeSourceList) {}
 	EStage				GetStage() { return m_eStage; }
 	const char*			GetProgressLabel() { return m_szProgressLabel; }
 	int					GetFileProgress() { return m_iFileProgress; }
@@ -176,6 +202,8 @@ public:
 	bool				GetParQuick() { return m_bParQuick; }
 	void				SetForceRepair(bool bForceRepair) { m_bForceRepair = bForceRepair; }
 	bool				GetForceRepair() { return m_bForceRepair; }
+	void				SetParFull(bool bParFull) { m_bParFull = bParFull; }
+	bool				GetParFull() { return m_bParFull; }
 	EStatus				GetStatus() { return m_eStatus; }
 	void				AddParFile(const char* szParFilename);
 	void				QueueChanged();
