@@ -17,8 +17,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 1221 $
- * $Date: 2015-02-26 15:57:38 -0500 (Thu, 26 Feb 2015) $
+ * $Revision$
+ * $Date$
  *
  */
 
@@ -44,14 +44,13 @@
 
 #include "nzbget.h"
 #include "ParCoordinator.h"
+#include "DupeCoordinator.h"
+#include "ParParser.h"
 #include "Options.h"
 #include "DiskState.h"
 #include "Log.h"
 #include "Util.h"
 
-extern Options* g_pOptions;
-extern DiskState* g_pDiskState;
-
 #ifndef DISABLE_PARCHECK
 bool ParCoordinator::PostParChecker::RequestMorePars(int iBlockNeeded, int* pBlockFound)
 {
@@ -146,6 +145,69 @@ ParChecker::EFileStatus ParCoordinator::PostParChecker::FindFileCrc(const char*
 		ParChecker::fsUnknown;
 }
 
+void ParCoordinator::PostParChecker::RequestDupeSources(DupeSourceList* pDupeSourceList)
+{
+	DownloadQueue* pDownloadQueue = DownloadQueue::Lock();
+
+	NZBList dupeList;
+	g_pDupeCoordinator->ListHistoryDupes(pDownloadQueue, m_pPostInfo->GetNZBInfo(), &dupeList);
+
+	if (!dupeList.empty())
+	{
+		PostDupeMatcher dupeMatcher(m_pPostInfo);
+		PrintMessage(Message::mkInfo, "Checking %s for dupe scan usability", m_pPostInfo->GetNZBInfo()->GetName());
+		bool bSizeComparisonPossible = dupeMatcher.Prepare();
+		for (NZBList::iterator it = dupeList.begin(); it != dupeList.end(); it++)
+		{
+			NZBInfo* pDupeNZBInfo = *it;
+			if (bSizeComparisonPossible)
+			{
+				PrintMessage(Message::mkInfo, "Checking %s for dupe scan usability", Util::BaseFileName(pDupeNZBInfo->GetDestDir()));
+			}
+			bool bUseDupe = !bSizeComparisonPossible || dupeMatcher.MatchDupeContent(pDupeNZBInfo->GetDestDir());
+			if (bUseDupe)
+			{
+				PrintMessage(Message::mkInfo, "Adding %s to dupe scan sources", Util::BaseFileName(pDupeNZBInfo->GetDestDir()));
+				pDupeSourceList->push_back(new ParChecker::DupeSource(pDupeNZBInfo->GetID(), pDupeNZBInfo->GetDestDir()));
+			}
+		}
+		if (pDupeSourceList->empty())
+		{
+			PrintMessage(Message::mkInfo, "No usable dupe scan sources found");
+		}
+	}
+
+	DownloadQueue::Unlock();
+}
+
+void ParCoordinator::PostParChecker::StatDupeSources(DupeSourceList* pDupeSourceList)
+{
+	DownloadQueue* pDownloadQueue = DownloadQueue::Lock();
+
+	int iTotalExtraParBlocks = 0;
+	for (DupeSourceList::iterator it = pDupeSourceList->begin(); it != pDupeSourceList->end(); it++)
+	{
+		DupeSource* pDupeSource = *it;
+		if (pDupeSource->GetUsedBlocks() > 0)
+		{
+			for (HistoryList::iterator it = pDownloadQueue->GetHistory()->begin(); it != pDownloadQueue->GetHistory()->end(); it++)
+			{
+				HistoryInfo* pHistoryInfo = *it;
+				if (pHistoryInfo->GetKind() == HistoryInfo::hkNzb &&
+					pHistoryInfo->GetNZBInfo()->GetID() == pDupeSource->GetID())
+				{
+					pHistoryInfo->GetNZBInfo()->SetExtraParBlocks(pHistoryInfo->GetNZBInfo()->GetExtraParBlocks() - pDupeSource->GetUsedBlocks());
+				}
+			}
+		}
+		iTotalExtraParBlocks += pDupeSource->GetUsedBlocks();
+	}
+
+	m_pPostInfo->GetNZBInfo()->SetExtraParBlocks(m_pPostInfo->GetNZBInfo()->GetExtraParBlocks() + iTotalExtraParBlocks);
+
+	DownloadQueue::Unlock();
+}
+
 void ParCoordinator::PostParRenamer::UpdateProgress()
 {
 	m_pOwner->UpdateParRenameProgress();
@@ -184,6 +246,18 @@ void ParCoordinator::PostParRenamer::RegisterRenamedFile(const char* szOldFilena
 	}
 }
 
+void ParCoordinator::PostDupeMatcher::PrintMessage(Message::EKind eKind, const char* szFormat, ...)
+{
+	char szText[1024];
+	va_list args;
+	va_start(args, szFormat);
+	vsnprintf(szText, 1024, szFormat, args);
+	va_end(args);
+	szText[1024-1] = '\0';
+
+	m_pPostInfo->GetNZBInfo()->AddMessage(eKind, szText);
+}
+
 #endif
 
 ParCoordinator::ParCoordinator()
@@ -235,111 +309,6 @@ void ParCoordinator::PausePars(DownloadQueue* pDownloadQueue, NZBInfo* pNZBInfo)
 		DownloadQueue::eaGroupPauseExtraPars, 0, NULL);
 }
 
-bool ParCoordinator::FindMainPars(const char* szPath, ParFileList* pFileList)
-{
-	if (pFileList)
-	{
-		pFileList->clear();
-	}
-
-	DirBrowser dir(szPath);
-	while (const char* filename = dir.Next())
-	{
-		int iBaseLen = 0;
-		if (ParseParFilename(filename, &iBaseLen, NULL))
-		{
-			if (!pFileList)
-			{
-				return true;
-			}
-
-			// check if the base file already added to list
-			bool exists = false;
-			for (ParFileList::iterator it = pFileList->begin(); it != pFileList->end(); it++)
-			{
-				const char* filename2 = *it;
-				exists = SameParCollection(filename, filename2);
-				if (exists)
-				{
-					break;
-				}
-			}
-			if (!exists)
-			{
-				pFileList->push_back(strdup(filename));
-			}
-		}
-	}
-	return pFileList && !pFileList->empty();
-}
-
-bool ParCoordinator::SameParCollection(const char* szFilename1, const char* szFilename2)
-{
-	int iBaseLen1 = 0, iBaseLen2 = 0;
-	return ParseParFilename(szFilename1, &iBaseLen1, NULL) &&
-		ParseParFilename(szFilename2, &iBaseLen2, NULL) &&
-		iBaseLen1 == iBaseLen2 &&
-		!strncasecmp(szFilename1, szFilename2, iBaseLen1);
-}
-
-bool ParCoordinator::ParseParFilename(const char* szParFilename, int* iBaseNameLen, int* iBlocks)
-{
-	char szFilename[1024];
-	strncpy(szFilename, szParFilename, 1024);
-	szFilename[1024-1] = '\0';
-	for (char* p = szFilename; *p; p++) *p = tolower(*p); // convert string to lowercase
-
-	int iLen = strlen(szFilename);
-	if (iLen < 6)
-	{
-		return false;
-	}
-
-	// find last occurence of ".par2" and trim filename after it
-	char* szEnd = szFilename;
-	while (char* p = strstr(szEnd, ".par2")) szEnd = p + 5;
-	*szEnd = '\0';
-
-	iLen = strlen(szFilename);
-	if (iLen < 6)
-	{
-		return false;
-	}
-
-	if (strcasecmp(szFilename + iLen - 5, ".par2"))
-	{
-		return false;
-	}
-	*(szFilename + iLen - 5) = '\0';
-
-	int blockcnt = 0;
-	char* p = strrchr(szFilename, '.');
-	if (p && !strncasecmp(p, ".vol", 4))
-	{
-		char* b = strchr(p, '+');
-		if (!b)
-		{
-			b = strchr(p, '-');
-		}
-		if (b)
-		{
-			blockcnt = atoi(b+1);
-			*p = '\0';
-		}
-	}
-
-	if (iBaseNameLen)
-	{
-		*iBaseNameLen = strlen(szFilename);
-	}
-	if (iBlocks)
-	{
-		*iBlocks = blockcnt;
-	}
-	
-	return true;
-}
-
 #ifndef DISABLE_PARCHECK
 
 /**
@@ -471,7 +440,9 @@ void ParCoordinator::ParCheckCompleted()
 /**
 * Unpause par2-files
 * returns true, if the files with required number of blocks were unpaused,
-* or false if there are no more files in queue for this collection or not enough blocks
+* or false if there are no more files in queue for this collection or not enough blocks.
+* special case: returns true if there are any unpaused par2-files in the queue regardless
+* of the amount of blocks; this is to keep par-checker wait for download completion.
 */
 bool ParCoordinator::RequestMorePars(NZBInfo* pNZBInfo, const char* szParFilename, int iBlockNeeded, int* pBlockFound)
 {
@@ -549,6 +520,17 @@ bool ParCoordinator::RequestMorePars(NZBInfo* pNZBInfo, const char* szParFilenam
 		}
 	}
 
+	bool bHasUnpausedParFiles = false;
+	for (FileList::iterator it = pNZBInfo->GetFileList()->begin(); it != pNZBInfo->GetFileList()->end(); it++)
+	{
+		FileInfo* pFileInfo = *it;
+		if (pFileInfo->GetParFile() && !pFileInfo->GetPaused())
+		{
+			bHasUnpausedParFiles = true;
+			break;
+		}
+	}
+
 	DownloadQueue::Unlock();
 
 	if (pBlockFound)
@@ -562,7 +544,7 @@ bool ParCoordinator::RequestMorePars(NZBInfo* pNZBInfo, const char* szParFilenam
 	}
 	blocks.clear();
 
-	bool bOK = iBlockNeeded <= 0;
+	bool bOK = iBlockNeeded <= 0 || bHasUnpausedParFiles;
 
 	return bOK;
 }
@@ -576,7 +558,7 @@ void ParCoordinator::FindPars(DownloadQueue* pDownloadQueue, NZBInfo* pNZBInfo,
 	char* szBaseParFilename = Util::BaseFileName(szParFilename);
 	char szMainBaseFilename[1024];
 	int iMainBaseLen = 0;
-	if (!ParseParFilename(szBaseParFilename, &iMainBaseLen, NULL))
+	if (!ParParser::ParseParFilename(szBaseParFilename, &iMainBaseLen, NULL))
 	{
 		// should not happen
         pNZBInfo->PrintMessage(Message::mkError, "Internal error: could not parse filename %s", szBaseParFilename);
@@ -591,14 +573,14 @@ void ParCoordinator::FindPars(DownloadQueue* pDownloadQueue, NZBInfo* pNZBInfo,
 	{
 		FileInfo* pFileInfo = *it;
 		int iBlocks = 0;
-		if (ParseParFilename(pFileInfo->GetFilename(), NULL, &iBlocks) &&
+		if (ParParser::ParseParFilename(pFileInfo->GetFilename(), NULL, &iBlocks) &&
 			iBlocks > 0)
 		{
 			bool bUseFile = true;
 
 			if (bExactParName)
 			{
-				bUseFile = SameParCollection(pFileInfo->GetFilename(), Util::BaseFileName(szParFilename));
+				bUseFile = ParParser::SameParCollection(pFileInfo->GetFilename(), Util::BaseFileName(szParFilename));
 			}
 			else if (bStrictParName)
 			{
