@@ -1,7 +1,7 @@
 /*
  *  This file is part of nzbget
  *
- *  Copyright (C) 2007-2014 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2007-2015 Andrey Prygunkov <hugbug@users.sourceforge.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,8 +17,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 1139 $
- * $Date: 2014-10-09 23:11:42 +0200 (Thu, 09 Oct 2014) $
+ * $Revision$
+ * $Date$
  *
  */
 
@@ -50,20 +50,11 @@
 #include "DupeCoordinator.h"
 #include "PostScript.h"
 #include "Util.h"
-#include "Scheduler.h"
-#include "Scanner.h"
 #include "Unpack.h"
+#include "Cleanup.h"
 #include "NZBFile.h"
-#include "StatMeter.h"
 #include "QueueScript.h"
-
-extern HistoryCoordinator* g_pHistoryCoordinator;
-extern DupeCoordinator* g_pDupeCoordinator;
-extern Options* g_pOptions;
-extern Scheduler* g_pScheduler;
-extern Scanner* g_pScanner;
-extern StatMeter* g_pStatMeter;
-extern QueueScriptCoordinator* g_pQueueScriptCoordinator;
+#include "ParParser.h"
 
 PrePostProcessor::PrePostProcessor()
 {
@@ -100,54 +91,19 @@ void PrePostProcessor::Run()
 		DownloadQueue::Unlock();
 	}
 
-	g_pScheduler->FirstCheck();
-
-	int iDiskSpaceInterval = 1000;
-	int iSchedulerInterval = 1000;
-	int iHistoryInterval = 600000;
-	const int iStepMSec = 200;
-
 	while (!IsStopped())
 	{
-		// check incoming nzb directory
-		g_pScanner->Check();
-
-		if (!g_pOptions->GetPauseDownload() && 
-			g_pOptions->GetDiskSpace() > 0 && !g_pStatMeter->GetStandBy() && 
-			iDiskSpaceInterval >= 1000)
+		if (!g_pOptions->GetTempPausePostprocess())
 		{
-			// check free disk space every 1 second
-			CheckDiskSpace();
-			iDiskSpaceInterval = 0;
+			// check post-queue every 200 msec
+			CheckPostQueue();
 		}
-		iDiskSpaceInterval += iStepMSec;
-
-		// check post-queue every 200 msec
-		CheckPostQueue();
-
-		if (iSchedulerInterval >= 1000)
-		{
-			// check scheduler tasks every 1 second
-			g_pScheduler->IntervalCheck();
-			iSchedulerInterval = 0;
-		}
-		iSchedulerInterval += iStepMSec;
-
-		if (iHistoryInterval >= 600000)
-		{
-			// check history (remove old entries) every 10 minutes
-			g_pHistoryCoordinator->IntervalCheck();
-			iHistoryInterval = 0;
-		}
-		iHistoryInterval += iStepMSec;
 
 		Util::SetStandByMode(!m_pCurJob);
 
-		usleep(iStepMSec * 1000);
+		usleep(200 * 1000);
 	}
 
-	g_pHistoryCoordinator->Cleanup();
-
 	debug("Exiting PrePostProcessor-loop");
 }
 
@@ -198,7 +154,8 @@ void PrePostProcessor::DownloadQueueUpdate(Subject* Caller, void* Aspect)
 	{
 		// the deleting of nzbs is usually handled via eaFileDeleted-event, but when deleting nzb without
 		// any files left the eaFileDeleted-event is not fired and we need to process eaNzbDeleted-event instead
-		info("Collection %s deleted from queue", pQueueAspect->pNZBInfo->GetName());
+		pQueueAspect->pNZBInfo->PrintMessage(Message::mkInfo,
+			"Collection %s deleted from queue", pQueueAspect->pNZBInfo->GetName());
 		NZBDeleted(pQueueAspect->pDownloadQueue, pQueueAspect->pNZBInfo);
 	}
 	else if ((pQueueAspect->eAction == DownloadQueue::eaFileCompleted ||
@@ -222,7 +179,8 @@ void PrePostProcessor::DownloadQueueUpdate(Subject* Caller, void* Aspect)
 				 IsNZBFileCompleted(pQueueAspect->pNZBInfo, false, true))) &&
 				 pQueueAspect->pFileInfo->GetNZBInfo()->GetDeleteStatus() != NZBInfo::dsHealth)
 			{
-				info("Collection %s completely downloaded", pQueueAspect->pNZBInfo->GetName());
+				pQueueAspect->pNZBInfo->PrintMessage(Message::mkInfo,
+					"Collection %s completely downloaded", pQueueAspect->pNZBInfo->GetName());
 				g_pQueueScriptCoordinator->EnqueueScript(pQueueAspect->pNZBInfo, QueueScriptCoordinator::qeNzbDownloaded);
 				NZBDownloaded(pQueueAspect->pDownloadQueue, pQueueAspect->pNZBInfo);
 			}
@@ -232,7 +190,8 @@ void PrePostProcessor::DownloadQueueUpdate(Subject* Caller, void* Aspect)
 				!pQueueAspect->pNZBInfo->GetParCleanup() &&
 				IsNZBFileCompleted(pQueueAspect->pNZBInfo, false, true))
 			{
-				info("Collection %s deleted from queue", pQueueAspect->pNZBInfo->GetName());
+				pQueueAspect->pNZBInfo->PrintMessage(Message::mkInfo,
+					"Collection %s deleted from queue", pQueueAspect->pNZBInfo->GetName());
 				NZBDeleted(pQueueAspect->pDownloadQueue, pQueueAspect->pNZBInfo);
 			}
 		}
@@ -254,8 +213,10 @@ void PrePostProcessor::NZBAdded(DownloadQueue* pDownloadQueue, NZBInfo* pNZBInfo
 		m_ParCoordinator.PausePars(pDownloadQueue, pNZBInfo);
 	}
 
-	if (g_pOptions->GetDupeCheck() && pNZBInfo->GetDupeMode() != dmForce &&
-		pNZBInfo->GetDeleteStatus() == NZBInfo::dsDupe)
+	if (pNZBInfo->GetDeleteStatus() == NZBInfo::dsDupe ||
+		pNZBInfo->GetDeleteStatus() == NZBInfo::dsCopy ||
+		pNZBInfo->GetDeleteStatus() == NZBInfo::dsGood ||
+		pNZBInfo->GetDeleteStatus() == NZBInfo::dsScan)
 	{
 		NZBCompleted(pDownloadQueue, pNZBInfo, false);
 	}
@@ -267,9 +228,15 @@ void PrePostProcessor::NZBAdded(DownloadQueue* pDownloadQueue, NZBInfo* pNZBInfo
 
 void PrePostProcessor::NZBDownloaded(DownloadQueue* pDownloadQueue, NZBInfo* pNZBInfo)
 {
+	if (pNZBInfo->GetDeleteStatus() == NZBInfo::dsHealth ||
+		pNZBInfo->GetDeleteStatus() == NZBInfo::dsBad)
+	{
+		g_pQueueScriptCoordinator->EnqueueScript(pNZBInfo, QueueScriptCoordinator::qeNzbDeleted);
+	}
+
 	if (!pNZBInfo->GetPostInfo() && g_pOptions->GetDecode())
 	{
-		info("Queueing %s for post-processing", pNZBInfo->GetName());
+		pNZBInfo->PrintMessage(Message::mkInfo, "Queueing %s for post-processing", pNZBInfo->GetName());
 
 		pNZBInfo->EnterPostProcess();
 		m_iJobCount++;
@@ -329,12 +296,21 @@ void PrePostProcessor::NZBCompleted(DownloadQueue* pDownloadQueue, NZBInfo* pNZB
 	if (g_pOptions->GetDupeCheck() && pNZBInfo->GetDupeMode() != dmForce &&
 		(pNZBInfo->GetDeleteStatus() == NZBInfo::dsNone ||
 		 pNZBInfo->GetDeleteStatus() == NZBInfo::dsHealth ||
-		 pNZBInfo->GetDeleteStatus() == NZBInfo::dsBad))
+		 pNZBInfo->GetDeleteStatus() == NZBInfo::dsBad ||
+		 pNZBInfo->GetDeleteStatus() == NZBInfo::dsScan))
 	{
 		g_pDupeCoordinator->NZBCompleted(pDownloadQueue, pNZBInfo);
 		bNeedSave = true;
 	}
 
+	if (pNZBInfo->GetDeleteStatus() > NZBInfo::dsNone &&
+		pNZBInfo->GetDeleteStatus() != NZBInfo::dsHealth &&
+		pNZBInfo->GetDeleteStatus() != NZBInfo::dsBad)
+		// nzbs deleted by health check or marked as bad are processed as downloaded with failure status
+	{
+		g_pQueueScriptCoordinator->EnqueueScript(pNZBInfo, QueueScriptCoordinator::qeNzbDeleted);
+	}
+
 	if (!bAddToHistory)
 	{
 		g_pHistoryCoordinator->DeleteDiskFiles(pNZBInfo);
@@ -393,26 +369,6 @@ void PrePostProcessor::DeleteCleanup(NZBInfo* pNZBInfo)
 	}
 }
 
-void PrePostProcessor::CheckDiskSpace()
-{
-	long long lFreeSpace = Util::FreeDiskSize(g_pOptions->GetDestDir());
-	if (lFreeSpace > -1 && lFreeSpace / 1024 / 1024 < g_pOptions->GetDiskSpace())
-	{
-		warn("Low disk space on %s. Pausing download", g_pOptions->GetDestDir());
-		g_pOptions->SetPauseDownload(true);
-	}
-
-	if (!Util::EmptyStr(g_pOptions->GetInterDir()))
-	{
-		lFreeSpace = Util::FreeDiskSize(g_pOptions->GetInterDir());
-		if (lFreeSpace > -1 && lFreeSpace / 1024 / 1024 < g_pOptions->GetDiskSpace())
-		{
-			warn("Low disk space on %s. Pausing download", g_pOptions->GetInterDir());
-			g_pOptions->SetPauseDownload(true);
-		}
-	}
-}
-
 void PrePostProcessor::CheckPostQueue()
 {
 	DownloadQueue* pDownloadQueue = DownloadQueue::Lock();
@@ -449,13 +405,15 @@ void PrePostProcessor::CheckPostQueue()
 
 				if (!pPostInfo->GetNZBInfo()->GetFileList()->empty())
 				{
-					info("Downloading all remaining files for manual par-check for %s", pPostInfo->GetNZBInfo()->GetName());
+					pPostInfo->GetNZBInfo()->PrintMessage(Message::mkInfo,
+						"Downloading all remaining files for manual par-check for %s", pPostInfo->GetNZBInfo()->GetName());
 					pDownloadQueue->EditEntry(pPostInfo->GetNZBInfo()->GetID(), DownloadQueue::eaGroupResume, 0, NULL);
 					pPostInfo->SetStage(PostInfo::ptFinished);
 				}
 				else
 				{
-					info("There are no par-files remain for download for %s", pPostInfo->GetNZBInfo()->GetName());
+					pPostInfo->GetNZBInfo()->PrintMessage(Message::mkInfo,
+						"There are no par-files remain for download for %s", pPostInfo->GetNZBInfo()->GetName());
 					pPostInfo->SetStage(PostInfo::ptQueued);
 				}
 			}
@@ -495,7 +453,7 @@ NZBInfo* PrePostProcessor::GetNextJob(DownloadQueue* pDownloadQueue)
 	for (NZBList::iterator it = pDownloadQueue->GetQueue()->begin(); it != pDownloadQueue->GetQueue()->end(); it++)
 	{
 		NZBInfo* pNZBInfo1 = *it;
-		if (pNZBInfo1->GetPostInfo() && !g_pQueueScriptCoordinator->HasJob(pNZBInfo1->GetID()) &&
+		if (pNZBInfo1->GetPostInfo() && !g_pQueueScriptCoordinator->HasJob(pNZBInfo1->GetID(), NULL) &&
 			(!pNZBInfo || pNZBInfo1->GetPriority() > pNZBInfo->GetPriority()) &&
 			(!g_pOptions->GetPausePostProcess() || pNZBInfo1->GetForcePriority()))
 		{
@@ -557,14 +515,15 @@ void PrePostProcessor::StartJob(DownloadQueue* pDownloadQueue, PostInfo* pPostIn
 	else if (pPostInfo->GetNZBInfo()->GetParStatus() == NZBInfo::psNone &&
 		pPostInfo->GetNZBInfo()->GetDeleteStatus() == NZBInfo::dsNone)
 	{
-		if (m_ParCoordinator.FindMainPars(pPostInfo->GetNZBInfo()->GetDestDir(), NULL))
+		if (ParParser::FindMainPars(pPostInfo->GetNZBInfo()->GetDestDir(), NULL))
 		{
 			UpdatePauseState(g_pOptions->GetParPauseQueue(), "par-check");
 			m_ParCoordinator.StartParCheckJob(pPostInfo);
 		}
 		else
 		{
-			info("Nothing to par-check for %s", pPostInfo->GetNZBInfo()->GetName());
+			pPostInfo->GetNZBInfo()->PrintMessage(Message::mkInfo,
+				"Nothing to par-check for %s", pPostInfo->GetNZBInfo()->GetName());
 			pPostInfo->GetNZBInfo()->SetParStatus(NZBInfo::psSkipped);
 			pPostInfo->SetWorking(false);
 			pPostInfo->SetStage(PostInfo::ptQueued);
@@ -572,20 +531,27 @@ void PrePostProcessor::StartJob(DownloadQueue* pDownloadQueue, PostInfo* pPostIn
 		return;
 	}
 	else if (pPostInfo->GetNZBInfo()->GetParStatus() == NZBInfo::psSkipped &&
-		pPostInfo->GetNZBInfo()->CalcHealth() < pPostInfo->GetNZBInfo()->CalcCriticalHealth(false) &&
-		pPostInfo->GetNZBInfo()->CalcCriticalHealth(false) < 1000 &&
-		m_ParCoordinator.FindMainPars(pPostInfo->GetNZBInfo()->GetDestDir(), NULL))
+		((g_pOptions->GetParScan() != Options::psDupe &&
+		  pPostInfo->GetNZBInfo()->CalcHealth() < pPostInfo->GetNZBInfo()->CalcCriticalHealth(false) &&
+		  pPostInfo->GetNZBInfo()->CalcCriticalHealth(false) < 1000) ||
+		  pPostInfo->GetNZBInfo()->CalcHealth() == 0) &&
+		ParParser::FindMainPars(pPostInfo->GetNZBInfo()->GetDestDir(), NULL))
 	{
-		warn("Skipping par-check for %s due to health %.1f%% below critical %.1f%%", pPostInfo->GetNZBInfo()->GetName(),
+		pPostInfo->GetNZBInfo()->PrintMessage(Message::mkWarning,
+			pPostInfo->GetNZBInfo()->CalcHealth() == 0 ?
+				"Skipping par-check for %s due to health 0%%" :
+				"Skipping par-check for %s due to health %.1f%% below critical %.1f%%",
+			pPostInfo->GetNZBInfo()->GetName(),
 			pPostInfo->GetNZBInfo()->CalcHealth() / 10.0, pPostInfo->GetNZBInfo()->CalcCriticalHealth(false) / 10.0);
 		pPostInfo->GetNZBInfo()->SetParStatus(NZBInfo::psFailure);
 		return;
 	}
 	else if (pPostInfo->GetNZBInfo()->GetParStatus() == NZBInfo::psSkipped &&
 		pPostInfo->GetNZBInfo()->GetFailedSize() - pPostInfo->GetNZBInfo()->GetParFailedSize() > 0 &&
-		m_ParCoordinator.FindMainPars(pPostInfo->GetNZBInfo()->GetDestDir(), NULL))
+		ParParser::FindMainPars(pPostInfo->GetNZBInfo()->GetDestDir(), NULL))
 	{
-		info("Collection %s with health %.1f%% needs par-check",
+		pPostInfo->GetNZBInfo()->PrintMessage(Message::mkInfo,
+			"Collection %s with health %.1f%% needs par-check",
 			pPostInfo->GetNZBInfo()->GetName(), pPostInfo->GetNZBInfo()->CalcHealth() / 10.0);
 		pPostInfo->SetRequestParCheck(true);
 		return;
@@ -603,13 +569,18 @@ void PrePostProcessor::StartJob(DownloadQueue* pDownloadQueue, PostInfo* pPostIn
 
 	bool bCleanup = !bUnpack &&
 		pPostInfo->GetNZBInfo()->GetCleanupStatus() == NZBInfo::csNone &&
+		!Util::EmptyStr(g_pOptions->GetExtCleanupDisk()) &&
 		((pPostInfo->GetNZBInfo()->GetParStatus() == NZBInfo::psSuccess &&
 		  pPostInfo->GetNZBInfo()->GetUnpackStatus() != NZBInfo::usFailure &&
 		  pPostInfo->GetNZBInfo()->GetUnpackStatus() != NZBInfo::usSpace &&
 		  pPostInfo->GetNZBInfo()->GetUnpackStatus() != NZBInfo::usPassword) ||
 		 (pPostInfo->GetNZBInfo()->GetUnpackStatus() == NZBInfo::usSuccess &&
-		  pPostInfo->GetNZBInfo()->GetParStatus() != NZBInfo::psFailure)) &&
-		!Util::EmptyStr(g_pOptions->GetExtCleanupDisk());
+		  pPostInfo->GetNZBInfo()->GetParStatus() != NZBInfo::psFailure) ||
+		 ((pPostInfo->GetNZBInfo()->GetUnpackStatus() == NZBInfo::usNone || 
+		   pPostInfo->GetNZBInfo()->GetUnpackStatus() == NZBInfo::usSkipped) &&
+		  (pPostInfo->GetNZBInfo()->GetParStatus() == NZBInfo::psNone ||
+		   pPostInfo->GetNZBInfo()->GetParStatus() == NZBInfo::psSkipped) &&
+		  pPostInfo->GetNZBInfo()->CalcHealth() == 1000));
 
 	bool bMoveInter = !bUnpack &&
 		pPostInfo->GetNZBInfo()->GetMoveStatus() == NZBInfo::msNone &&
@@ -626,7 +597,8 @@ void PrePostProcessor::StartJob(DownloadQueue* pDownloadQueue, PostInfo* pPostIn
 
 	if (bUnpack && bParFailed)
 	{
-		warn("Skipping unpack for %s due to %s", pPostInfo->GetNZBInfo()->GetName(),
+		pPostInfo->GetNZBInfo()->PrintMessage(Message::mkWarning,
+			"Skipping unpack for %s due to %s", pPostInfo->GetNZBInfo()->GetName(),
 			pPostInfo->GetNZBInfo()->GetParStatus() == NZBInfo::psManual ? "required par-repair" : "par-failure");
 		pPostInfo->GetNZBInfo()->SetUnpackStatus(NZBInfo::usSkipped);
 		bUnpack = false;
@@ -703,7 +675,7 @@ void PrePostProcessor::JobCompleted(DownloadQueue* pDownloadQueue, PostInfo* pPo
 			 pNZBInfo->CalcCriticalHealth(false) < 1000);
 		if (g_pOptions->GetParCleanupQueue() && bCanCleanupQueue && !pNZBInfo->GetFileList()->empty())
 		{
-			info("Cleaning up download queue for %s", pNZBInfo->GetName());
+			pNZBInfo->PrintMessage(Message::mkInfo, "Cleaning up download queue for %s", pNZBInfo->GetName());
 			pNZBInfo->SetParCleanup(true);
 			pDownloadQueue->EditEntry(pNZBInfo->GetID(), DownloadQueue::eaGroupDelete, 0, NULL);
 		}
@@ -809,7 +781,8 @@ bool PrePostProcessor::PostQueueDelete(DownloadQueue* pDownloadQueue, IDList* pI
 			{
 				if (pPostInfo->GetWorking())
 				{
-					info("Deleting active post-job %s", pPostInfo->GetNZBInfo()->GetName());
+					pPostInfo->GetNZBInfo()->PrintMessage(Message::mkInfo,
+						"Deleting active post-job %s", pPostInfo->GetNZBInfo()->GetName());
 					pPostInfo->SetDeleted(true);
 #ifndef DISABLE_PARCHECK
 					if (PostInfo::ptLoadingPars <= pPostInfo->GetStage() && pPostInfo->GetStage() <= PostInfo::ptRenaming)
@@ -834,7 +807,8 @@ bool PrePostProcessor::PostQueueDelete(DownloadQueue* pDownloadQueue, IDList* pI
 				}
 				else
 				{
-					info("Deleting queued post-job %s", pPostInfo->GetNZBInfo()->GetName());
+					pPostInfo->GetNZBInfo()->PrintMessage(Message::mkInfo,
+						"Deleting queued post-job %s", pPostInfo->GetNZBInfo()->GetName());
 					JobCompleted(pDownloadQueue, pPostInfo);
 					bOK = true;
 				}
