@@ -17,8 +17,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 1245 $
- * $Date: 2015-03-26 18:28:30 -0400 (Thu, 26 Mar 2015) $
+ * $Revision$
+ * $Date$
  *
  */
 
@@ -50,12 +50,10 @@
 #include "Log.h"
 #include "Util.h"
 
-extern Options* g_pOptions;
-
 static const char* FORMATVERSION_SIGNATURE = "nzbget diskstate file version ";
 
-#ifdef WIN32
-// Windows doesn't have standard "vsscanf"
+#if (defined(WIN32) && _MSC_VER < 1800)
+// Visual Studio prior 2013 doesn't have standard "vsscanf"
 // Hack from http://stackoverflow.com/questions/2457331/replacement-for-vsscanf-on-msvc
 int vsscanf(const char *s, const char *fmt, va_list ap)
 {
@@ -69,6 +67,170 @@ int vsscanf(const char *s, const char *fmt, va_list ap)
 }
 #endif
 
+/* Parse signature and return format version number
+*/
+int ParseFormatVersion(const char* szFormatSignature)
+{
+	if (strncmp(szFormatSignature, FORMATVERSION_SIGNATURE, strlen(FORMATVERSION_SIGNATURE)))
+	{
+		return 0;
+	}
+
+	return atoi(szFormatSignature + strlen(FORMATVERSION_SIGNATURE));
+}
+
+
+class StateFile
+{
+private:
+	char			m_szDestFilename[1024];
+	char			m_szTempFilename[1024];
+	int				m_iFormatVersion;
+	int				m_iFileVersion;
+	FILE*			m_pFile;
+
+public:
+					StateFile(const char* szFilename, int iFormatVersion);
+					~StateFile();
+	void			Discard();
+	bool			FileExists();
+	FILE*			BeginWriteTransaction();
+	bool			FinishWriteTransaction();
+	FILE*			BeginReadTransaction();
+	int				GetFileVersion() { return m_iFileVersion; }
+	const char*		GetDestFilename() { return m_szDestFilename; }
+};
+
+
+StateFile::StateFile(const char* szFilename, int iFormatVersion)
+{
+	m_pFile = NULL;
+
+	m_iFormatVersion = iFormatVersion;
+
+	snprintf(m_szDestFilename, 1024, "%s%s", g_pOptions->GetQueueDir(), szFilename);
+	m_szDestFilename[1024-1] = '\0';
+
+	snprintf(m_szTempFilename, 1024, "%s%s.new", g_pOptions->GetQueueDir(), szFilename);
+	m_szTempFilename[1024-1] = '\0';
+}
+
+StateFile::~StateFile()
+{
+	if (m_pFile)
+	{
+		fclose(m_pFile);
+	}
+}
+
+void StateFile::Discard()
+{
+	remove(m_szDestFilename);
+}
+
+bool StateFile::FileExists()
+{
+	return Util::FileExists(m_szDestFilename) || Util::FileExists(m_szTempFilename);
+}
+
+FILE* StateFile::BeginWriteTransaction()
+{
+	m_pFile = fopen(m_szTempFilename, FOPEN_WB);
+
+	if (!m_pFile)
+	{
+		char szErrBuf[256];
+		Util::GetLastErrorMessage(szErrBuf, sizeof(szErrBuf));
+		error("Error saving diskstate: Could not create file %s: %s", m_szTempFilename, szErrBuf);
+		return NULL;
+	}
+
+	fprintf(m_pFile, "%s%i\n", FORMATVERSION_SIGNATURE, m_iFormatVersion);
+
+	return m_pFile;
+}
+
+bool StateFile::FinishWriteTransaction()
+{
+	char szErrBuf[256];
+
+	// flush file content before renaming
+	if (g_pOptions->GetFlushQueue())
+	{
+		debug("Flushing data for file %s", Util::BaseFileName(m_szTempFilename));
+		fflush(m_pFile);
+		if (!Util::FlushFileBuffers(fileno(m_pFile), szErrBuf, sizeof(szErrBuf)))
+		{
+			warn("Could not flush file %s into disk: %s", m_szTempFilename, szErrBuf);
+		}
+	}
+
+	fclose(m_pFile);
+	m_pFile = NULL;
+
+	// now rename to dest file name
+	remove(m_szDestFilename);
+	if (rename(m_szTempFilename, m_szDestFilename))
+	{
+		Util::GetLastErrorMessage(szErrBuf, sizeof(szErrBuf));
+		error("Error saving diskstate: Could not rename file %s to %s: %s",
+			m_szTempFilename, m_szDestFilename, szErrBuf);
+		return false;
+	}
+
+	// flush directory buffer after renaming
+	if (g_pOptions->GetFlushQueue())
+	{
+		debug("Flushing directory for file %s", Util::BaseFileName(m_szDestFilename));
+		if (!Util::FlushDirBuffers(m_szDestFilename, szErrBuf, sizeof(szErrBuf)))
+		{
+			warn("Could not flush directory buffers for file %s into disk: %s", m_szDestFilename, szErrBuf);
+		}
+	}
+
+	return true;
+}
+
+FILE* StateFile::BeginReadTransaction()
+{
+	if (!Util::FileExists(m_szDestFilename) && Util::FileExists(m_szTempFilename))
+	{
+		// disaster recovery: temp-file exists but the dest-file doesn't
+		warn("Restoring diskstate file %s from %s", Util::BaseFileName(m_szDestFilename), Util::BaseFileName(m_szTempFilename));
+		if (rename(m_szTempFilename, m_szDestFilename))
+		{
+			char szErrBuf[256];
+			Util::GetLastErrorMessage(szErrBuf, sizeof(szErrBuf));
+			error("Error restoring diskstate: Could not rename file %s to %s: %s",
+				m_szTempFilename, m_szDestFilename, szErrBuf);
+			return NULL;
+		}
+	}
+
+	m_pFile = fopen(m_szDestFilename, FOPEN_RB);
+
+	if (!m_pFile)
+	{
+		char szErrBuf[256];
+		Util::GetLastErrorMessage(szErrBuf, sizeof(szErrBuf));
+		error("Error reading diskstate: could not open file %s: %s", m_szDestFilename, szErrBuf);
+		return NULL;
+	}
+
+	char FileSignatur[128];
+	fgets(FileSignatur, sizeof(FileSignatur), m_pFile);
+	m_iFileVersion = ParseFormatVersion(FileSignatur);
+	if (m_iFileVersion > m_iFormatVersion)
+	{
+		error("Could not load diskstate due to file version mismatch");
+		fclose(m_pFile);
+		m_pFile = NULL;
+		return NULL;
+	}
+
+	return m_pFile;
+}
+
 /*
  * Standard fscanf scans beoynd current line if the next line is empty.
  * This wrapper fixes that.
@@ -89,18 +251,6 @@ int DiskState::fscanf(FILE* infile, const char* Format, ...)
 	return res;
 }
 
-/* Parse signature and return format version number
-*/
-int DiskState::ParseFormatVersion(const char* szFormatSignature)
-{
-	if (strncmp(szFormatSignature, FORMATVERSION_SIGNATURE, strlen(FORMATVERSION_SIGNATURE)))
-	{
-		return 0;
-	}
-
-	return atoi(szFormatSignature + strlen(FORMATVERSION_SIGNATURE));
-}
-
 /* Save Download Queue to Disk.
  * The Disk State consists of file "queue", which contains the order of files,
  * and of one diskstate-file for each file in download queue.
@@ -116,78 +266,45 @@ bool DiskState::SaveDownloadQueue(DownloadQueue* pDownloadQueue)
 {
 	debug("Saving queue to disk");
 
-	char destFilename[1024];
-	snprintf(destFilename, 1024, "%s%s", g_pOptions->GetQueueDir(), "queue");
-	destFilename[1024-1] = '\0';
-
-	char tempFilename[1024];
-	snprintf(tempFilename, 1024, "%s%s", g_pOptions->GetQueueDir(), "queue.new");
-	tempFilename[1024-1] = '\0';
+	StateFile stateFile("queue", 55);
 
 	if (pDownloadQueue->GetQueue()->empty() && 
 		pDownloadQueue->GetHistory()->empty())
 	{
-		remove(destFilename);
+		stateFile.Discard();
 		return true;
 	}
 
-	FILE* outfile = fopen(tempFilename, FOPEN_WB);
-
+	FILE* outfile = stateFile.BeginWriteTransaction();
 	if (!outfile)
 	{
-		error("Error saving diskstate: Could not create file %s", tempFilename);
 		return false;
 	}
 
-	fprintf(outfile, "%s%i\n", FORMATVERSION_SIGNATURE, 53);
-
 	// save nzb-infos
 	SaveNZBQueue(pDownloadQueue, outfile);
 
 	// save history
 	SaveHistory(pDownloadQueue, outfile);
 
-	fclose(outfile);
-
 	// now rename to dest file name
-	remove(destFilename);
-	if (rename(tempFilename, destFilename))
-	{
-		error("Error saving diskstate: Could not rename file %s to %s", tempFilename, destFilename);
-		return false;
-	}
-
-	return true;
+	return stateFile.FinishWriteTransaction();
 }
 
 bool DiskState::LoadDownloadQueue(DownloadQueue* pDownloadQueue, Servers* pServers)
 {
 	debug("Loading queue from disk");
 
-	bool bOK = false;
-	int iFormatVersion = 0;
-
-	char fileName[1024];
-	snprintf(fileName, 1024, "%s%s", g_pOptions->GetQueueDir(), "queue");
-	fileName[1024-1] = '\0';
-
-	FILE* infile = fopen(fileName, FOPEN_RB);
+	StateFile stateFile("queue", 55);
 
+	FILE* infile = stateFile.BeginReadTransaction();
 	if (!infile)
 	{
-		error("Error reading diskstate: could not open file %s", fileName);
 		return false;
 	}
 
-	char FileSignatur[128];
-	fgets(FileSignatur, sizeof(FileSignatur), infile);
-	iFormatVersion = ParseFormatVersion(FileSignatur);
-	if (iFormatVersion < 3 || iFormatVersion > 53)
-	{
-		error("Could not load diskstate due to file version mismatch");
-		fclose(infile);
-		return false;
-	}
+	bool bOK = false;
+	int iFormatVersion = stateFile.GetFileVersion();
 
 	NZBList nzbList(false);
 	NZBList sortList(false);
@@ -256,10 +373,9 @@ bool DiskState::LoadDownloadQueue(DownloadQueue* pDownloadQueue, Servers* pServe
 
 error:
 
-	fclose(infile);
 	if (!bOK)
 	{
-		error("Error reading diskstate for file %s", fileName);
+		error("Error reading diskstate for download queue and history");
 	}
 
 	NZBInfo::ResetGenID(true);
@@ -356,19 +472,21 @@ void DiskState::SaveNZBInfo(NZBInfo* pNZBInfo, FILE* outfile)
 	fprintf(outfile, "%s\n", pNZBInfo->GetQueuedFilename());
 	fprintf(outfile, "%s\n", pNZBInfo->GetName());
 	fprintf(outfile, "%s\n", pNZBInfo->GetCategory());
-	fprintf(outfile, "%i,%i,%i,%i\n", (int)pNZBInfo->GetPriority(), 
+	fprintf(outfile, "%i,%i,%i,%i,%i\n", (int)pNZBInfo->GetPriority(), 
 		pNZBInfo->GetPostInfo() ? (int)pNZBInfo->GetPostInfo()->GetStage() + 1 : 0,
-		(int)pNZBInfo->GetDeletePaused(), (int)pNZBInfo->GetManyDupeFiles());
+		(int)pNZBInfo->GetDeletePaused(), (int)pNZBInfo->GetManyDupeFiles(), pNZBInfo->GetFeedID());
 	fprintf(outfile, "%i,%i,%i,%i,%i,%i,%i\n", (int)pNZBInfo->GetParStatus(), (int)pNZBInfo->GetUnpackStatus(),
 		(int)pNZBInfo->GetMoveStatus(), (int)pNZBInfo->GetRenameStatus(), (int)pNZBInfo->GetDeleteStatus(),
 		(int)pNZBInfo->GetMarkStatus(), (int)pNZBInfo->GetUrlStatus());
 	fprintf(outfile, "%i,%i,%i\n", (int)pNZBInfo->GetUnpackCleanedUpDisk(), (int)pNZBInfo->GetHealthPaused(),
 		(int)pNZBInfo->GetAddUrlPaused());
-	fprintf(outfile, "%i,%i,%i\n", pNZBInfo->GetFileCount(), pNZBInfo->GetParkedFileCount(), pNZBInfo->GetMessageCount());
+	fprintf(outfile, "%i,%i,%i\n", pNZBInfo->GetFileCount(), pNZBInfo->GetParkedFileCount(),
+		pNZBInfo->GetMessageCount());
 	fprintf(outfile, "%i,%i\n", (int)pNZBInfo->GetMinTime(), (int)pNZBInfo->GetMaxTime());
-	fprintf(outfile, "%i,%i,%i\n", (int)pNZBInfo->GetParFull(), 
+	fprintf(outfile, "%i,%i,%i,%i\n", (int)pNZBInfo->GetParFull(), 
 		pNZBInfo->GetPostInfo() ? (int)pNZBInfo->GetPostInfo()->GetForceParFull() : 0,
-		pNZBInfo->GetPostInfo() ? (int)pNZBInfo->GetPostInfo()->GetForceRepair() : 0);
+		pNZBInfo->GetPostInfo() ? (int)pNZBInfo->GetPostInfo()->GetForceRepair() : 0,
+		pNZBInfo->GetExtraParBlocks());
 
 	fprintf(outfile, "%u,%u\n", pNZBInfo->GetFullContentHash(), pNZBInfo->GetFilteredContentHash());
 
@@ -501,8 +619,12 @@ bool DiskState::LoadNZBInfo(NZBInfo* pNZBInfo, Servers* pServers, FILE* infile,
 
 	if (true) // clang requires a block for goto to work
 	{
-		int iPriority = 0, iPostProcess = 0, iPostStage = 0, iDeletePaused = 0, iManyDupeFiles = 0;
-		if (iFormatVersion >= 45)
+		int iPriority = 0, iPostProcess = 0, iPostStage = 0, iDeletePaused = 0, iManyDupeFiles = 0, iFeedID = 0;
+		if (iFormatVersion >= 54)
+		{
+			if (fscanf(infile, "%i,%i,%i,%i,%i\n", &iPriority, &iPostStage, &iDeletePaused, &iManyDupeFiles, &iFeedID) != 5) goto error;
+		}
+		else if (iFormatVersion >= 45)
 		{
 			if (fscanf(infile, "%i,%i,%i,%i\n", &iPriority, &iPostStage, &iDeletePaused, &iManyDupeFiles) != 4) goto error;
 		}
@@ -530,6 +652,7 @@ bool DiskState::LoadNZBInfo(NZBInfo* pNZBInfo, Servers* pServers, FILE* infile,
 			pNZBInfo->EnterPostProcess();
 			pNZBInfo->GetPostInfo()->SetStage((PostInfo::EStage)iPostStage);
 		}
+		pNZBInfo->SetFeedID(iFeedID);
 	}
 
 	if (iFormatVersion >= 8 && iFormatVersion < 18)
@@ -549,8 +672,8 @@ bool DiskState::LoadNZBInfo(NZBInfo* pNZBInfo, Servers* pServers, FILE* infile,
 
 	if (iFormatVersion >= 18)
 	{
-		int iParStatus, iUnpackStatus, iScriptStatus, iMoveStatus = 0,
-			iRenameStatus = 0, iDeleteStatus = 0, iMarkStatus = 0, iUrlStatus = 0;
+		int iParStatus, iUnpackStatus, iScriptStatus, iMoveStatus = 0, iRenameStatus = 0,
+			iDeleteStatus = 0, iMarkStatus = 0, iUrlStatus = 0;
 		if (iFormatVersion >= 46)
 		{
 			if (fscanf(infile, "%i,%i,%i,%i,%i,%i,%i\n", &iParStatus, &iUnpackStatus, &iMoveStatus,
@@ -592,6 +715,7 @@ bool DiskState::LoadNZBInfo(NZBInfo* pNZBInfo, Servers* pServers, FILE* infile,
 		pNZBInfo->SetDeleteStatus((NZBInfo::EDeleteStatus)iDeleteStatus);
 		pNZBInfo->SetMarkStatus((NZBInfo::EMarkStatus)iMarkStatus);
 		if (pNZBInfo->GetKind() == NZBInfo::nkNzb ||
+			(NZBInfo::EUrlStatus)iUrlStatus >= NZBInfo::lsFailed ||
 			(NZBInfo::EUrlStatus)iUrlStatus >= NZBInfo::lsScanSkipped)
 		{
 			pNZBInfo->SetUrlStatus((NZBInfo::EUrlStatus)iUrlStatus);
@@ -673,9 +797,17 @@ bool DiskState::LoadNZBInfo(NZBInfo* pNZBInfo, Servers* pServers, FILE* infile,
 
 	if (iFormatVersion >= 51)
 	{
-		int iParFull, iForceParFull, iForceRepair;
-		if (fscanf(infile, "%i,%i,%i\n", &iParFull, &iForceParFull, &iForceRepair) != 3) goto error;
+		int iParFull, iForceParFull, iForceRepair, iExtraParBlocks = 0;
+		if (iFormatVersion >= 55)
+		{
+			if (fscanf(infile, "%i,%i,%i,%i\n", &iParFull, &iForceParFull, &iForceRepair, &iExtraParBlocks) != 4) goto error;
+		}
+		else
+		{
+			if (fscanf(infile, "%i,%i,%i\n", &iParFull, &iForceParFull, &iForceRepair) != 3) goto error;
+		}
 		pNZBInfo->SetParFull((bool)iParFull);
+		pNZBInfo->SetExtraParBlocks(iExtraParBlocks);
 		if (pNZBInfo->GetPostInfo())
 		{
 			pNZBInfo->GetPostInfo()->SetForceParFull((bool)iForceParFull);
@@ -2023,81 +2155,49 @@ bool DiskState::SaveFeeds(Feeds* pFeeds, FeedHistory* pFeedHistory)
 {
 	debug("Saving feeds state to disk");
 
-	char destFilename[1024];
-	snprintf(destFilename, 1024, "%s%s", g_pOptions->GetQueueDir(), "feeds");
-	destFilename[1024-1] = '\0';
-
-	char tempFilename[1024];
-	snprintf(tempFilename, 1024, "%s%s", g_pOptions->GetQueueDir(), "feeds.new");
-	tempFilename[1024-1] = '\0';
+	StateFile stateFile("feeds", 3);
 
 	if (pFeeds->empty() && pFeedHistory->empty())
 	{
-		remove(destFilename);
+		stateFile.Discard();
 		return true;
 	}
 
-	FILE* outfile = fopen(tempFilename, FOPEN_WB);
-
+	FILE* outfile = stateFile.BeginWriteTransaction();
 	if (!outfile)
 	{
-		error("Error saving diskstate: Could not create file %s", tempFilename);
 		return false;
 	}
 
-	fprintf(outfile, "%s%i\n", FORMATVERSION_SIGNATURE, 3);
-
 	// save status
 	SaveFeedStatus(pFeeds, outfile);
 
 	// save history
 	SaveFeedHistory(pFeedHistory, outfile);
 
-	fclose(outfile);
-
 	// now rename to dest file name
-	remove(destFilename);
-	if (rename(tempFilename, destFilename))
-	{
-		error("Error saving diskstate: Could not rename file %s to %s", tempFilename, destFilename);
-		return false;
-	}
-
-	return true;
+	return stateFile.FinishWriteTransaction();
 }
 
 bool DiskState::LoadFeeds(Feeds* pFeeds, FeedHistory* pFeedHistory)
 {
 	debug("Loading feeds state from disk");
 
-	bool bOK = false;
+	StateFile stateFile("feeds", 3);
 
-	char fileName[1024];
-	snprintf(fileName, 1024, "%s%s", g_pOptions->GetQueueDir(), "feeds");
-	fileName[1024-1] = '\0';
-
-	if (!Util::FileExists(fileName))
+	if (!stateFile.FileExists())
 	{
 		return true;
 	}
 
-	FILE* infile = fopen(fileName, FOPEN_RB);
-
+	FILE* infile = stateFile.BeginReadTransaction();
 	if (!infile)
 	{
-		error("Error reading diskstate: could not open file %s", fileName);
 		return false;
 	}
 
-	char FileSignatur[128];
-	fgets(FileSignatur, sizeof(FileSignatur), infile);
-	int iFormatVersion = ParseFormatVersion(FileSignatur);
-	if (iFormatVersion > 3)
-	{
-		error("Could not load diskstate due to file version mismatch");
-		fclose(infile);
-		return false;
-	}
+	bool bOK = false;
+	int iFormatVersion = stateFile.GetFileVersion();
 
 	// load feed status
 	if (!LoadFeedStatus(pFeeds, infile, iFormatVersion)) goto error;
@@ -2109,10 +2209,9 @@ bool DiskState::LoadFeeds(Feeds* pFeeds, FeedHistory* pFeedHistory)
 
 error:
 
-	fclose(infile);
 	if (!bOK)
 	{
-		error("Error reading diskstate for file %s", fileName);
+		error("Error reading diskstate for feeds");
 	}
 
 	return bOK;
@@ -2410,81 +2509,49 @@ bool DiskState::SaveStats(Servers* pServers, ServerVolumes* pServerVolumes)
 {
 	debug("Saving stats to disk");
 
-	char destFilename[1024];
-	snprintf(destFilename, 1024, "%s%s", g_pOptions->GetQueueDir(), "stats");
-	destFilename[1024-1] = '\0';
-
-	char tempFilename[1024];
-	snprintf(tempFilename, 1024, "%s%s", g_pOptions->GetQueueDir(), "stats.new");
-	tempFilename[1024-1] = '\0';
+	StateFile stateFile("stats", 3);
 
 	if (pServers->empty())
 	{
-		remove(destFilename);
+		stateFile.Discard();
 		return true;
 	}
 
-	FILE* outfile = fopen(tempFilename, FOPEN_WB);
-
+	FILE* outfile = stateFile.BeginWriteTransaction();
 	if (!outfile)
 	{
-		error("Error saving diskstate: Could not create file %s", tempFilename);
 		return false;
 	}
 
-	fprintf(outfile, "%s%i\n", FORMATVERSION_SIGNATURE, 3);
-
 	// save server names
 	SaveServerInfo(pServers, outfile);
 
 	// save stat
 	SaveVolumeStat(pServerVolumes, outfile);
 
-	fclose(outfile);
-
 	// now rename to dest file name
-	remove(destFilename);
-	if (rename(tempFilename, destFilename))
-	{
-		error("Error saving diskstate: Could not rename file %s to %s", tempFilename, destFilename);
-		return false;
-	}
-
-	return true;
+	return stateFile.FinishWriteTransaction();
 }
 
 bool DiskState::LoadStats(Servers* pServers, ServerVolumes* pServerVolumes, bool* pPerfectMatch)
 {
 	debug("Loading stats from disk");
 
-	bool bOK = false;
+	StateFile stateFile("stats", 3);
 
-	char fileName[1024];
-	snprintf(fileName, 1024, "%s%s", g_pOptions->GetQueueDir(), "stats");
-	fileName[1024-1] = '\0';
-
-	if (!Util::FileExists(fileName))
+	if (!stateFile.FileExists())
 	{
 		return true;
 	}
 
-	FILE* infile = fopen(fileName, FOPEN_RB);
-
+	FILE* infile = stateFile.BeginReadTransaction();
 	if (!infile)
 	{
-		error("Error reading diskstate: could not open file %s", fileName);
 		return false;
 	}
 
-	char FileSignatur[128];
-	fgets(FileSignatur, sizeof(FileSignatur), infile);
-	int iFormatVersion = ParseFormatVersion(FileSignatur);
-	if (iFormatVersion > 3)
-	{
-		error("Could not load diskstate due to file version mismatch");
-		fclose(infile);
-		return false;
-	}
+	bool bOK = false;
+	int iFormatVersion = stateFile.GetFileVersion();
 
 	if (!LoadServerInfo(pServers, infile, iFormatVersion, pPerfectMatch)) goto error;
 
@@ -2497,10 +2564,9 @@ bool DiskState::LoadStats(Servers* pServers, ServerVolumes* pServerVolumes, bool
 
 error:
 
-	fclose(infile);
 	if (!bOK)
 	{
-		error("Error reading diskstate for file %s", fileName);
+		error("Error reading diskstate for statistics");
 	}
 
 	return bOK;
