@@ -2,7 +2,7 @@
  *  This file is part of nzbget
  *
  *  Copyright (C) 2004 Sven Henkel <sidddy@users.sourceforge.net>
- *  Copyright (C) 2007-2014 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2007-2015 Andrey Prygunkov <hugbug@users.sourceforge.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,8 +18,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 1139 $
- * $Date: 2014-10-09 23:11:42 +0200 (Thu, 09 Oct 2014) $
+ * $Revision$
+ * $Date$
  *
  */
 
@@ -35,6 +35,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <stdio.h>
+#include <stdarg.h>
 #include <ctype.h>
 #include <sys/stat.h>
 #include <algorithm>
@@ -42,12 +43,10 @@
 #include "nzbget.h"
 #include "DownloadInfo.h"
 #include "ArticleWriter.h"
+#include "DiskState.h"
 #include "Options.h"
 #include "Util.h"
 
-extern Options* g_pOptions;
-extern ArticleCache* g_pArticleCache;
-
 int FileInfo::m_iIDGen = 0;
 int FileInfo::m_iIDMax = 0;
 int NZBInfo::m_iIDGen = 0;
@@ -55,7 +54,6 @@ int NZBInfo::m_iIDMax = 0;
 DownloadQueue* DownloadQueue::g_pDownloadQueue = NULL;
 bool DownloadQueue::g_bLoaded = false;
 
-
 NZBParameter::NZBParameter(const char* szName)
 {
 	m_szName = strdup(szName);
@@ -305,6 +303,7 @@ NZBInfo::NZBInfo() : m_FileList(true)
 	m_eDeleteStatus = dsNone;
 	m_eMarkStatus = ksNone;
 	m_eUrlStatus = lsNone;
+	m_iExtraParBlocks = 0;
 	m_bAddUrlPaused = false;
 	m_bDeleting = false;
 	m_bDeletePaused = false;
@@ -342,6 +341,9 @@ NZBInfo::NZBInfo() : m_FileList(true)
 	m_bReprocess = false;
 	m_tQueueScriptTime = 0;
 	m_bParFull = false;
+	m_iMessageCount = 0;
+	m_iCachedMessageCount = 0;
+	m_iFeedID = 0;
 }
 
 NZBInfo::~NZBInfo()
@@ -360,12 +362,6 @@ NZBInfo::~NZBInfo()
 
 	ClearCompletedFiles();
 
-	for (Messages::iterator it = m_Messages.begin(); it != m_Messages.end(); it++)
-	{
-		delete *it;
-	}
-	m_Messages.clear();
-
 	m_FileList.Clear();
 }
 
@@ -588,9 +584,8 @@ int NZBInfo::CalcHealth()
 		return 1000;
 	}
 
-	int iHealth = (int)(Util::Int64ToFloat(m_lSize - m_lParSize -
-		(m_lCurrentFailedSize - m_lParCurrentFailedSize)) * 1000.0 /
-		Util::Int64ToFloat(m_lSize - m_lParSize));
+	int iHealth = (int)((m_lSize - m_lParSize -
+		(m_lCurrentFailedSize - m_lParCurrentFailedSize)) * 1000 / (m_lSize - m_lParSize));
 
 	if (iHealth == 1000 && m_lCurrentFailedSize - m_lParCurrentFailedSize > 0)
 	{
@@ -607,9 +602,13 @@ int NZBInfo::CalcCriticalHealth(bool bAllowEstimation)
 		return 1000;
 	}
 
+	if (m_lSize == m_lParSize)
+	{
+		return 0;
+	}
+
 	long long lGoodParSize = m_lParSize - m_lParCurrentFailedSize;
-	int iCriticalHealth = (int)(Util::Int64ToFloat(m_lSize - lGoodParSize*2) * 1000.0 /
-		Util::Int64ToFloat(m_lSize - lGoodParSize));
+	int iCriticalHealth = (int)((m_lSize - lGoodParSize*2) * 1000 / (m_lSize - lGoodParSize));
 
 	if (lGoodParSize*2 > m_lSize)
 	{
@@ -658,27 +657,81 @@ void NZBInfo::UpdateMinMaxTime()
 	}
 }
 
-NZBInfo::Messages* NZBInfo::LockMessages()
+MessageList* NZBInfo::LockCachedMessages()
 {
 	m_mutexLog.Lock();
 	return &m_Messages;
 }
 
-void NZBInfo::UnlockMessages()
+void NZBInfo::UnlockCachedMessages()
 {
 	m_mutexLog.Unlock();
 }
 
-void NZBInfo::AppendMessage(Message::EKind eKind, time_t tTime, const char * szText)
+void NZBInfo::AddMessage(Message::EKind eKind, const char * szText)
 {
-	if (tTime == 0)
+	switch (eKind)
 	{
-		tTime = time(NULL);
+		case Message::mkDetail:
+			detail("%s", szText);
+			break;
+
+		case Message::mkInfo:
+			info("%s", szText);
+			break;
+
+		case Message::mkWarning:
+			warn("%s", szText);
+			break;
+
+		case Message::mkError:
+			error("%s", szText);
+			break;
+
+		case Message::mkDebug:
+			debug("%s", szText);
+			break;
 	}
 
 	m_mutexLog.Lock();
-	Message* pMessage = new Message(++m_iIDMessageGen, eKind, tTime, szText);
+	Message* pMessage = new Message(++m_iIDMessageGen, eKind, time(NULL), szText);
 	m_Messages.push_back(pMessage);
+
+	if (g_pOptions->GetSaveQueue() && g_pOptions->GetServerMode() && g_pOptions->GetNzbLog())
+	{
+		g_pDiskState->AppendNZBMessage(m_iID, eKind, szText);
+		m_iMessageCount++;
+	}
+
+	while (m_Messages.size() > (unsigned int)g_pOptions->GetLogBufferSize())
+	{
+		Message* pMessage = m_Messages.front();
+		delete pMessage;
+		m_Messages.pop_front();
+	}
+
+	m_iCachedMessageCount = m_Messages.size();
+	m_mutexLog.Unlock();
+}
+
+void NZBInfo::PrintMessage(Message::EKind eKind, const char* szFormat, ...)
+{
+	char tmp2[1024];
+
+	va_list ap;
+	va_start(ap, szFormat);
+	vsnprintf(tmp2, 1024, szFormat, ap);
+	tmp2[1024-1] = '\0';
+	va_end(ap);
+
+	AddMessage(eKind, tmp2);
+}
+
+void NZBInfo::ClearMessages()
+{
+	m_mutexLog.Lock();
+	m_Messages.Clear();
+	m_iCachedMessageCount = 0;
 	m_mutexLog.Unlock();
 }
 
@@ -716,6 +769,7 @@ void NZBInfo::CopyFileList(NZBInfo* pSrcNZBInfo)
 	SetParFailedSize(pSrcNZBInfo->GetParFailedSize());
 	SetParCurrentFailedSize(pSrcNZBInfo->GetParCurrentFailedSize());
 
+	SetTotalArticles(pSrcNZBInfo->GetTotalArticles());
 	SetSuccessArticles(pSrcNZBInfo->GetSuccessArticles());
 	SetFailedArticles(pSrcNZBInfo->GetFailedArticles());
 	SetCurrentSuccessArticles(pSrcNZBInfo->GetSuccessArticles());
@@ -735,6 +789,7 @@ void NZBInfo::LeavePostProcess()
 {
 	delete m_pPostInfo;
 	m_pPostInfo = NULL;
+	ClearMessages();
 }
 
 void NZBInfo::SetActiveDownloads(int iActiveDownloads)
@@ -759,14 +814,16 @@ void NZBInfo::SetActiveDownloads(int iActiveDownloads)
 bool NZBInfo::IsDupeSuccess()
 {
 	bool bFailure =
-		m_eDeleteStatus != NZBInfo::dsNone ||
+		m_eMarkStatus != NZBInfo::ksSuccess &&
+		m_eMarkStatus != NZBInfo::ksGood &&
+		(m_eDeleteStatus != NZBInfo::dsNone ||
 		m_eMarkStatus == NZBInfo::ksBad ||
 		m_eParStatus == NZBInfo::psFailure ||
 		m_eUnpackStatus == NZBInfo::usFailure ||
 		m_eUnpackStatus == NZBInfo::usPassword ||
 		(m_eParStatus == NZBInfo::psSkipped &&
 		 m_eUnpackStatus == NZBInfo::usSkipped &&
-		 CalcHealth() < CalcCriticalHealth(true));
+		 CalcHealth() < CalcCriticalHealth(true)));
 	return !bFailure;
 }
 
@@ -788,6 +845,10 @@ const char* NZBInfo::MakeTextStatus(bool bIgnoreScriptStatus)
 		{
 			szStatus = "SUCCESS/GOOD";
 		}
+		else if (m_eMarkStatus == NZBInfo::ksSuccess)
+		{
+			szStatus = "SUCCESS/MARK";
+		}
 		else if (m_eDeleteStatus == NZBInfo::dsHealth)
 		{
 			szStatus = "FAILURE/HEALTH";
@@ -804,6 +865,18 @@ const char* NZBInfo::MakeTextStatus(bool bIgnoreScriptStatus)
 		{
 			szStatus = "FAILURE/BAD";
 		}
+		else if (m_eDeleteStatus == NZBInfo::dsGood)
+		{
+			szStatus = "DELETED/GOOD";
+		}
+		else if (m_eDeleteStatus == NZBInfo::dsCopy)
+		{
+			szStatus = "DELETED/COPY";
+		}
+		else if (m_eDeleteStatus == NZBInfo::dsScan)
+		{
+			szStatus = "FAILURE/SCAN";
+		}
 		else if (m_eParStatus == NZBInfo::psFailure)
 		{
 			szStatus = "FAILURE/PAR";
@@ -1196,6 +1269,10 @@ PostInfo::PostInfo()
 	m_bRequestParCheck = false;
 	m_bForceParFull = false;
 	m_bForceRepair = false;
+	m_bParRepaired = false;
+	m_bUnpackTried = false;
+	m_bPassListTried = false;
+	m_eLastUnpackStatus = 0;
 	m_szProgressLabel = strdup("");
 	m_iFileProgress = 0;
 	m_iStageProgress = 0;
@@ -1203,7 +1280,6 @@ PostInfo::PostInfo()
 	m_tStageTime = 0;
 	m_eStage = ptQueued;
 	m_pPostThread = NULL;
-	m_iIDMessageGen = 0;
 }
 
 PostInfo::~ PostInfo()
@@ -1212,11 +1288,6 @@ PostInfo::~ PostInfo()
 
 	free(m_szProgressLabel);
 
-	for (Messages::iterator it = m_Messages.begin(); it != m_Messages.end(); it++)
-	{
-		delete *it;
-	}
-
 	for (ParredFiles::iterator it = m_ParredFiles.begin(); it != m_ParredFiles.end(); it++)
 	{
 		free(*it);
@@ -1229,32 +1300,6 @@ void PostInfo::SetProgressLabel(const char* szProgressLabel)
 	m_szProgressLabel = strdup(szProgressLabel);
 }
 
-PostInfo::Messages* PostInfo::LockMessages()
-{
-	m_mutexLog.Lock();
-	return &m_Messages;
-}
-
-void PostInfo::UnlockMessages()
-{
-	m_mutexLog.Unlock();
-}
-
-void PostInfo::AppendMessage(Message::EKind eKind, const char * szText)
-{
-	m_mutexLog.Lock();
-	Message* pMessage = new Message(++m_iIDMessageGen, eKind, time(NULL), szText);
-	m_Messages.push_back(pMessage);
-
-	while (m_Messages.size() > (unsigned int)g_pOptions->GetLogBufferSize())
-	{
-		Message* pMessage = m_Messages.front();
-		delete pMessage;
-		m_Messages.pop_front();
-	}
-	m_mutexLog.Unlock();
-}
-
 
 DupInfo::DupInfo()
 {
@@ -1354,6 +1399,29 @@ void HistoryInfo::GetName(char* szBuffer, int iSize)
 }
 
 
+HistoryList::~HistoryList()
+{
+	for (iterator it = begin(); it != end(); it++)
+	{
+		delete *it;
+	}
+}
+
+HistoryInfo* HistoryList::Find(int iID)
+{
+	for (iterator it = begin(); it != end(); it++)
+	{
+		HistoryInfo* pHistoryInfo = *it;
+		if (pHistoryInfo->GetID() == iID)
+		{
+			return pHistoryInfo;
+		}
+	}
+
+	return NULL;
+}
+
+
 DownloadQueue* DownloadQueue::Lock()
 {
 	g_pDownloadQueue->m_LockMutex.Lock();
