@@ -2,7 +2,7 @@
  *  This file is part of nzbget
  *
  *  Copyright (C) 2004 Sven Henkel <sidddy@users.sourceforge.net>
- *  Copyright (C) 2007-2014 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2007-2015 Andrey Prygunkov <hugbug@users.sourceforge.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,8 +18,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 1139 $
- * $Date: 2014-10-09 23:11:42 +0200 (Thu, 09 Oct 2014) $
+ * $Revision$
+ * $Date$
  *
  */
 
@@ -391,14 +391,18 @@ public:
 		dsManual,
 		dsHealth,
 		dsDupe,
-		dsBad
+		dsBad,
+		dsGood,
+		dsCopy,
+		dsScan
 	};
 
 	enum EMarkStatus
 	{
 		ksNone,
 		ksBad,
-		ksGood
+		ksGood,
+		ksSuccess
 	};
 
 	enum EUrlStatus
@@ -418,8 +422,6 @@ public:
 		nkUrl
 	};
 
-	typedef std::deque<Message*>		Messages;
-
 	static const int FORCE_PRIORITY = 900;
 
 	friend class DupInfo;
@@ -467,6 +469,7 @@ private:
 	EDeleteStatus		m_eDeleteStatus;
 	EMarkStatus			m_eMarkStatus;
 	EUrlStatus			m_eUrlStatus;
+	int					m_iExtraParBlocks;
 	bool				m_bAddUrlPaused;
 	bool				m_bDeletePaused;
 	bool				m_bManyDupeFiles;
@@ -489,7 +492,7 @@ private:
 	ServerStatList		m_ServerStats;
 	ServerStatList		m_CurrentServerStats;
 	Mutex				m_mutexLog;
-	Messages			m_Messages;
+	MessageList			m_Messages;
 	int					m_iIDMessageGen;
 	PostInfo*			m_pPostInfo;
 	long long 			m_lDownloadedSize;
@@ -502,10 +505,15 @@ private:
 	bool				m_bReprocess;
 	time_t				m_tQueueScriptTime;
 	bool				m_bParFull;
+	int					m_iMessageCount;
+	int					m_iCachedMessageCount;
+	int					m_iFeedID;
 
 	static int			m_iIDGen;
 	static int			m_iIDMax;
 
+	void				ClearMessages();
+
 public:
 						NZBInfo();
 						~NZBInfo();
@@ -599,6 +607,8 @@ public:
 	EMarkStatus			GetMarkStatus() { return m_eMarkStatus; }
 	void				SetMarkStatus(EMarkStatus eMarkStatus) { m_eMarkStatus = eMarkStatus; }
 	EUrlStatus			GetUrlStatus() { return m_eUrlStatus; }
+	int					GetExtraParBlocks() { return m_iExtraParBlocks; }
+	void				SetExtraParBlocks(int iExtraParBlocks) { m_iExtraParBlocks = iExtraParBlocks; }
 	void				SetUrlStatus(EUrlStatus eUrlStatus) { m_eUrlStatus = eUrlStatus; }
 	const char*			GetQueuedFilename() { return m_szQueuedFilename; }
 	void				SetQueuedFilename(const char* szQueuedFilename);
@@ -657,6 +667,8 @@ public:
 	void 				SetQueueScriptTime(time_t tQueueScriptTime) { m_tQueueScriptTime = tQueueScriptTime; }
 	void				SetParFull(bool bParFull) { m_bParFull = bParFull; }
 	bool				GetParFull() { return m_bParFull; }
+	int					GetFeedID() { return m_iFeedID; }
+	void				SetFeedID(int iFeedID) { m_iFeedID = iFeedID; }
 
 	void				CopyFileList(NZBInfo* pSrcNZBInfo);
 	void				UpdateMinMaxTime();
@@ -666,9 +678,13 @@ public:
 	bool				IsDupeSuccess();
 	const char*			MakeTextStatus(bool bIgnoreScriptStatus);
 
-	void				AppendMessage(Message::EKind eKind, time_t tTime, const char* szText);
-	Messages*			LockMessages();
-	void				UnlockMessages();
+	void				AddMessage(Message::EKind eKind, const char* szText);
+	void				PrintMessage(Message::EKind eKind, const char* szFormat, ...);
+	int					GetMessageCount() { return m_iMessageCount; }
+	void				SetMessageCount(int iMessageCount) { m_iMessageCount = iMessageCount; }
+	int					GetCachedMessageCount() { return m_iCachedMessageCount; }
+	MessageList*		LockCachedMessages();
+	void				UnlockCachedMessages();
 };
 
 typedef std::deque<NZBInfo*> NZBQueueBase;
@@ -703,7 +719,6 @@ public:
 		ptFinished
 	};
 
-	typedef std::deque<Message*>	Messages;
 	typedef std::vector<char*>		ParredFiles;
 
 private:
@@ -713,6 +728,10 @@ private:
 	bool				m_bRequestParCheck;
 	bool				m_bForceParFull;
 	bool				m_bForceRepair;
+	bool				m_bParRepaired;
+	bool				m_bUnpackTried;
+	bool				m_bPassListTried;
+	int					m_eLastUnpackStatus;
 	EStage				m_eStage;
 	char*				m_szProgressLabel;
 	int					m_iFileProgress;
@@ -721,9 +740,6 @@ private:
 	time_t				m_tStageTime;
 	Thread*				m_pPostThread;
 	
-	Mutex				m_mutexLog;
-	Messages			m_Messages;
-	int					m_iIDMessageGen;
 	ParredFiles			m_ParredFiles;
 
 public:
@@ -753,11 +769,16 @@ public:
 	void				SetForceParFull(bool bForceParFull) { m_bForceParFull = bForceParFull; }
 	bool				GetForceRepair() { return m_bForceRepair; }
 	void				SetForceRepair(bool bForceRepair) { m_bForceRepair = bForceRepair; }
+	bool				GetParRepaired() { return m_bParRepaired; }
+	void				SetParRepaired(bool bParRepaired) { m_bParRepaired = bParRepaired; }
+	bool				GetUnpackTried() { return m_bUnpackTried; }
+	void				SetUnpackTried(bool bUnpackTried) { m_bUnpackTried = bUnpackTried; }
+	bool				GetPassListTried() { return m_bPassListTried; }
+	void				SetPassListTried(bool bPassListTried) { m_bPassListTried = bPassListTried; }
+	int					GetLastUnpackStatus() { return m_eLastUnpackStatus; }
+	void				SetLastUnpackStatus(int eUnpackStatus) { m_eLastUnpackStatus = eUnpackStatus; }
 	Thread*				GetPostThread() { return m_pPostThread; }
 	void				SetPostThread(Thread* pPostThread) { m_pPostThread = pPostThread; }
-	void				AppendMessage(Message::EKind eKind, const char* szText);
-	Messages*			LockMessages();
-	void				UnlockMessages();
 	ParredFiles*		GetParredFiles() { return &m_ParredFiles; }
 };
 
@@ -843,7 +864,14 @@ public:
 	void				GetName(char* szBuffer, int iSize);		// needs locking (for shared objects)
 };
 
-typedef std::deque<HistoryInfo*> HistoryList;
+typedef std::deque<HistoryInfo*> HistoryListBase;
+
+class HistoryList : public HistoryListBase
+{
+public:
+						~HistoryList();
+	HistoryInfo*		Find(int iID);
+};
 
 class DownloadQueue : public Subject
 {
@@ -897,6 +925,7 @@ public:
 		eaGroupSetDupeKey,		// set duplicate key
 		eaGroupSetDupeScore,	// set duplicate score
 		eaGroupSetDupeMode,		// set duplicate mode
+		eaGroupSort,			// sort groups
 		eaPostDelete,			// cancel post-processing
 		eaHistoryDelete,		// hide history-item
 		eaHistoryFinalDelete,	// delete history-item
@@ -909,7 +938,10 @@ public:
 		eaHistorySetDupeMode,	// set duplicate mode
 		eaHistorySetDupeBackup,	// set duplicate backup flag
 		eaHistoryMarkBad,		// mark history-item as bad (and download other duplicate)
-		eaHistoryMarkGood		// mark history-item as good (and push it into dup-history)
+		eaHistoryMarkGood,		// mark history-item as good (and push it into dup-history)
+		eaHistoryMarkSuccess,	// mark history-item as success (and do nothing more)
+		eaHistorySetCategory,	// set or change category for history-item
+		eaHistorySetName		// set history-item name (rename)
 	};
 
 	enum EMatchMode
@@ -934,7 +966,6 @@ protected:
 	static void				Loaded() { g_bLoaded = true; }
 
 public:
-	virtual					~DownloadQueue() {}
 	static bool				IsLoaded() { return g_bLoaded; }
 	static DownloadQueue*	Lock();
 	static void				Unlock();
