@@ -1,7 +1,7 @@
 /*
  *  This file is part of nzbget
  *
- *  Copyright (C) 2007-2014 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2007-2015 Andrey Prygunkov <hugbug@users.sourceforge.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,8 +17,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 1123 $
- * $Date: 2014-09-13 23:30:42 +0200 (Sat, 13 Sep 2014) $
+ * $Revision$
+ * $Date$
  *
  */
 
@@ -50,9 +50,6 @@
 #include "HistoryCoordinator.h"
 #include "DupeCoordinator.h"
 
-extern HistoryCoordinator* g_pHistoryCoordinator;
-extern Options* g_pOptions;
-
 bool DupeCoordinator::SameNameOrKey(const char* szName1, const char* szDupeKey1,
 	const char* szName2, const char* szDupeKey2)
 {
@@ -92,18 +89,31 @@ void DupeCoordinator::NZBFound(DownloadQueue* pDownloadQueue, NZBInfo* pNZBInfo)
 		// in queue - the new item is skipped
 		if (pQueuedNZBInfo != pNZBInfo && bSameContent && pNZBInfo->GetKind() == NZBInfo::nkNzb)
 		{
+			char szMessage[1024];
 			if (!strcmp(pNZBInfo->GetName(), pQueuedNZBInfo->GetName()))
 			{
-				warn("Skipping duplicate %s, already queued", pNZBInfo->GetName());
+				snprintf(szMessage, 1024, "Skipping duplicate %s, already queued", pNZBInfo->GetName());
 			}
 			else
 			{
-				warn("Skipping duplicate %s, already queued as %s",
+				snprintf(szMessage, 1024, "Skipping duplicate %s, already queued as %s",
 					pNZBInfo->GetName(), pQueuedNZBInfo->GetName());
 			}
-			// Flag saying QueueCoordinator to skip nzb-file
-			pNZBInfo->SetDeleteStatus(NZBInfo::dsManual);
-			g_pHistoryCoordinator->DeleteDiskFiles(pNZBInfo);
+			szMessage[1024-1] = '\0';
+
+			if (pNZBInfo->GetFeedID())
+			{
+				warn("%s", szMessage);
+				// Flag saying QueueCoordinator to skip nzb-file
+				pNZBInfo->SetDeleteStatus(NZBInfo::dsManual);
+				g_pHistoryCoordinator->DeleteDiskFiles(pNZBInfo);
+			}
+			else
+			{
+				pNZBInfo->SetDeleteStatus(NZBInfo::dsCopy);
+				pNZBInfo->AddMessage(Message::mkWarning, szMessage);
+			}
+
 			return;
 		}
 	}
@@ -162,9 +172,9 @@ void DupeCoordinator::NZBFound(DownloadQueue* pDownloadQueue, NZBInfo* pNZBInfo)
 	bool bSameContent = false;
 	const char* szDupeName = NULL;
 
-	// find duplicates in queue having exactly same content
+	// find duplicates in history having exactly same content
 	// also: nzb-files having duplicates marked as good are skipped
-	// also (only in score mode): nzb-files having success-duplicates in dup-history but don't having duplicates in recent history are skipped
+	// also (only in score mode): nzb-files having success-duplicates in dup-history but not having duplicates in recent history are skipped
 	for (HistoryList::iterator it = pDownloadQueue->GetHistory()->begin(); it != pDownloadQueue->GetHistory()->end(); it++)
 	{
 		HistoryInfo* pHistoryInfo = *it;
@@ -236,7 +246,7 @@ void DupeCoordinator::NZBFound(DownloadQueue* pDownloadQueue, NZBInfo* pNZBInfo)
 			{
 				// Flag saying QueueCoordinator to skip nzb-file
 				pNZBInfo->SetDeleteStatus(NZBInfo::dsDupe);
-				info("Collection %s is duplicate to %s", pNZBInfo->GetName(), pHistoryInfo->GetNZBInfo()->GetName());
+				info("Collection %s is a duplicate to %s", pNZBInfo->GetName(), pHistoryInfo->GetNZBInfo()->GetName());
 				return;
 			}
 		}
@@ -244,21 +254,33 @@ void DupeCoordinator::NZBFound(DownloadQueue* pDownloadQueue, NZBInfo* pNZBInfo)
 
 	if (bSkip)
 	{
+		char szMessage[1024];
 		if (!strcmp(pNZBInfo->GetName(), szDupeName))
 		{
-			warn("Skipping duplicate %s, found in history with %s", pNZBInfo->GetName(),
+			snprintf(szMessage, 1024, "Skipping duplicate %s, found in history with %s", pNZBInfo->GetName(),
 				bSameContent ? "exactly same content" : bGood ? "good status" : "success status");
 		}
 		else
 		{
-			warn("Skipping duplicate %s, found in history %s with %s",
+			snprintf(szMessage, 1024, "Skipping duplicate %s, found in history %s with %s",
 				pNZBInfo->GetName(), szDupeName,
 				bSameContent ? "exactly same content" : bGood ? "good status" : "success status");
 		}
+		szMessage[1024-1] = '\0';
+
+		if (pNZBInfo->GetFeedID())
+		{
+			warn("%s", szMessage);
+			// Flag saying QueueCoordinator to skip nzb-file
+			pNZBInfo->SetDeleteStatus(NZBInfo::dsManual);
+			g_pHistoryCoordinator->DeleteDiskFiles(pNZBInfo);
+		}
+		else
+		{
+			pNZBInfo->SetDeleteStatus(bSameContent ? NZBInfo::dsCopy : NZBInfo::dsGood);
+			pNZBInfo->AddMessage(Message::mkWarning, szMessage);
+		}
 
-		// Flag saying QueueCoordinator to skip nzb-file
-		pNZBInfo->SetDeleteStatus(NZBInfo::dsManual);
-		g_pHistoryCoordinator->DeleteDiskFiles(pNZBInfo);
 		return;
 	}
 
@@ -283,7 +305,7 @@ void DupeCoordinator::NZBFound(DownloadQueue* pDownloadQueue, NZBInfo* pNZBInfo)
 				{
 					// Flag saying QueueCoordinator to skip nzb-file
 					pNZBInfo->SetDeleteStatus(NZBInfo::dsDupe);
-					info("Collection %s is duplicate to %s", pNZBInfo->GetName(), pQueuedNZBInfo->GetName());
+					info("Collection %s is a duplicate to %s", pNZBInfo->GetName(), pQueuedNZBInfo->GetName());
 					return;
 				}
 
@@ -334,8 +356,7 @@ void DupeCoordinator::ReturnBestDupe(DownloadQueue* pDownloadQueue, NZBInfo* pNZ
 
 		if (pHistoryInfo->GetKind() == HistoryInfo::hkNzb &&
 			pHistoryInfo->GetNZBInfo()->GetDupeMode() != dmForce &&
-			(pHistoryInfo->GetNZBInfo()->IsDupeSuccess() ||
-			 pHistoryInfo->GetNZBInfo()->GetMarkStatus() == NZBInfo::ksGood) &&
+			pHistoryInfo->GetNZBInfo()->IsDupeSuccess() &&
 			SameNameOrKey(pHistoryInfo->GetNZBInfo()->GetName(), pHistoryInfo->GetNZBInfo()->GetDupeKey(), szNZBName, szDupeKey))
 		{
 			if (!bHistoryDupe || pHistoryInfo->GetNZBInfo()->GetDupeScore() > iHistoryScore)
@@ -412,20 +433,25 @@ void DupeCoordinator::ReturnBestDupe(DownloadQueue* pDownloadQueue, NZBInfo* pNZ
 	}
 }
 
-void DupeCoordinator::HistoryMark(DownloadQueue* pDownloadQueue, HistoryInfo* pHistoryInfo, bool bGood)
+void DupeCoordinator::HistoryMark(DownloadQueue* pDownloadQueue, HistoryInfo* pHistoryInfo, NZBInfo::EMarkStatus eMarkStatus)
 {
 	char szNZBName[1024];
 	pHistoryInfo->GetName(szNZBName, 1024);
 
-	info("Marking %s as %s", szNZBName, (bGood ? "good" : "bad"));
+    const char* szMarkStatusName[] = { "NONE", "bad", "good", "success" };
+
+	info("Marking %s as %s", szNZBName, szMarkStatusName[eMarkStatus]);
 
 	if (pHistoryInfo->GetKind() == HistoryInfo::hkNzb)
 	{
-		pHistoryInfo->GetNZBInfo()->SetMarkStatus(bGood ? NZBInfo::ksGood : NZBInfo::ksBad);
+		pHistoryInfo->GetNZBInfo()->SetMarkStatus(eMarkStatus);
 	}
 	else if (pHistoryInfo->GetKind() == HistoryInfo::hkDup)
 	{
-		pHistoryInfo->GetDupInfo()->SetStatus(bGood ? DupInfo::dsGood : DupInfo::dsBad);
+		pHistoryInfo->GetDupInfo()->SetStatus(
+			eMarkStatus == NZBInfo::ksGood ? DupInfo::dsGood :
+			eMarkStatus == NZBInfo::ksSuccess ? DupInfo::dsSuccess :
+			DupInfo::dsBad);
 	}
 	else
 	{
@@ -442,13 +468,13 @@ void DupeCoordinator::HistoryMark(DownloadQueue* pDownloadQueue, HistoryInfo* pH
 		return;
 	}
 
-	if (bGood)
+	if (eMarkStatus == NZBInfo::ksGood)
 	{
 		// mark as good
 		// moving all duplicates from history to dup-history
 		HistoryCleanup(pDownloadQueue, pHistoryInfo);
 	}
-	else
+	else if (eMarkStatus == NZBInfo::ksBad)
 	{
 		// mark as bad
 		const char* szDupeKey = pHistoryInfo->GetKind() == HistoryInfo::hkNzb ? pHistoryInfo->GetNZBInfo()->GetDupeKey() :
@@ -562,3 +588,25 @@ DupeCoordinator::EDupeStatus DupeCoordinator::GetDupeStatus(DownloadQueue* pDown
 
 	return eStatuses;
 }
+
+void DupeCoordinator::ListHistoryDupes(DownloadQueue* pDownloadQueue, NZBInfo* pNZBInfo, NZBList* pDupeList)
+{
+	if (pNZBInfo->GetDupeMode() == dmForce)
+	{
+		return;
+	}
+
+	// find duplicates in history
+	for (HistoryList::iterator it = pDownloadQueue->GetHistory()->begin(); it != pDownloadQueue->GetHistory()->end(); it++)
+	{
+		HistoryInfo* pHistoryInfo = *it;
+
+		if (pHistoryInfo->GetKind() == HistoryInfo::hkNzb &&
+			pHistoryInfo->GetNZBInfo()->GetDupeMode() != dmForce &&
+			SameNameOrKey(pHistoryInfo->GetNZBInfo()->GetName(), pHistoryInfo->GetNZBInfo()->GetDupeKey(),
+				pNZBInfo->GetName(), pNZBInfo->GetDupeKey()))
+		{
+			pDupeList->push_back(pHistoryInfo->GetNZBInfo());
+		}
+	}
+}
