@@ -2,7 +2,7 @@
  *  This file is part of nzbget
  *
  *  Copyright (C) 2004 Sven Henkel <sidddy@users.sourceforge.net>
- *  Copyright (C) 2007-2014 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2007-2015 Andrey Prygunkov <hugbug@users.sourceforge.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,8 +18,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 1125 $
- * $Date: 2014-09-16 16:54:50 -0400 (Tue, 16 Sep 2014) $
+ * $Revision$
+ * $Date$
  *
  */
 
@@ -49,8 +49,11 @@ private:
 	{
 	private:
 		QueueCoordinator*	m_pOwner;
+		bool				m_bMassEdit;
+		bool				m_bWantSave;
 		friend class QueueCoordinator;
 	public:
+							CoordinatorDownloadQueue(): m_bMassEdit(false), m_bWantSave(false) {}
 		virtual bool		EditEntry(int ID, EEditAction eAction, int iOffset, const char* szText);
 		virtual bool		EditList(IDList* pIDList, NameList* pNameList, EMatchMode eMatchMode, EEditAction eAction, int iOffset, const char* szText);
 		virtual void		Save();
@@ -97,4 +100,6 @@ public:
 	bool					SplitQueueEntries(DownloadQueue* pDownloadQueue, FileList* pFileList, const char* szName, NZBInfo** pNewNZBInfo);
 };
 
+extern QueueCoordinator* g_pQueueCoordinator;
+
 #endif
