@@ -17,8 +17,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 1221 $
- * $Date: 2015-02-26 15:57:38 -0500 (Thu, 26 Feb 2015) $
+ * $Revision$
+ * $Date$
  *
  */
 
@@ -54,15 +54,8 @@
 #include "HistoryCoordinator.h"
 #include "UrlCoordinator.h"
 
-extern QueueCoordinator* g_pQueueCoordinator;
-extern HistoryCoordinator* g_pHistoryCoordinator;
-extern UrlCoordinator* g_pUrlCoordinator;
-extern PrePostProcessor* g_pPrePostProcessor;
-extern Options* g_pOptions;
-
 const int MAX_ID = 1000000000;
 
-
 class GroupSorter
 {
 public:
