@@ -1,7 +1,7 @@
 /*
  *  This file is part of nzbget
  *
- *  Copyright (C) 2007-2014 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2007-2015 Andrey Prygunkov <hugbug@users.sourceforge.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,8 +17,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 1146 $
- * $Date: 2014-10-16 22:40:09 +0200 (Thu, 16 Oct 2014) $
+ * $Revision$
+ * $Date$
  *
  */
 
@@ -46,204 +46,10 @@
 #include "Options.h"
 #include "Log.h"
 #include "QueueCoordinator.h"
-#include "QueueScript.h"
+#include "HistoryCoordinator.h"
+#include "ScanScript.h"
 #include "Util.h"
 
-extern QueueCoordinator* g_pQueueCoordinator;
-extern Options* g_pOptions;
-extern Scanner* g_pScanner;
-
-class ScanScriptController : public NZBScriptController
-{
-private:
-	const char*			m_szNZBFilename;
-	const char*			m_szUrl;
-	const char*			m_szDirectory;
-	char**				m_pNZBName;
-	char**				m_pCategory;
-	int*				m_iPriority;
-	NZBParameterList*	m_pParameters;
-	bool*				m_bAddTop;
-	bool*				m_bAddPaused;
-	char**				m_pDupeKey;
-	int*				m_iDupeScore;
-	EDupeMode*			m_eDupeMode;
-	int					m_iPrefixLen;
-
-	void				PrepareParams(const char* szScriptName);
-
-protected:
-	virtual void		ExecuteScript(Options::Script* pScript);
-	virtual void		AddMessage(Message::EKind eKind, const char* szText);
-
-public:
-	static void			ExecuteScripts(const char* szNZBFilename, const char* szUrl,
-							const char* szDirectory, char** pNZBName, char** pCategory, int* iPriority,
-							NZBParameterList* pParameters, bool* bAddTop, bool* bAddPaused,
-							char** pDupeKey, int* iDupeScore, EDupeMode* eDupeMode);
-};
-
-
-void ScanScriptController::ExecuteScripts(const char* szNZBFilename,
-	const char* szUrl, const char* szDirectory, char** pNZBName, char** pCategory,
-	int* iPriority, NZBParameterList* pParameters, bool* bAddTop, bool* bAddPaused,
-	char** pDupeKey, int* iDupeScore, EDupeMode* eDupeMode)
-{
-	ScanScriptController* pScriptController = new ScanScriptController();
-
-	pScriptController->m_szNZBFilename = szNZBFilename;
-	pScriptController->m_szUrl = szUrl;
-	pScriptController->m_szDirectory = szDirectory;
-	pScriptController->m_pNZBName = pNZBName;
-	pScriptController->m_pCategory = pCategory;
-	pScriptController->m_pParameters = pParameters;
-	pScriptController->m_iPriority = iPriority;
-	pScriptController->m_bAddTop = bAddTop;
-	pScriptController->m_bAddPaused = bAddPaused;
-	pScriptController->m_pDupeKey = pDupeKey;
-	pScriptController->m_iDupeScore = iDupeScore;
-	pScriptController->m_eDupeMode = eDupeMode;
-	pScriptController->m_iPrefixLen = 0;
-
-	pScriptController->ExecuteScriptList(g_pOptions->GetScanScript());
-
-	delete pScriptController;
-}
-
-void ScanScriptController::ExecuteScript(Options::Script* pScript)
-{
-	if (!pScript->GetScanScript() || !Util::FileExists(m_szNZBFilename))
-	{
-		return;
-	}
-
-	PrintMessage(Message::mkInfo, "Executing scan-script %s for %s", pScript->GetName(), Util::BaseFileName(m_szNZBFilename));
-
-	SetScript(pScript->GetLocation());
-	SetArgs(NULL, false);
-
-	char szInfoName[1024];
-	snprintf(szInfoName, 1024, "scan-script %s for %s", pScript->GetName(), Util::BaseFileName(m_szNZBFilename));
-	szInfoName[1024-1] = '\0';
-	SetInfoName(szInfoName);
-
-	SetLogPrefix(pScript->GetDisplayName());
-	m_iPrefixLen = strlen(pScript->GetDisplayName()) + 2; // 2 = strlen(": ");
-	PrepareParams(pScript->GetName());
-
-	Execute();
-
-	SetLogPrefix(NULL);
-}
-
-void ScanScriptController::PrepareParams(const char* szScriptName)
-{
-	ResetEnv();
-
-	SetEnvVar("NZBNP_FILENAME", m_szNZBFilename);
-	SetEnvVar("NZBNP_URL", m_szUrl);
-	SetEnvVar("NZBNP_NZBNAME", strlen(*m_pNZBName) > 0 ? *m_pNZBName : Util::BaseFileName(m_szNZBFilename));
-	SetEnvVar("NZBNP_CATEGORY", *m_pCategory);
-	SetIntEnvVar("NZBNP_PRIORITY", *m_iPriority);
-	SetIntEnvVar("NZBNP_TOP", *m_bAddTop ? 1 : 0);
-	SetIntEnvVar("NZBNP_PAUSED", *m_bAddPaused ? 1 : 0);
-	SetEnvVar("NZBNP_DUPEKEY", *m_pDupeKey);
-	SetIntEnvVar("NZBNP_DUPESCORE", *m_iDupeScore);
-
-	const char* szDupeModeName[] = { "SCORE", "ALL", "FORCE" };
-	SetEnvVar("NZBNP_DUPEMODE", szDupeModeName[*m_eDupeMode]);
-
-	// remove trailing slash
-	char szDir[1024];
-	strncpy(szDir, m_szDirectory, 1024);
-	szDir[1024-1] = '\0';
-	int iLen = strlen(szDir);
-	if (szDir[iLen-1] == PATH_SEPARATOR)
-	{
-		szDir[iLen-1] = '\0';
-	}
-	SetEnvVar("NZBNP_DIRECTORY", szDir);
-
-	PrepareEnvScript(m_pParameters, szScriptName);
-}
-
-void ScanScriptController::AddMessage(Message::EKind eKind, const char* szText)
-{
-	const char* szMsgText = szText + m_iPrefixLen;
-
-	if (!strncmp(szMsgText, "[NZB] ", 6))
-	{
-		debug("Command %s detected", szMsgText + 6);
-		if (!strncmp(szMsgText + 6, "NZBNAME=", 8))
-		{
-			free(*m_pNZBName);
-			*m_pNZBName = strdup(szMsgText + 6 + 8);
-		}
-		else if (!strncmp(szMsgText + 6, "CATEGORY=", 9))
-		{
-			free(*m_pCategory);
-			*m_pCategory = strdup(szMsgText + 6 + 9);
-			g_pScanner->InitPPParameters(*m_pCategory, m_pParameters, true);
-		}
-		else if (!strncmp(szMsgText + 6, "NZBPR_", 6))
-		{
-			char* szParam = strdup(szMsgText + 6 + 6);
-			char* szValue = strchr(szParam, '=');
-			if (szValue)
-			{
-				*szValue = '\0';
-				m_pParameters->SetParameter(szParam, szValue + 1);
-			}
-			else
-			{
-				error("Invalid command \"%s\" received from %s", szMsgText, GetInfoName());
-			}
-			free(szParam);
-		}
-		else if (!strncmp(szMsgText + 6, "PRIORITY=", 9))
-		{
-			*m_iPriority = atoi(szMsgText + 6 + 9);
-		}
-		else if (!strncmp(szMsgText + 6, "TOP=", 4))
-		{
-			*m_bAddTop = atoi(szMsgText + 6 + 4) != 0;
-		}
-		else if (!strncmp(szMsgText + 6, "PAUSED=", 7))
-		{
-			*m_bAddPaused = atoi(szMsgText + 6 + 7) != 0;
-		}
-		else if (!strncmp(szMsgText + 6, "DUPEKEY=", 8))
-		{
-			free(*m_pDupeKey);
-			*m_pDupeKey = strdup(szMsgText + 6 + 8);
-		}
-		else if (!strncmp(szMsgText + 6, "DUPESCORE=", 10))
-		{
-			*m_iDupeScore = atoi(szMsgText + 6 + 10);
-		}
-		else if (!strncmp(szMsgText + 6, "DUPEMODE=", 9))
-		{
-			const char* szDupeMode = szMsgText + 6 + 9;
-			if (strcasecmp(szDupeMode, "score") && strcasecmp(szDupeMode, "all") && strcasecmp(szDupeMode, "force"))
-			{
-				error("Invalid value \"%s\" for command \"DUPEMODE\" received from %s", szDupeMode, GetInfoName());
-				return;
-			}
-			*m_eDupeMode = !strcasecmp(szDupeMode, "all") ? dmAll :
-				!strcasecmp(szDupeMode, "force") ? dmForce : dmScore;
-		}
-		else
-		{
-			error("Invalid command \"%s\" received from %s", szMsgText, GetInfoName());
-		}
-	}
-	else
-	{
-		ScriptController::AddMessage(eKind, szText);
-	}
-}
-
-
 Scanner::FileData::FileData(const char* szFilename)
 {
 	m_szFilename = strdup(szFilename);
@@ -346,8 +152,13 @@ void Scanner::ClearQueueList()
 	m_QueueList.clear();
 }
 
-void Scanner::Check()
+void Scanner::ServiceWork()
 {
+	if (!DownloadQueue::IsLoaded())
+	{
+		return;
+	}
+
 	m_mutexScan.Lock();
 
 	if (m_bRequestedNZBDirScan || 
@@ -545,7 +356,6 @@ void Scanner::ProcessIncomingFile(const char* szDirectory, const char* szBaseFil
 	int iDupeScore = 0;
 	EDupeMode eDupeMode = dmScore;
 	EAddStatus eAddStatus = asSkipped;
-	bool bAdded = false;
 	QueueData* pQueueData = NULL;
 	NZBInfo* pUrlInfo = NULL;
 	int iNZBID = 0;
@@ -601,8 +411,9 @@ void Scanner::ProcessIncomingFile(const char* szDirectory, const char* szBaseFil
 		bool bRenameOK = Util::RenameBak(szFullFilename, "nzb", true, szRenamedName, 1024);
 		if (bRenameOK)
 		{
-			bAdded = AddFileToQueue(szRenamedName, szNZBName, szNZBCategory, iPriority,
+			bool bAdded = AddFileToQueue(szRenamedName, szNZBName, szNZBCategory, iPriority,
 				szDupeKey, iDupeScore, eDupeMode, pParameters, bAddTop, bAddPaused, pUrlInfo, &iNZBID);
+			eAddStatus = bAdded ? asSuccess : asFailed;
 		}
 		else
 		{
@@ -613,8 +424,9 @@ void Scanner::ProcessIncomingFile(const char* szDirectory, const char* szBaseFil
 	}
 	else if (bExists && !strcasecmp(szExtension, ".nzb"))
 	{
-		bAdded = AddFileToQueue(szFullFilename, szNZBName, szNZBCategory, iPriority,
+		bool bAdded = AddFileToQueue(szFullFilename, szNZBName, szNZBCategory, iPriority,
 			szDupeKey, iDupeScore, eDupeMode, pParameters, bAddTop, bAddPaused, pUrlInfo, &iNZBID);
+		eAddStatus = bAdded ? asSuccess : asFailed;
 	}
 
 	delete pParameters;
@@ -625,7 +437,7 @@ void Scanner::ProcessIncomingFile(const char* szDirectory, const char* szBaseFil
 
 	if (pQueueData)
 	{
-		pQueueData->SetAddStatus(eAddStatus == asFailed ? asFailed : bAdded ? asSuccess : asSkipped);
+		pQueueData->SetAddStatus(eAddStatus);
 		pQueueData->SetNZBID(iNZBID);
 	}
 }
@@ -650,9 +462,9 @@ void Scanner::InitPPParameters(const char* szCategory, NZBParameterList* pParame
 
 	if (bReset)
 	{
-		for (Options::Scripts::iterator it = g_pOptions->GetScripts()->begin(); it != g_pOptions->GetScripts()->end(); it++)
+		for (ScriptConfig::Scripts::iterator it = g_pScriptConfig->GetScripts()->begin(); it != g_pScriptConfig->GetScripts()->end(); it++)
 		{
-			Options::Script* pScript = *it;
+			ScriptConfig::Script* pScript = *it;
 			char szParam[1024];
 			snprintf(szParam, 1024, "%s:", pScript->GetName());
 			szParam[1024-1] = '\0';
@@ -684,8 +496,8 @@ bool Scanner::AddFileToQueue(const char* szFilename, const char* szNZBName, cons
 
 	info("Adding collection %s to queue", szBasename);
 
-	NZBFile* pNZBFile = NZBFile::Create(szFilename, szCategory);
-	bool bOK = pNZBFile != NULL;
+	NZBFile* pNZBFile = new NZBFile(szFilename, szCategory);
+	bool bOK = pNZBFile->Parse();
 	if (!bOK)
 	{
 		error("Could not add collection %s to queue", szBasename);
@@ -699,54 +511,60 @@ bool Scanner::AddFileToQueue(const char* szFilename, const char* szNZBName, cons
 		error("Could not rename file %s to %s: %s", szFilename, bakname2, Util::GetLastErrorMessage(szSysErrStr, sizeof(szSysErrStr)));
 	}
 
+	NZBInfo* pNZBInfo = pNZBFile->GetNZBInfo();
+	pNZBInfo->SetQueuedFilename(bakname2);
+
+	if (szNZBName && strlen(szNZBName) > 0)
+	{
+		pNZBInfo->SetName(NULL);
+#ifdef WIN32
+		char* szAnsiFilename = strdup(szNZBName);
+		WebUtil::Utf8ToAnsi(szAnsiFilename, strlen(szAnsiFilename) + 1);
+		pNZBInfo->SetFilename(szAnsiFilename);
+		free(szAnsiFilename);
+#else
+		pNZBInfo->SetFilename(szNZBName);
+#endif
+		pNZBInfo->BuildDestDirName();
+	}
+
+	pNZBInfo->SetDupeKey(szDupeKey);
+	pNZBInfo->SetDupeScore(iDupeScore);
+	pNZBInfo->SetDupeMode(eDupeMode);
+	pNZBInfo->SetPriority(iPriority);
+	if (pUrlInfo)
+	{
+		pNZBInfo->SetURL(pUrlInfo->GetURL());
+		pNZBInfo->SetUrlStatus(pUrlInfo->GetUrlStatus());
+		pNZBInfo->SetFeedID(pUrlInfo->GetFeedID());
+	}
+
+	if (pNZBFile->GetPassword())
+	{
+		pNZBInfo->GetParameters()->SetParameter("*Unpack:Password", pNZBFile->GetPassword());
+	}
+
+	pNZBInfo->GetParameters()->CopyFrom(pParameters);
+
+	for (::FileList::iterator it = pNZBInfo->GetFileList()->begin(); it != pNZBInfo->GetFileList()->end(); it++)
+	{
+		FileInfo* pFileInfo = *it;
+		pFileInfo->SetPaused(bAddPaused);
+	}
+
 	if (bOK)
 	{
-		NZBInfo* pNZBInfo = pNZBFile->GetNZBInfo();
-		pNZBInfo->SetQueuedFilename(bakname2);
-
-		if (szNZBName && strlen(szNZBName) > 0)
-		{
-			pNZBInfo->SetName(NULL);
-#ifdef WIN32
-			char* szAnsiFilename = strdup(szNZBName);
-			WebUtil::Utf8ToAnsi(szAnsiFilename, strlen(szAnsiFilename) + 1);
-			pNZBInfo->SetFilename(szAnsiFilename);
-			free(szAnsiFilename);
-#else
-			pNZBInfo->SetFilename(szNZBName);
-#endif
-			pNZBInfo->BuildDestDirName();
-		}
-
-		pNZBInfo->SetDupeKey(szDupeKey);
-		pNZBInfo->SetDupeScore(iDupeScore);
-		pNZBInfo->SetDupeMode(eDupeMode);
-		pNZBInfo->SetPriority(iPriority);
-		if (pUrlInfo)
-		{
-			pNZBInfo->SetURL(pUrlInfo->GetURL());
-			pNZBInfo->SetUrlStatus(pUrlInfo->GetUrlStatus());
-		}
-
-		if (pNZBFile->GetPassword())
-		{
-			pNZBInfo->GetParameters()->SetParameter("*Unpack:Password", pNZBFile->GetPassword());
-		}
-
-		pNZBInfo->GetParameters()->CopyFrom(pParameters);
-
-		for (::FileList::iterator it = pNZBInfo->GetFileList()->begin(); it != pNZBInfo->GetFileList()->end(); it++)
-		{
-			FileInfo* pFileInfo = *it;
-			pFileInfo->SetPaused(bAddPaused);
-		}
-
 		g_pQueueCoordinator->AddNZBFileToQueue(pNZBFile, pUrlInfo, bAddTop);
+	}
+	else if (!pUrlInfo)
+	{
+		pNZBFile->GetNZBInfo()->SetDeleteStatus(NZBInfo::dsScan);
+		g_pQueueCoordinator->AddNZBFileToQueue(pNZBFile, pUrlInfo, bAddTop);
+	}
 
-		if (pNZBID)
-		{
-			*pNZBID = pNZBInfo->GetID();
-		}
+	if (pNZBID)
+	{
+		*pNZBID = pNZBInfo->GetID();
 	}
 
 	delete pNZBFile;
