@@ -18,8 +18,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 1221 $
- * $Date: 2015-02-26 15:57:38 -0500 (Thu, 26 Feb 2015) $
+ * $Revision$
+ * $Date$
  *
  */
 
@@ -52,9 +52,6 @@
 #include "Scanner.h"
 #include "StatMeter.h"
 
-extern Options* g_pOptions;
-extern Scanner* g_pScanner;
-extern StatMeter* g_pStatMeter;
 extern void ExitProc();
 extern void Reload();
 
