@@ -18,8 +18,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 1245 $
- * $Date: 2015-03-26 18:28:30 -0400 (Thu, 26 Mar 2015) $
+ * $Revision$
+ * $Date$
  *
  */
 
@@ -27,6 +27,10 @@
 #ifndef MESSAGEBASE_H
 #define MESSAGEBASE_H
 
+#if (!(defined(WIN32) && _MSC_VER < 1600))
+#include <stdint.h>
+#endif
+
 static const int32_t NZBMESSAGE_SIGNATURE = 0x6E7A6228; // = "nzb-XX" (protocol version)
 static const int NZBREQUESTFILENAMESIZE = 512;
 static const int NZBREQUESTPASSWORDSIZE = 32;
