@@ -2,7 +2,7 @@
  *  This file is part of nzbget
  *
  *  Copyright (C) 2005 Bo Cordes Petersen <placebodk@sourceforge.net>
- *  Copyright (C) 2007-2013 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2007-2015 Andrey Prygunkov <hugbug@users.sourceforge.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,8 +18,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 1149 $
- * $Date: 2014-10-21 15:32:07 -0400 (Tue, 21 Oct 2014) $
+ * $Revision$
+ * $Date$
  *
  */
 
@@ -50,8 +50,6 @@
 #include "Options.h"
 #include "Util.h"
 
-extern Options* g_pOptions;
-
 //*****************************************************************
 // RemoteServer
 
@@ -225,6 +223,10 @@ void RequestProcessor::Run()
 			processor.SetUrl(szUrl);
 			processor.SetHttpMethod(eHttpMethod);
 			processor.Execute();
+
+			m_pConnection->SetGracefull(true);
+			m_pConnection->Disconnect();
+
 			bOK = true;
 		}
 	}
