@@ -17,8 +17,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 1217 $
- * $Date: 2015-02-20 16:05:51 -0500 (Fri, 20 Feb 2015) $
+ * $Revision$
+ * $Date$
  *
  */
 
@@ -53,6 +53,10 @@ private:
 	EUserAccess			m_eUserAccess;
 	bool				m_bGZip;
 	char*				m_szOrigin;
+	int					m_iContentLen;
+	char				m_szAuthInfo[256+1];
+	char				m_szAuthToken[48+1];
+	static char			m_szServerAuthToken[3][48+1];
 
 	void				Dispatch();
 	void				SendAuthResponse();
@@ -63,10 +67,14 @@ private:
 	void				SendRedirectResponse(const char* szURL);
 	const char*			DetectContentType(const char* szFilename);
 	bool				IsAuthorizedIP(const char* szRemoteAddr);
+	void				ParseHeaders();
+	void				ParseURL();
+	bool				CheckCredentials();
 
 public:
 						WebProcessor();
 						~WebProcessor();
+	static void			Init();
 	void				Execute();
 	void				SetConnection(Connection* pConnection) { m_pConnection = pConnection; }
 	void				SetUrl(const char* szUrl);
