@@ -17,8 +17,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 1234 $
- * $Date: 2015-03-17 17:58:49 -0400 (Tue, 17 Mar 2015) $
+ * $Revision$
+ * $Date$
  *
  */
 
@@ -98,6 +98,9 @@ protected:
 	void				BuildBoolResponse(bool bOK);
 	void				BuildIntResponse(int iValue);
 	void				AppendResponse(const char* szPart);
+	void				AppendFmtResponse(const char* szFormat, ...);
+	void				AppendCondResponse(const char* szPart, bool bCond);
+	void				OptimizeResponse(int iRecordCount);
 	bool				IsJson();
 	bool				CheckSafeMethod();
 	bool				NextParamAsInt(int* iValue);
