@@ -18,8 +18,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 1221 $
- * $Date: 2015-02-26 15:57:38 -0500 (Thu, 26 Feb 2015) $
+ * $Revision$
+ * $Date$
  *
  */
 
@@ -46,12 +46,24 @@
 #include "Log.h"
 #include "Util.h"
 
-extern Options* g_pOptions;
+Log* g_pLog = NULL;
+
+void Log::Init()
+{
+	g_pLog = new Log();
+}
+
+void Log::Final()
+{
+	delete g_pLog;
+	g_pLog = NULL;
+}
 
 Log::Log()
 {
 	m_Messages.clear();
 	m_iIDGen = 0;
+	m_bOptInit = false;
 	m_szLogFilename = NULL;
 	m_tLastWritten = 0;
 #ifdef DEBUG
@@ -290,27 +302,6 @@ void detail(const char* msg, ...)
 	g_pLog->m_mutexLog.Unlock();
 }
 
-void abort(const char* msg, ...)
-{
-	char tmp2[1024];
-
-	va_list ap;
-	va_start(ap, msg);
-	vsnprintf(tmp2, 1024, msg, ap);
-	tmp2[1024-1] = '\0';
-	va_end(ap);
-
-	g_pLog->m_mutexLog.Lock();
-
-	printf("\n%s", tmp2);
-
-	g_pLog->Filelog(tmp2);
-
-	g_pLog->m_mutexLog.Unlock();
-
-	exit(-1);
-}
-
 //************************************************************
 // Message
 
@@ -360,7 +351,7 @@ void Log::AddMessage(Message::EKind eKind, const char * szText)
 	Message* pMessage = new Message(++m_iIDGen, eKind, time(NULL), szText);
 	m_Messages.push_back(pMessage);
 
-	if (g_pOptions)
+	if (m_bOptInit && g_pOptions)
 	{
 		while (m_Messages.size() > (unsigned int)g_pOptions->GetLogBufferSize())
 		{
@@ -530,6 +521,8 @@ void Log::InitOptions()
 			i++;
 		}
 	}
+
+	m_bOptInit = true;
 }
 
 void Log::RegisterDebuggable(Debuggable* pDebuggable)
