@@ -17,8 +17,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 1226 $
- * $Date: 2015-03-02 15:49:05 -0500 (Mon, 02 Mar 2015) $
+ * $Revision$
+ * $Date$
  *
  */
 
@@ -56,8 +56,6 @@
 
 // System global variable holding environments variables
 extern char** environ;
-
-extern Options* g_pOptions;
 extern char* (*g_szEnvironmentVariables)[];
 
 ScriptController::RunningScripts ScriptController::m_RunningScripts;
@@ -601,7 +599,7 @@ int ScriptController::Execute()
 		fclose(m_pReadpipe);
 	}
 
-	if (m_bTerminated)
+	if (m_bTerminated && m_szInfoName)
 	{
 		warn("Interrupted %s", m_szInfoName);
 	}
