@@ -18,8 +18,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 1189 $
- * $Date: 2015-01-22 15:57:39 -0500 (Thu, 22 Jan 2015) $
+ * $Revision$
+ * $Date$
  *
  */
 
@@ -39,24 +39,6 @@ public:
 	void					Unlock();
 };
 
-#ifdef HAVE_SPINLOCK
-class SpinLock
-{
-private:
-#ifdef WIN32
-	void*					m_pSpinLockObj;
-#else
-	volatile void*			m_pSpinLockObj;
-#endif
-	
-public:
-							SpinLock();
-							~SpinLock();
-	void					Lock();
-	void					Unlock();
-};
-#endif
-
 class Thread
 {
 private:
