@@ -17,8 +17,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 1242 $
- * $Date: 2015-03-24 13:18:22 -0400 (Tue, 24 Mar 2015) $
+ * $Revision$
+ * $Date$
  *
  */
 
@@ -70,11 +70,6 @@ extern StatMeter* g_pStatMeter;
 
 bool bMayStartBrowser = true;
 
-void SetupFirstStart()
-{
-	g_pWinConsole->SetupFirstStart();
-}
-
 BOOL WINAPI WinConsole::ConsoleCtrlHandler(DWORD dwCtrlType)
 {
 	switch (dwCtrlType)
@@ -108,6 +103,7 @@ WinConsole::WinConsole()
 	m_bConsole = false;
 	m_bWebUI = true;
 	m_bAutoParam = false;
+	m_bDoubleClick = false;
 	m_hTrayWindow = 0;
 	m_bRunning = false;
 	m_bRunningService = false;
@@ -322,7 +318,7 @@ LRESULT WinConsole::TrayWndProc(HWND hwndWin, UINT uMsg, WPARAM wParam, LPARAM l
 	switch (uMsg)
 	{
 		case UM_TRAYICON:
-			if (lParam == WM_LBUTTONUP)
+			if (lParam == WM_LBUTTONUP && !m_bDoubleClick)
 			{
 				g_pOptions->SetPauseDownload(!g_pOptions->GetPauseDownload());
 				g_pOptions->SetPausePostProcess(g_pOptions->GetPauseDownload());
@@ -330,6 +326,10 @@ LRESULT WinConsole::TrayWndProc(HWND hwndWin, UINT uMsg, WPARAM wParam, LPARAM l
 				g_pOptions->SetResumeTime(0);
 				UpdateTrayIcon();
 			}
+			else if (lParam == WM_LBUTTONDBLCLK && m_bDoubleClick)
+			{
+				ShowWebUI();
+			}
 			else if (lParam == WM_RBUTTONDOWN)
 			{
 				ShowMenu();
@@ -583,6 +583,8 @@ BOOL WinConsole::PrefsDialogProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM
 			SendDlgItemMessage(hwndDlg, IDC_PREF_TRAY, BM_SETCHECK, m_bTray, 0);
 			SendDlgItemMessage(hwndDlg, IDC_PREF_CONSOLE, BM_SETCHECK, m_bConsole, 0);
 			SendDlgItemMessage(hwndDlg, IDC_PREF_WEBUI, BM_SETCHECK, m_bWebUI, 0);
+			SendDlgItemMessage(hwndDlg, IDC_PREF_TRAYPAUSE, BM_SETCHECK, !m_bDoubleClick, 0);
+			SendDlgItemMessage(hwndDlg, IDC_PREF_TRAYWEBUI, BM_SETCHECK, m_bDoubleClick, 0);
 
 			return FALSE;
 
@@ -597,6 +599,7 @@ BOOL WinConsole::PrefsDialogProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM
 				m_bTray = SendDlgItemMessage(hwndDlg, IDC_PREF_TRAY, BM_GETCHECK, 0, 0) == BST_CHECKED;
 				m_bConsole = SendDlgItemMessage(hwndDlg, IDC_PREF_CONSOLE, BM_GETCHECK, 0, 0) == BST_CHECKED;
 				m_bWebUI = SendDlgItemMessage(hwndDlg, IDC_PREF_WEBUI, BM_GETCHECK, 0, 0) == BST_CHECKED;
+				m_bDoubleClick = SendDlgItemMessage(hwndDlg, IDC_PREF_TRAYWEBUI, BM_GETCHECK, 0, 0) == BST_CHECKED;
 
 				SavePrefs();
 				if (!m_bRunning)
@@ -632,6 +635,9 @@ void WinConsole::SavePrefs()
 	val = m_bWebUI;
 	RegSetValueEx(hKey, "ShowWebUI", 0, REG_DWORD, (BYTE*)&val, sizeof(val));
 
+	val = m_bDoubleClick;
+	RegSetValueEx(hKey, "TrayDoubleClick", 0, REG_DWORD, (BYTE*)&val, sizeof(val));
+
 	RegCloseKey(hKey);
 
 	// Autostart-setting
@@ -676,6 +682,11 @@ void WinConsole::LoadPrefs()
 			m_bWebUI = val;
 		}
 
+		if (RegQueryValueEx(hKey, "TrayDoubleClick", 0, &typ, (LPBYTE)&val, &cval) == ERROR_SUCCESS)
+		{
+			m_bDoubleClick = val;
+		}
+
 		RegCloseKey(hKey);
 	}
 
@@ -798,7 +809,7 @@ void WinConsole::UpdateTrayIcon()
 	{
 		m_pNidIcon->hIcon = m_hWorkingIcon;
 		char szSpeed[100];
-		Util::FormatSpeed(g_pStatMeter->CalcCurrentDownloadSpeed(), szSpeed, sizeof(szSpeed));
+		Util::FormatSpeed(szSpeed, sizeof(szSpeed), g_pStatMeter->CalcCurrentDownloadSpeed());
 		char szTip[200];
 		snprintf(szTip, sizeof(szTip), "NZBGet - downloading at %s", szSpeed);
 		szTip[200-1] = '\0';
