@@ -1,7 +1,7 @@
 /*
  * This file is part of nzbget
  *
- * Copyright (C) 2012-2013 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ * Copyright (C) 2012-2015 Andrey Prygunkov <hugbug@users.sourceforge.net>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,8 +17,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 778 $
- * $Date: 2013-08-07 16:09:43 -0400 (Wed, 07 Aug 2013) $
+ * $Revision$
+ * $Date$
  *
  */
 
@@ -75,6 +75,11 @@
 
 	var methods =
 	{
+		defaults : function()
+		{
+			return defaults;
+		},
+		
 		init : function(options)
 		{
 			return this.each(function()
@@ -97,7 +102,9 @@
 					config.pagerContainer = $(config.pagerContainer);
 					config.infoContainer = $(config.infoContainer);
 					config.headerCheck = $(config.headerCheck);
-
+					
+					var searcher = new FastSearcher();
+					
 					// Create a timer which gets reset upon every keyup event.
 					// Perform filter only when the timer's wait is reached (user finished typing or paused long enough to elapse the timer).
 					// Do not perform the filter is the query has not changed.
@@ -120,21 +127,11 @@
 
 						var timerCallback = function()
 						{
-							var value = inputBox.value;
+							var value = inputBox.value.trim();
 							var data = $this.data('fasttable');
-
-							if ((value != data.lastFilter) || (overrideBool))
+							if ((value != data.lastFilter) || overrideBool)
 							{
-								data.lastFilter = value;
-								if (data.content)
-								{
-									data.curPage = 1;
-									refresh(data);
-								}
-								if (data.config.filterInputCallback)
-								{
-									data.config.filterInputCallback(value);
-								}								
+								applyFilter(data, value);
 							}
 						};
 
@@ -148,16 +145,8 @@
 					config.filterClearButton.click(function()
 					{
 						var data = $this.data('fasttable');
-						data.lastFilter = '';
 						data.config.filterInput.val('');
-						if (data.content)
-						{
-							refresh(data);
-						}
-						if (data.config.filterClearCallback)
-						{
-							data.config.filterClearCallback();
-						}								
+						applyFilter(data, '');
 					});
 						
 					config.pagerContainer.on('click', 'li', function (e)
@@ -191,8 +180,10 @@
 							maxPages : parseInt(config.maxPages),
 							pageDots : Util.parseBool(config.pageDots),
 							curPage : 1,
-							checkedRows: [],
-							lastClickedRowID: null
+							checkedRows: {},
+							checkedCount: 0,
+							lastClickedRowID: null,
+							searcher: searcher
 						});
 				}
 			});
@@ -217,6 +208,11 @@
 
 		setCurPage : setCurPage,
 		
+		applyFilter : function(filter)
+		{
+			applyFilter($(this).data('fasttable'), filter);
+		},
+
 		filteredContent : function()
 		{
 			return $(this).data('fasttable').filteredContent;
@@ -232,6 +228,11 @@
 			return $(this).data('fasttable').checkedRows;
 		},
 		
+		checkedCount : function()
+		{
+			return $(this).data('fasttable').checkedCount;
+		},
+
 		checkRow : function(id, checked)
 		{
 			checkRow($(this).data('fasttable'), id, checked);
@@ -242,21 +243,6 @@
 		titleCheckClick : titleCheckClick
 	};
 
-	function has_words(str, words, caseSensitive)
-	{
-		var text = caseSensitive ? str : str.toLowerCase();
-
-		for (var i = 0; i < words.length; i++)
-		{
-			if (text.indexOf(words[i]) === -1)
-			{
-				return false;
-			}
-		}
-
-		return true;
-	}
-
 	function updateContent(content)
 	{
 		var data = $(this).data('fasttable');
@@ -267,6 +253,26 @@
 		refresh(data);
 	}
 
+	function applyFilter(data, filter)
+	{
+		data.lastFilter = filter;
+		if (data.content)
+		{
+			data.curPage = 1;
+			data.hasFilter = filter !== '';
+			data.searcher.compile(filter);
+			refresh(data);
+		}
+		if (filter !== '' && data.config.filterInputCallback)
+		{
+			data.config.filterInputCallback(filter);
+		}								
+		if (filter === '' && data.config.filterClearCallback)
+		{
+			data.config.filterClearCallback();
+		}								
+	}
+
 	function refresh(data)
 	{
 		refilter(data);
@@ -278,23 +284,17 @@
 
 	function refilter(data)
 	{
-		var filterInput = data.config.filterInput;
-		var phrase = filterInput.length > 0 ? filterInput.val() : '';
-		var caseSensitive = data.config.filterCaseSensitive;
-		var words = caseSensitive ? phrase.split(' ') : phrase.toLowerCase().split(' ');
-		var hasFilter = !(words.length === 1 && words[0] === '');
-
 		data.availableContent = [];
 		data.filteredContent = [];
 		for (var i = 0; i < data.content.length; i++)
 		{
 			var item = data.content[i];
-			if (hasFilter && item.search === undefined && data.config.fillSearchCallback)
+			if (data.hasFilter && item.search === undefined && data.config.fillSearchCallback)
 			{
 				data.config.fillSearchCallback(item);
 			}
-			
-			if (!hasFilter || has_words(item.search, words, caseSensitive))
+
+			if (!data.hasFilter || data.searcher.exec(item.data))
 			{
 				data.availableContent.push(item);
 				if (!data.config.filterCallback || data.config.filterCallback(item))
@@ -322,7 +322,7 @@
 			var row = table.insertRow(table.rows.length);
 
 			row.fasttableID = item.id;
-			if (data.checkedRows.indexOf(item.id) > -1)
+			if (data.checkedRows[item.id])
 			{
 				row.className = 'checked';
 			}
@@ -601,13 +601,13 @@
 		var hasUnselectedItems = false;
 		for (var i = 0; i < filteredContent.length; i++)
 		{
-			if (checkedRows.indexOf(filteredContent[i].id) === -1)
+			if (checkedRows[filteredContent[i].id])
 			{
-				hasUnselectedItems = true;
+				hasSelectedItems = true;
 			}
 			else
 			{
-				hasSelectedItems = true;
+				hasUnselectedItems = true;
 			}
 		}
 		
@@ -635,7 +635,7 @@
 
 		if (event.shiftKey && data.lastClickedRowID != null)
 		{
-			var checked = checkedRows.indexOf(id) > -1;
+			var checked = checkedRows[id];
 			doToggle = !checkRange(data, id, data.lastClickedRowID, !checked);
 		}
 
@@ -658,7 +658,7 @@
 		var hasSelectedItems = false;
 		for (var i = 0; i < filteredContent.length; i++)
 		{
-			if (checkedRows.indexOf(filteredContent[i].id) > -1)
+			if (checkedRows[filteredContent[i].id])
 			{
 				hasSelectedItems = true;
 				break;
@@ -672,14 +672,16 @@
 	function toggleCheck(data, id)
 	{
 		var checkedRows = data.checkedRows;
-		var index = checkedRows.indexOf(id);
-		if (index > -1)
+		var index = checkedRows[id];
+		if (checkedRows[id])
 		{
-			checkedRows.splice(index, 1);
+			checkedRows[id] = undefined;
+			data.checkedCount--;
 		}
 		else
 		{
-			checkedRows.push(id);
+			checkedRows[id] = true;
+			data.checkedCount++;
 		}
 	}
 	
@@ -722,18 +724,19 @@
 	{
 		if (checked)
 		{
-			if (data.checkedRows.indexOf(id) === -1)
+			if (!data.checkedRows[id])
 			{
-				data.checkedRows.push(id);
+				data.checkedCount++;
 			}
+			data.checkedRows[id] = true;
 		}
 		else
 		{
-			var index = data.checkedRows.indexOf(id);
-			if (index > -1)
+			if (data.checkedRows[id])
 			{
-				data.checkedRows.splice(index, 1);
+				data.checkedCount--;
 			}
+			data.checkedRows[id] = undefined;
 		}
 	}
 	
@@ -753,19 +756,14 @@
 	{
 		var filteredContent = data.filteredContent;
 		var checkedRows = data.checkedRows;
-
-		var ids = [];
+		data.checkedRows = {}
+		data.checkedCount = 0;
 		for (var i = 0; i < data.content.length; i++)
 		{
-			ids.push(data.content[i].id);
-		}
-
-		for (var i = 0; i < checkedRows.length; i++)
-		{
-			if (ids.indexOf(checkedRows[i]) === -1)
+			if (checkedRows[data.content[i].id])
 			{
-				checkedRows.splice(i, 1);
-				i--;
+				data.checkedRows[data.content[i].id] = true;
+				data.checkedCount++;
 			}
 		}
 	}
@@ -774,7 +772,6 @@
 	{
 		filterInput: '#table-filter',
 		filterClearButton: '#table-clear',
-		filterCaseSensitive: false,
 		pagerContainer: '#table-pager',
 		infoContainer: '#table-info',
 		pageSize: 10,
@@ -794,4 +791,269 @@
 		headerCheck: '#table-header-check'
 	};
 
-})(jQuery);
\ No newline at end of file
+})(jQuery);
+
+function FastSearcher()
+{
+	'use strict';
+
+	this.source;
+	this.len;
+	this.p;
+
+	this.initLexer = function(source)
+	{
+		this.source = source;
+		this.len = source.length;
+		this.p = 0;
+	}
+	
+	this.nextToken = function()
+	{
+		while (this.p < this.len)
+		{
+			var ch = this.source[this.p++];
+			switch (ch) {
+				case ' ':
+				case '\t':
+					continue;
+
+				case '-':
+				case '(':
+				case ')':
+				case '|':
+					return ch;
+
+				default:
+					this.p--;
+					var token = '';
+					var quote = false;
+					while (this.p < this.len)
+					{
+						var ch = this.source[this.p++];
+						if (quote)
+						{
+							if (ch === '"')
+							{
+								quote = false;
+								ch = '';
+							}
+						}
+						else
+						{
+							if (ch === '"')
+							{
+								quote = true;
+								ch = '';
+							}
+							else if (' \t()|'.indexOf(ch) > -1)
+							{
+								this.p--;
+								return token;
+							}
+						}
+						token += ch;
+					}
+					return token;
+			}
+		}
+		return null;
+	}
+
+	this.compile = function(searchstr)
+	{
+		var _this = this;
+		this.initLexer(searchstr);
+
+		function expression(greedy)
+		{
+			var node = null;
+			while (true)
+			{
+				var token = _this.nextToken();
+				var node2 = null;
+				switch (token)
+				{
+					case null:
+					case ')':
+						return node;
+
+					case '-':
+						node2 = expression(false);
+						node2 = node2 ? _this.not(node2) : node2;
+						break;
+
+					case '(':
+						node2 = expression(true);
+						break;
+
+					case '|':
+						node2 = expression(false);
+						break;
+
+					default:
+						node2 = _this.term(token);
+				}
+
+				if (node && node2)
+				{
+					node = token === '|' ? _this.or(node, node2) : _this.and(node, node2);
+				}
+				else if (node2)
+				{
+					node = node2;
+				}
+
+				if (!greedy && node)
+				{
+					return node;
+				}
+			}
+		}
+
+		this.root = expression(true);
+	}
+
+	this.root = null;
+	this.data = null;
+
+	this.exec = function(data) {
+		this.data = data;
+		return this.root ? this.root.eval() : true;
+	}
+
+	this.and = function(L, R) {
+		return {
+			L: L, R: R,
+			eval: function() { return this.L.eval() && this.R.eval(); }
+		};
+	}
+
+	this.or = function(L, R) {
+		return {
+			L: L, R: R,
+			eval: function() { return this.L.eval() || this.R.eval(); }
+		};
+	}
+
+	this.not = function(M) {
+		return {
+			M: M,
+			eval: function() { return !this.M.eval();}
+		};
+	}
+
+	this.term = function(term) {
+		return this.compileTerm(term);
+	}
+
+	var COMMANDS = [ ':', '>=', '<=', '<>', '>', '<', '=' ];
+
+	this.compileTerm = function(term) {
+		var _this = this;
+		var text = term.toLowerCase();
+		var field;
+		
+		var command;
+		var commandIndex;
+		for (var i = 0; i < COMMANDS.length; i++)
+		{
+			var cmd = COMMANDS[i];
+			var p = term.indexOf(cmd);
+			if (p > -1 && (p < commandIndex || commandIndex === undefined))
+			{
+				commandIndex = p;
+				command = cmd;
+			}
+		}
+		
+		if (command !== undefined)
+		{
+			field = term.substring(0, commandIndex);
+			text = text.substring(commandIndex + command.length);
+		}
+
+		return {
+			command: command,
+			text: text,
+			field: field,
+			eval: function() { return _this.evalTerm(this); }
+		};
+	}
+	
+	this.evalTerm = function(term) {
+		var text = term.text;
+		var field = term.field;
+		var content = this.fieldValue(this.data, field);
+
+		if (content === undefined)
+		{
+			return false;
+		}
+
+		switch (term.command)
+		{
+			case undefined:
+			case ':':
+				return content.toString().toLowerCase().indexOf(text) > -1;
+			case '=':
+				return content.toString().toLowerCase() == text;
+			case '<>':
+				return content.toString().toLowerCase() != text;
+			case '>':
+				return parseInt(content) > parseInt(text);
+			case '>=':
+				return parseInt(content) >= parseInt(text);
+			case '<':
+				return parseInt(content) < parseInt(text);
+			case '<=':
+				return parseInt(content) <= parseInt(text);
+			default:
+				return false;
+		}
+	}
+
+	this.fieldValue = function(data, field) {
+		var value = '';
+		if (field !== undefined)
+		{
+			value = data[field];
+			if (value === undefined)
+			{
+				if (this.nameMap === undefined)
+				{
+					this.buildNameMap(data);
+				}
+				value = data[this.nameMap[field.toLowerCase()]];
+			}
+		}
+		else
+		{
+			if (data._search === true)
+			{
+				for (var prop in data)
+				{
+					value += ' ' + data[prop];
+				}
+			}
+			else
+			{
+				for (var i = 0; i < data._search.length; i++)
+				{
+					value += ' ' + data[data._search[i]];
+				}
+			}
+		}
+		return value;
+	}
+
+	this.nameMap;
+	this.buildNameMap = function(data)
+	{
+		this.nameMap = {};
+		for (var prop in data)
+		{
+			this.nameMap[prop.toLowerCase()] = prop;
+		}
+	}
+}
